USE [igops]

GO

DROP VIEW IF EXISTS approval_request_policy_v
GO

DROP VIEW IF EXISTS request_item_psod_v
GO

DROP VIEW IF EXISTS request_lifecycle_v
GO

DROP VIEW IF EXISTS resource_request_v
GO

CREATE VIEW approval_request_policy_v AS
	SELECT
		aas.policy_id,
	    arap.unique_policy_id,
	    arap.name,
	    aas.step_sequence,
	    aas.approver_type,
	    aas.escalation_period,
	    aas.escalator_type,
	    aas.timeout_period,
	    aas.timeout_action,
	    aas.id
	   	FROM access_approval_step aas
     	JOIN access_req_approval_policy arap ON aas.policy_id = arap.id
GO

CREATE VIEW request_item_psod_v AS
	SELECT 
		rri.id,
	    rr.request_status,
	    rri.item_type AS req_item_type,
	    rri.unique_item_id AS req_item_unique_id,
	    perm.name AS permission_name,
	    role.name AS role_name,
	    ds.name AS app_name,
	    requser.display_name AS request_user_name,
	    requser.unique_user_id AS request_user_id,
	    rri.last_status_update_time AS appr_deny_time,
	    recipuser.display_name AS recipient_user_name,
	    recipuser.unique_user_id AS recipient_user_id,
	    sod.name AS psod_policy_name,
	    sod.unique_policy_id AS psod_policy_id,
	    sod.id AS psod_id,
	    sod.deleted AS policy_status,
	    sod.creation_time,
	    sod.description AS policy_description,
	    sod.detection_status,
	    sod.state,
	    sod.update_time,
	    change.display_name AS changed_by,
	    creater.display_name AS created_by,
	    pati.decision AS psod_appr_status,
	    pati.decision_date AS psod_appr_status_time,
	    pati.decided_by AS psod_appr_id,
	    psodappuser.display_name AS psod_appr_name,
	    pol_owners.owners
	   	FROM resource_request_item rri
		JOIN resource_request rr ON rr.id = rri.resource_request_id
		LEFT JOIN resource_request_item_step rris ON rris.resource_request_item_id = rri.id
		LEFT JOIN suser requser ON requser.unique_user_id = rr.unique_requester_id AND requser.effective = 1
		LEFT JOIN suser recipuser ON recipuser.unique_user_id = rri.unique_recipient_id AND recipuser.effective = 1
		LEFT JOIN spermission perm ON perm.unique_permission_id = rri.unique_item_id AND perm.effective = 1 AND rri.item_type = 'PERMISSION'
		LEFT JOIN role_policy role ON role.unique_policy_id = rri.unique_item_id AND role.deleted = 0 AND rri.item_type = 'TECHNICAL_ROLE'
		LEFT JOIN application app ON CAST(app.id AS varchar(255)) = rri.unique_item_id AND rri.item_type = 'APPLICATION'
		LEFT JOIN data_source ds ON ds.id = app.id AND ds.deleted = 0
		LEFT JOIN change_request_item cri ON cri.id = rri.change_request_item_id
		LEFT JOIN change_item_action cia ON cia.change_request_item = cri.id
		LEFT JOIN suser fufusr ON cri.fulfiller = fufusr.unique_user_id AND fufusr.effective = 1
		LEFT JOIN resource_request_approval_step rras ON rras.id = rris.approval_step_id
		LEFT JOIN suser stepuser ON stepuser.unique_user_id = rras.finished_by AND stepuser.effective = 1
		LEFT JOIN suser stepuser2 ON stepuser2.unique_user_id = rris.status_set_by AND stepuser2.effective = 1
		LEFT JOIN suser apprvuser ON apprvuser.unique_user_id = rri.status_changed_by AND apprvuser.effective = 1 AND rras.id IS NOT NULL
		LEFT JOIN psodv_approval_task_item pati ON pati.resource_request_item_id = rri.id
		LEFT JOIN psodv_approval_task pat ON pat.id = pati.psodv_approval_task_id
		LEFT JOIN suser psodsubuser ON psodsubuser.unique_user_id = pat.completed_by AND psodsubuser.effective = 1
		LEFT JOIN suser psodappuser ON psodappuser.unique_user_id = pati.decided_by AND psodappuser.effective = 1
		LEFT JOIN sod_policy sod ON sod.id = pat.policy_id
		LEFT JOIN suser change ON change.unique_user_id = sod.changed_by AND change.deleted = 0 AND change.effective = 1
		LEFT JOIN suser creater ON creater.unique_user_id = sod.created_by AND creater.deleted = 0 AND creater.effective = 1
		LEFT JOIN (SELECT pol_owners.sod_policy,
          	STUFF((SELECT ' | ' + pov_owners.owner_name
					FROM (SELECT DISTINCT pov.owner_name
						FROM policy_owners_v pov
						WHERE pov.sod_policy = pol_owners.sod_policy) AS pov_owners
						ORDER BY pov_owners.owner_name FOR XML PATH('')), 1, 3, '') owners
				FROM policy_owners_v pol_owners
				GROUP BY pol_owners.sod_policy) pol_owners ON pol_owners.sod_policy = sod.id
GO

CREATE VIEW request_lifecycle_v AS
	SELECT
		perm.name AS permissionname,
	    perm.unique_permission_id,
	    stepuser.display_name AS stepapprovername,
	    stepuser.unique_user_id AS stepapproveruserid,
	    rras.finish_time AS stepapprovaltime,
	    stepuser2.display_name AS statussettername,
	    stepuser2.unique_user_id AS statussetteruserid,
	    rris.status_time AS statussettime,
	    rr.id AS cartid,
	    rri.id,
	    rras.step AS stepapprovalorder,
	    rr.request_status,
	    rris.item_approval_status AS stepapproveordenystatus,
	    role.name AS rolename,
	    ds.name AS appname,
	    requser.display_name AS requestusername,
	    requser.unique_user_id AS requestuserid,
	    rr.creation_time AS requesttime,
	    apprvuser.display_name AS approveordenyusername,
	    apprvuser.unique_user_id AS approveordenyuserid,
	    CASE
	        WHEN rras.id IS NULL THEN NULL
	        ELSE rri.item_approval_status
	    END AS approveordenystatus,
	    rri.item_approval_status,
	    rri.last_status_update_time AS approveordenytime,
	    recipuser.display_name AS recipientusername,
	    recipuser.unique_user_id AS recipientuserid,
	    rri.add_item AS add_request,
	    rri.reason AS item_reason,
	    rr.reason AS request_reason,
	    rras.finish_comment AS approval_comment,
	    cia.change_item_status AS fulfillment_status,
	    cia.item_comment AS fulfillment_comment,
	    cia.last_updated AS fulfillment_date,
	    fufusr.display_name AS fulfillerusername,
	    fufusr.unique_user_id AS fulfilleruserid,
	    CASE
	        WHEN cri.fulfiller_type = 'GROUP' THEN sgrp.name
	        WHEN cri.fulfiller_type = 'USER' THEN susr.display_name
	        ELSE NULL
	    END AS assigned_fulfiller
	   	FROM resource_request_item rri
		JOIN resource_request rr ON rr.id = rri.resource_request_id
		LEFT JOIN resource_request_item_step rris ON rris.resource_request_item_id = rri.id
		LEFT JOIN suser requser ON requser.unique_user_id = rr.unique_requester_id AND requser.effective = 1
		LEFT JOIN suser recipuser ON recipuser.unique_user_id = rri.unique_recipient_id AND recipuser.effective = 1
		LEFT JOIN spermission perm ON perm.unique_permission_id = rri.unique_item_id AND perm.effective = 1 AND rri.item_type = 'PERMISSION'
		LEFT JOIN role_policy role ON role.unique_policy_id = rri.unique_item_id AND role.deleted = 0 AND rri.item_type = 'TECHNICAL_ROLE'
		LEFT JOIN application app ON CAST(app.id AS NVARCHAR(255)) = rri.unique_item_id AND rri.item_type = 'APPLICATION'
		LEFT JOIN data_source ds ON ds.id = app.id AND ds.deleted = 0
		LEFT JOIN fulfillment_item_combined fic ON fic.id = rri.change_request_item_id
		LEFT JOIN change_request_item cri ON cri.id = fic.id
		LEFT JOIN change_item_action cia ON cia.id = cri.latest_action
		LEFT JOIN suser fufusr ON cia.fulfiller_id = fufusr.unique_user_id AND fufusr.effective = 1
		LEFT JOIN resource_request_approval_step rras ON rras.id = rris.approval_step_id
		LEFT JOIN suser stepuser ON stepuser.unique_user_id = rras.finished_by AND stepuser.effective = 1
		LEFT JOIN suser stepuser2 ON stepuser2.unique_user_id = rris.status_set_by AND stepuser2.effective = 1
		LEFT JOIN suser apprvuser ON apprvuser.unique_user_id = rri.status_changed_by AND apprvuser.effective = 1 AND rras.id IS NOT NULL
		LEFT JOIN sgroup sgrp ON sgrp.unique_group_id = cri.fulfiller AND sgrp.effective = 1
		LEFT JOIN suser susr ON susr.unique_user_id = cri.fulfiller AND susr.effective = 1
GO

CREATE VIEW resource_request_v AS
	SELECT
		rreq.unique_requester_id,
		rreq.current_approval_step,
		rreq.change_set_id,
		rreq.reason,
		rreq.request_status,
		rreq.approval_policy_id,
		rreq.creation_time,
		rreq.approval_start_time,
		rreq.change_set_start_time,
		rreq.last_update_time,
		rreq.id,
		rreq.approval_strategy_json,
		rreq.request_node_json,
		rreq.psodv_approval_start_time
		FROM resource_request rreq
	    JOIN suser su ON su.unique_user_id = rreq.unique_requester_id AND su.effective = 1
GO

GRANT SELECT ON approval_request_policy_v TO ig_report_role
GO

GRANT SELECT ON request_item_psod_v TO ig_report_role
GO

GRANT SELECT ON request_lifecycle_v TO ig_report_role
GO

GRANT SELECT ON resource_request_v TO ig_report_role
GO