USE [igops]
GO

DROP VIEW IF EXISTS current_user_access_v
GO

DROP VIEW IF EXISTS auth_user_resource_v
GO

DROP VIEW IF EXISTS role_perm_policy_v
GO

CREATE VIEW current_user_access_v AS
  WITH  trole_users AS(
    SELECT
      role_user.policy,
      suser.unique_user_id
    FROM role_user
      JOIN suser ON role_user.suser = suser.id
) ,current_user_access AS (
  SELECT DISTINCT 
      users.id,
      users.display_name,
      users.job_title,
      STUFF((SELECT DISTINCT ' | ' + suser_email.email FROM suser suser1
        JOIN suser_email ON users.id = suser_email.suser AND suser1.effective = 1 AND suser1.deleted = 0
        WHERE suser1.id = users.id
        FOR XML PATH('')),1,3,'') email,
      users.department,
      users.user_id,
      users.first_name,
      users.last_name,
      users.risk,
      users.unique_user_id,
      supervisor.display_name supervisor,
      sci.violations,
      CASE
        WHEN sgroup.group_id IS NOT NULL THEN '1-GROUP'
        ELSE '1-NO-GROUP'
      END detail_type,
      sgroup.group_id group_id,
      sgroup.name group_name,
      sgroup.short_description group_description,
      NULL permission_id,
      NULL permission_name,
      NULL permission_description,
      NULL permission_type,
      NULL permission_application_name,
      NULL permission_risk,
      NULL permission_root,
      NULL permission_policy_violated,
      NULL permission_entitlements,
      NULL brole_id,
      NULL brole_name,
      NULL brole_description,
      NULL brole_risk,
      NULL brole_update_time,
      NULL supervisee_id,
      NULL supervisee_display_name,
      NULL supervisee_email,
      NULL supervisee_department,
      NULL supervisee_job_title,
      NULL trole_policy_id,
      NULL trole_item_name,
      NULL trole_unique_policy_id,
      NULL trole_description,
      NULL trole_risk,
      NULL trole_policy_violated,
      NULL account_name,
      NULL account_id,
      NULL account_risk,
      NULL account_description,
      NULL account_application_name
    FROM suser users
      LEFT JOIN sgroup_user direct ON direct.suser = users.collected_entity
      LEFT JOIN sgroup_indirect_suser_v indirect ON users.id  = indirect.id AND direct.sgroup = indirect.child_id
      LEFT JOIN sgroup ON sgroup.id = direct.sgroup OR sgroup.id = indirect.parent_id
      LEFT JOIN suser_email ON users.id = suser_email.suser
      LEFT JOIN suser supervisor ON supervisor.unique_user_id = users.supervisor_unique_id AND supervisor.effective = 1 AND supervisor.deleted = 0
    LEFT JOIN ( SELECT 
                     sci1.violator_name,
                     sci1.violator_id,
                     STUFF((SELECT ', ' + CAST(CONCAT(sci2.policy_name,(CONCAT(': SOD-', RIGHT(CONVERT(varchar, CONVERT(varbinary(4), sci2.id), 2), 4)))) AS nvarchar(MAX))
                     FROM (SELECT sc.id,
              CASE
                WHEN sv.suser IS NOT NULL THEN su.display_name
                ELSE sa.name
              END AS violator_name,
              CASE
                WHEN sv.suser IS NOT NULL THEN su.unique_user_id
                ELSE sa.unique_account_id
              END AS violator_id,
              sp.name AS policy_name
              FROM sod_case sc
            LEFT JOIN sod_policy sp ON sc.policy = sp.id
            LEFT JOIN sod_violation sv ON sc.current_violation = sv.id
            LEFT JOIN suser su ON sv.suser = su.id
            LEFT JOIN saccount sa ON sv.saccount = sa.id) sci2
                     WHERE sci2.violator_id = sci1.violator_id FOR XML PATH('')),1,2,'') AS violations
           FROM (SELECT sc.id,
              CASE
                WHEN sv.suser IS NOT NULL THEN su.display_name
                ELSE sa.name
              END AS violator_name,
              CASE
                WHEN sv.suser IS NOT NULL THEN su.unique_user_id
                ELSE sa.unique_account_id
              END AS violator_id,
              sp.name AS policy_name
              FROM sod_case sc
            LEFT JOIN sod_policy sp ON sc.policy = sp.id
            LEFT JOIN sod_violation sv ON sc.current_violation = sv.id
            LEFT JOIN suser su ON sv.suser = su.id
            LEFT JOIN saccount sa ON sv.saccount = sa.id)sci1
                 GROUP BY sci1.violator_name, sci1.violator_id) sci ON sci.violator_id = users.unique_user_id
    WHERE users.effective = 1 AND users.deleted = 0
    GROUP BY 
    users.id, 
    users.display_name, 
    users.job_title, 
    users.department, 
    users.user_id,
    users.first_name,
    users.last_name,
    users.risk,
    users.unique_user_id,
    supervisor.display_name, 
    sci.violations,  
    sgroup.group_id, 
    sgroup.name, 
    sgroup.short_description
     UNION ALL
     
 SELECT DISTINCT
      users.id,
      users.display_name,
      users.job_title,
      STUFF((SELECT DISTINCT ' | ' + suser_email.email FROM suser suser1
        JOIN suser_email ON users.id = suser_email.suser AND suser1.effective = 1 AND suser1.deleted = 0
        WHERE suser1.id = users.id
        FOR XML PATH('')),1,3,'') email,
      users.department,
      users.user_id,
      users.first_name,
      users.last_name,
      users.risk,
      users.unique_user_id,
      supervisor.display_name supervisor,
      sci.violations,
        CASE
          WHEN perm.permission_id IS NOT NULL THEN '2-PERMISSION'
          ELSE '2-NO-PERM'
        END detail_type,
      NULL group_id,
      NULL group_name,
      NULL group_description,
      perm.permission_id,
      perm.name permission_name,
      perm.description permission_description,
      perm.type permission_type,
      perm.application_name permission_application_name,
      perm.risk permission_risk,
      sav.permission permission_root,
      spv.policy_name permission_policy_violated,
      STUFF((SELECT ' | ' + CAST(perm_assoc.name as nvarchar(MAX))  FROM spermission_entailment perm_entail
        LEFT JOIN spermission perm ON perm.collected_entity = perm_entail.entailing
        LEFT JOIN spermission perm_assoc ON perm_assoc.collected_entity = perm_entail.entailed
        WHERE perm.deleted = 0 AND perm.effective = 1 AND perm_assoc.deleted = 0 AND perm_assoc.effective = 1 AND ent.id = perm.id ORDER BY associated_permission FOR XML PATH('')),1,3,'') permission_entitlements,
      NULL brole_id,
      NULL brole_name,
      NULL brole_description,
      NULL brole_risk,
      NULL brole_update_time,
      NULL supervisee_id,
      NULL supervisee_display_name,
      NULL supervisee_email,
      NULL supervisee_department,
      NULL supervisee_job_title,
      NULL trole_policy_id,
      NULL trole_item_name,
      NULL trole_unique_policy_id,
      NULL trole_description,
      NULL trole_risk,
      NULL trole_policy_violated,
      NULL account_name,
      NULL account_id,
      NULL account_risk,
      NULL account_description,
      NULL account_application_name
    FROM suser users
    
      LEFT JOIN 
    (SELECT
      perm.id permission_id, 
      sap.suser, perm.name, 
      perm.description, 
      perm.type, 
      data_source.name application_name, 
      perm.risk 
     FROM spermission perm JOIN data_source ON perm.application = data_source.id 
     JOIN suser_account_permission sap ON perm.id = sap.spermission 
     WHERE (exclude_from_catalog = 0 OR exclude_from_catalog IS NULL) AND effective = 1) perm ON users.id = perm.suser
      LEFT JOIN 
    (SELECT DISTINCT perm.id, 
        perm_assoc.name associated_permission 
    FROM spermission_entailment perm_entail
    LEFT JOIN spermission perm ON perm.collected_entity = perm_entail.entailing
    LEFT JOIN spermission perm_assoc ON perm_assoc.collected_entity = perm_entail.entailed
     WHERE ((((perm.deleted = 0) AND (perm.effective = 1)) AND (perm_assoc.deleted = 0)) AND (perm_assoc.effective = 1))) ent ON ent.id = perm.permission_id
      LEFT JOIN spermission perm1 ON perm1.id = perm.permission_id AND perm1.effective = 1 AND perm1.deleted = 0
      
     LEFT JOIN 
    (SELECT DISTINCT
        perm.name permission,
        perm.id,
        perm_entail.entailed associated_permission_id
    FROM spermission_entailment perm_entail
    LEFT JOIN spermission perm ON perm.collected_entity = perm_entail.entailing
    LEFT JOIN spermission perm_assoc ON perm_assoc.collected_entity = perm_entail.entailed
     WHERE perm.deleted = 0 AND perm.effective = 1 AND perm_assoc.deleted = 0 AND perm_assoc.effective = 1) sav ON sav.associated_permission_id = perm1.collected_entity
     
        LEFT JOIN (SELECT 
                sv.suser,
                sp.name policy_name, 
                sod_condition_item.unique_id
             FROM sod_policy sp
             LEFT JOIN sod_violation sv ON sp.id = sv.policy
             LEFT JOIN sod_condition ON sod_condition.policy = sp.id
             LEFT JOIN sod_condition_item ON sod_condition_item.condition = sod_condition.id) spv ON spv.unique_id = perm1.unique_permission_id AND users.collected_entity = spv.suser
  LEFT JOIN (SELECT DISTINCT
                     suser1.id suser_id,
          STUFF((SELECT DISTINCT ' | ' + suser_email.email FROM suser suser1
          JOIN suser_email ON suser1.id = suser_email.suser AND suser1.effective = 1 AND suser1.deleted = 0
          --WHERE suser1.id = users.id
          FOR XML PATH('')),1,3,'') email
                 FROM suser suser1
                     JOIN suser_email suser_email1 ON suser1.id = suser_email1.suser AND suser1.effective = 1 AND suser1.deleted = 0
                 GROUP BY suser1.id) suser_email ON users.id = suser_email.suser_id
      LEFT JOIN suser supervisor ON supervisor.unique_user_id = users.supervisor_unique_id AND supervisor.effective = 1 AND supervisor.deleted = 0
    LEFT JOIN ( SELECT 
                     sci1.violator_name,
                     sci1.violator_id,
                     STUFF((SELECT ', ' + CAST(CONCAT(sci2.policy_name,(CONCAT(': SOD-', RIGHT(CONVERT(varchar, CONVERT(varbinary(4), sci2.id), 2), 4)))) AS nvarchar(MAX))
                     FROM (SELECT sc.id,
              CASE
                WHEN sv.suser IS NOT NULL THEN su.display_name
                ELSE sa.name
              END AS violator_name,
              CASE
                WHEN sv.suser IS NOT NULL THEN su.unique_user_id
                ELSE sa.unique_account_id
              END AS violator_id,
              sp.name AS policy_name
              FROM sod_case sc
            LEFT JOIN sod_policy sp ON sc.policy = sp.id
            LEFT JOIN sod_violation sv ON sc.current_violation = sv.id
            LEFT JOIN suser su ON sv.suser = su.id
            LEFT JOIN saccount sa ON sv.saccount = sa.id) sci2
                     WHERE sci2.violator_id = sci1.violator_id FOR XML PATH('')),1,2,'') AS violations
           FROM (SELECT sc.id,
              CASE
                WHEN sv.suser IS NOT NULL THEN su.display_name
                ELSE sa.name
              END AS violator_name,
              CASE
                WHEN sv.suser IS NOT NULL THEN su.unique_user_id
                ELSE sa.unique_account_id
              END AS violator_id,
              sp.name AS policy_name
              FROM sod_case sc
            LEFT JOIN sod_policy sp ON sc.policy = sp.id
            LEFT JOIN sod_violation sv ON sc.current_violation = sv.id
            LEFT JOIN suser su ON sv.suser = su.id
            LEFT JOIN saccount sa ON sv.saccount = sa.id)sci1
                 GROUP BY sci1.violator_name, sci1.violator_id) sci ON sci.violator_id = users.unique_user_id 
    WHERE users.effective = 1 AND users.deleted = 0
    GROUP BY users.id, 
  users.display_name, 
  users.job_title, 
  suser_email.email, 
  users.department, 
  users.user_id, 
  users.first_name, 
  users.last_name, 
  users.risk, 
  users.unique_user_id, 
  supervisor.display_name, 
  sci.violations, 
  perm.permission_id, 
  perm.name, 
  perm.description, 
  perm.type, 
  perm.application_name, 
  perm.risk, 
  sav.permission, 
  spv.policy_name, 
  ent.associated_permission, 
  ent.id
 
     UNION ALL
     SELECT 
      users.id,
      users.display_name,
      users.job_title,
      STUFF((SELECT DISTINCT ' | ' + suser_email.email FROM suser suser1
        JOIN suser_email ON users.id = suser_email.suser AND suser1.effective = 1 AND suser1.deleted = 0
        WHERE suser1.id = users.id
        FOR XML PATH('')),1,3,'') email,
      users.department,
      users.user_id,
      users.first_name,
      users.last_name,
      users.risk,
      users.unique_user_id,
      supervisor.display_name supervisor,
      sci.violations,
        CASE
          WHEN br.id IS NOT NULL THEN '3-BROLE'
          ELSE '3-NO-BROLE'
        END detail_type,
      NULL group_id,
      NULL group_name,
      NULL group_description,
      NULL permission_id,
      NULL permission_name,
      NULL permission_description,
      NULL permission_type,
      NULL permission_application_name,
      NULL permission_risk,
      NULL permission_root,
      NULL permission_policy_violated,
      NULL permission_entitlements,
      br.id brole_id,
      br.name brole_name,
      br.description brole_description,
      br.risk brole_risk,
      br.update_time brole_update_time,
      NULL supervisee_id,
      NULL supervisee_display_name,
      NULL supervisee_email,
      NULL supervisee_department,
      NULL supervisee_job_title,
      NULL trole_policy_id,
      NULL trole_item_name,
      NULL trole_unique_policy_id,
      NULL trole_description,
      NULL trole_risk,
      NULL trole_policy_violated,
      NULL account_name,
      NULL account_id,
      NULL account_risk,
      NULL account_description,
      NULL account_application_name
    FROM business_role_member brm
      LEFT JOIN business_role br ON brm.unique_business_role_id = br.unique_id AND br.approval_state = 'PUBLISHED'
      RIGHT JOIN suser users ON users.unique_user_id = brm.unique_user_id
      LEFT JOIN suser_email ON users.id = suser_email.suser
      LEFT JOIN suser supervisor ON supervisor.unique_user_id = users.supervisor_unique_id AND supervisor.effective = 1 AND supervisor.deleted = 0
    LEFT JOIN ( SELECT 
                     sci1.violator_name,
                     sci1.violator_id,
                     STUFF((SELECT ', ' + CAST(CONCAT(sci2.policy_name,(CONCAT(': SOD-', RIGHT(CONVERT(varchar, CONVERT(varbinary(4), sci2.id), 2), 4)))) AS nvarchar(MAX))
                     FROM (SELECT sc.id,
              CASE
                WHEN sv.suser IS NOT NULL THEN su.display_name
                ELSE sa.name
              END AS violator_name,
              CASE
                WHEN sv.suser IS NOT NULL THEN su.unique_user_id
                ELSE sa.unique_account_id
              END AS violator_id,
              sp.name AS policy_name
              FROM sod_case sc
            LEFT JOIN sod_policy sp ON sc.policy = sp.id
            LEFT JOIN sod_violation sv ON sc.current_violation = sv.id
            LEFT JOIN suser su ON sv.suser = su.id
            LEFT JOIN saccount sa ON sv.saccount = sa.id) sci2
                     WHERE sci2.violator_id = sci1.violator_id FOR XML PATH('')),1,2,'') AS violations
           FROM (SELECT sc.id,
              CASE
                WHEN sv.suser IS NOT NULL THEN su.display_name
                ELSE sa.name
              END AS violator_name,
              CASE
                WHEN sv.suser IS NOT NULL THEN su.unique_user_id
                ELSE sa.unique_account_id
              END AS violator_id,
              sp.name AS policy_name
              FROM sod_case sc
            LEFT JOIN sod_policy sp ON sc.policy = sp.id
            LEFT JOIN sod_violation sv ON sc.current_violation = sv.id
            LEFT JOIN suser su ON sv.suser = su.id
            LEFT JOIN saccount sa ON sv.saccount = sa.id)sci1
                 GROUP BY sci1.violator_name, sci1.violator_id) sci ON sci.violator_id = users.unique_user_id 
    WHERE users.effective = 1 AND users.deleted = 0 and brm.deleted = 0
    GROUP BY 
    users.id, 
    supervisor.display_name, 
    sci.violations, 
    br.id, 
    br.name, 
    br.description, 
    br.risk, 
    br.update_time, 
    users.display_name, 
    users.job_title,
    users.department,
    users.user_id,
    users.first_name,
    users.last_name,
    users.risk,
    users.unique_user_id
    
    UNION ALL
    SELECT 
      users.id,
      users.display_name,
      users.job_title,
      STUFF((SELECT DISTINCT ' | ' + suser_email.email FROM suser suser1
        JOIN suser_email ON users.id = suser_email.suser AND suser1.effective = 1 AND suser1.deleted = 0
        WHERE suser1.id = users.id
        FOR XML PATH('')),1,3,'') email,
      users.department,
      users.user_id,
      users.first_name,
      users.last_name,
      users.risk,
      users.unique_user_id,
      supervisor.display_name supervisor,
      sci.violations,
        CASE
          WHEN supervisee.unique_user_id IS NOT NULL THEN '4-SUPERVISEE' 
          ELSE '4-NO-SUPERVISEE'
        END detail_type,
      NULL group_id,
      NULL group_name,
      NULL group_description,
      NULL permission_id,
      NULL permission_name,
      NULL permission_description,
      NULL permission_type,
      NULL permission_application_name,
      NULL permission_risk,
      NULL permission_root,
      NULL permission_policy_violated,
      NULL permission_entitlements,
      NULL brole_id,
      NULL brole_name,
      NULL brole_description,
      NULL brole_risk,
      NULL brole_update_time,
      supervisee.unique_user_id supervisee_id,
      supervisee.display_name supervisee_display_name,
      supervisee_email.email  supervisee_email,
      supervisee.department supervisee_department,
      supervisee.job_title supervisee_job_title,
      NULL trole_policy_id,
      NULL trole_item_name,
      NULL trole_unique_policy_id,
      NULL trole_description,
      NULL trole_risk,
      NULL trole_policy_violated,
      NULL account_name,
      NULL account_id,
      NULL account_risk,
      NULL account_description,
      NULL account_application_name
    FROM suser users
      LEFT JOIN suser supervisee ON supervisee.supervisor_unique_id = users.unique_user_id AND supervisee.deleted = '0' AND supervisee.effective = '1'
      LEFT JOIN suser supervisor ON supervisor.unique_user_id = users.supervisor_unique_id AND supervisor.effective = 1 AND supervisor.deleted = 0
      LEFT JOIN suser_email ON users.id = suser_email.suser
    LEFT JOIN 
        (SELECT
            suser.id suser,
            suser.display_name,
            STUFF((SELECT DISTINCT ' | ' + suser_email.email FROM suser suser1
          JOIN suser_email ON suser.id = suser_email.suser AND suser1.effective = 1 AND suser1.deleted = 0
          WHERE suser1.id = suser.id
          FOR XML PATH('')),1,3,'') email
        FROM suser
        JOIN suser_email ON suser.id = suser_email.suser AND suser.effective = '1' AND suser.deleted = '0'   
        GROUP BY suser.id, suser.display_name) supervisee_email ON supervisee.id = supervisee_email.suser
    LEFT JOIN ( SELECT 
                     sci1.violator_name,
                     sci1.violator_id,
                     STUFF((SELECT ', ' + CAST(CONCAT(sci2.policy_name,(CONCAT(': SOD-', RIGHT(CONVERT(varchar, CONVERT(varbinary(4), sci2.id), 2), 4)))) AS nvarchar(MAX))
                     FROM (SELECT sc.id,
              CASE
                WHEN sv.suser IS NOT NULL THEN su.display_name
                ELSE sa.name
              END AS violator_name,
              CASE
                WHEN sv.suser IS NOT NULL THEN su.unique_user_id
                ELSE sa.unique_account_id
              END AS violator_id,
              sp.name AS policy_name
              FROM sod_case sc
            LEFT JOIN sod_policy sp ON sc.policy = sp.id
            LEFT JOIN sod_violation sv ON sc.current_violation = sv.id
            LEFT JOIN suser su ON sv.suser = su.id
            LEFT JOIN saccount sa ON sv.saccount = sa.id) sci2
                     WHERE sci2.violator_id = sci1.violator_id FOR XML PATH('')),1,2,'') AS violations
           FROM (SELECT sc.id,
              CASE
                WHEN sv.suser IS NOT NULL THEN su.display_name
                ELSE sa.name
              END AS violator_name,
              CASE
                WHEN sv.suser IS NOT NULL THEN su.unique_user_id
                ELSE sa.unique_account_id
              END AS violator_id,
              sp.name AS policy_name
              FROM sod_case sc
            LEFT JOIN sod_policy sp ON sc.policy = sp.id
            LEFT JOIN sod_violation sv ON sc.current_violation = sv.id
            LEFT JOIN suser su ON sv.suser = su.id
            LEFT JOIN saccount sa ON sv.saccount = sa.id)sci1
                 GROUP BY sci1.violator_name, sci1.violator_id) sci ON sci.violator_id = users.unique_user_id 
    WHERE users.effective = 1 AND users.deleted = 0
    GROUP BY 
    users.id, 
    supervisor.display_name, 
    sci.violations, 
    supervisee.unique_user_id, 
    supervisee.display_name, 
    supervisee_email.email, 
    supervisee.department, 
    supervisee.job_title,
    users.display_name,
    users.job_title,
    users.department,
    users.user_id,
    users.first_name,
    users.last_name,
    users.risk,
    users.unique_user_id

    UNION ALL
        SELECT DISTINCT
      users.id,
      users.display_name,
      users.job_title,
      STUFF((SELECT DISTINCT ' | ' + suser_email.email FROM suser suser1
        JOIN suser_email ON users.id = suser_email.suser AND suser1.effective = 1 AND suser1.deleted = 0
        WHERE suser1.id = users.id
        FOR XML PATH('')),1,3,'') email,
      users.department,
      users.user_id,
      users.first_name,
      users.last_name,
      users.risk,
      users.unique_user_id,
      supervisor.display_name supervisor,
      sci.violations,
        CASE
          WHEN rperm.policy IS NOT NULL THEN '5-TROLE'
          ELSE '5-NO-TROLE'
        END detail_type,
      NULL group_id,
      NULL group_name,
      NULL group_description,
      NULL permission_id,
      NULL permission_name,
      NULL permission_description,
      NULL permission_type,
      NULL permission_application_name,
      NULL permission_risk,
      NULL permission_root,
      NULL permission_policy_violated,
      NULL permission_entitlements,
      NULL brole_id,
      NULL brole_name,
      NULL brole_description,
      NULL brole_risk,
      NULL  brole_update_time,
      NULL supervisee_id,
      NULL supervisee_display_name,
      NULL supervisee_email,
      NULL supervisee_department,
      NULL supervisee_job_title,
      rperm.policy trole_policy_id,
      rpolicy.name trole_item_name,
      rpolicy.unique_policy_id trole_unique_policy_id,
      rpolicy.description trole_description,
      rpolicy.risk trole_risk,
      policy.policy_name trole_policy_violated,
      NULL account_name,
      NULL account_id,
      NULL account_risk,
      NULL account_description,
      NULL account_application_name
    FROM suser users
      LEFT JOIN trole_users ruser ON users.unique_user_id = ruser.unique_user_id
      LEFT JOIN role_policy rpolicy ON rpolicy.id = ruser.policy
      LEFT JOIN role_permission rperm ON ruser.policy = rperm.policy
      LEFT JOIN sod_condition_item sod ON sod.unique_id = rpolicy.unique_policy_id AND users.id = sod.id
      LEFT JOIN sod_condition ON sod.condition = sod_condition.id
      LEFT JOIN sod_policy ON sod_condition.policy = sod_policy.id
      LEFT JOIN suser_email ON users.id = suser_email.suser
      LEFT JOIN suser supervisor ON supervisor.unique_user_id = users.supervisor_unique_id AND supervisor.effective = 1 AND supervisor.deleted = 0
    LEFT JOIN ( SELECT 
                     sci1.violator_name,
                     sci1.violator_id,
                     STUFF((SELECT ', ' + CAST(CONCAT(sci2.policy_name,(CONCAT(': SOD-', RIGHT(CONVERT(varchar, CONVERT(varbinary(4), sci2.id), 2), 4)))) AS nvarchar(MAX))
                     FROM (SELECT sc.id,
              CASE
                WHEN sv.suser IS NOT NULL THEN su.display_name
                ELSE sa.name
              END AS violator_name,
              CASE
                WHEN sv.suser IS NOT NULL THEN su.unique_user_id
                ELSE sa.unique_account_id
              END AS violator_id,
              sp.name AS policy_name
              FROM sod_case sc
            LEFT JOIN sod_policy sp ON sc.policy = sp.id
            LEFT JOIN sod_violation sv ON sc.current_violation = sv.id
            LEFT JOIN suser su ON sv.suser = su.id
            LEFT JOIN saccount sa ON sv.saccount = sa.id) sci2
                     WHERE sci2.violator_id = sci1.violator_id FOR XML PATH('')),1,2,'') AS violations
           FROM (SELECT sc.id,
              CASE
                WHEN sv.suser IS NOT NULL THEN su.display_name
                ELSE sa.name
              END AS violator_name,
              CASE
                WHEN sv.suser IS NOT NULL THEN su.unique_user_id
                ELSE sa.unique_account_id
              END AS violator_id,
              sp.name AS policy_name
              FROM sod_case sc
            LEFT JOIN sod_policy sp ON sc.policy = sp.id
            LEFT JOIN sod_violation sv ON sc.current_violation = sv.id
            LEFT JOIN suser su ON sv.suser = su.id
            LEFT JOIN saccount sa ON sv.saccount = sa.id)sci1
                 GROUP BY sci1.violator_name, sci1.violator_id) sci ON sci.violator_id = users.unique_user_id
    LEFT JOIN (SELECT DISTINCT 
          spv1.UNIQUE_ID,
          spv1.item_name,
          spv1.perm_id,
          users1.unique_user_id, 
          (STUFF((SELECT ', ' + (CONCAT(spv.policy_name, CONCAT(': SOD-', RIGHT(REPLICATE(0,4) + LEFT(CONVERT(VARCHAR(MAX), CONVERT(VARBINARY(2), spv.case_id, 0), 2), 4), 4) )))
            FROM suser users
              LEFT JOIN (SELECT sc.id AS case_id,
                CASE
                  WHEN sv.suser IS NOT NULL THEN su.display_name
                  ELSE sa.name
                END AS violator_name,
                sp.name as policy_name,
                sod_condition_item.unique_id,
                su.unique_user_id   
               FROM sod_case sc
               LEFT JOIN sod_policy sp ON sc.policy = sp.id
               LEFT JOIN sod_violation sv ON sc.current_violation = sv.id
               LEFT JOIN suser su ON sv.suser = su.collected_entity AND su.effective = 1 AND su.deleted = 0
               LEFT JOIN saccount sa ON sv.saccount = sa.id
               LEFT JOIN sod_condition ON sod_condition.policy = sc.policy
               LEFT JOIN sod_condition_item ON sod_condition_item.condition = sod_condition.id
               LEFT JOIN spermission spermission_1 ON spermission_1.unique_permission_id = sod_condition_item.unique_id AND spermission_1.effective = 1 AND spermission_1.deleted = 0
               LEFT JOIN role_policy ON role_policy.unique_policy_id = sod_condition_item.unique_id AND role_policy.deleted = 0
               ) spv ON spv.unique_user_id = users.unique_user_id
            WHERE users.unique_user_id = users1.unique_user_id AND spv.unique_id = spv1.unique_id AND spv.policy_name IS NOT NULL AND users.effective = 1 FOR XML PATH('')),1,1,'')) policy_name
            FROM suser users1
            LEFT JOIN (SELECT sc.id AS case_id,
                    spermission_1.id as perm_id,
                    CASE
                      WHEN sv.suser IS NOT NULL THEN su.display_name
                      ELSE sa.name
                    END AS violator_name,
                    sp.name as policy_name,
                    CASE
                    WHEN sod_condition_item.type = 'PERMISSION' THEN spermission_1.name
                    ELSE role_policy.name
                  END AS item_name,
                    sod_condition_item.unique_id,
                    su.unique_user_id
                   FROM sod_case sc
                   LEFT JOIN sod_policy sp ON sc.policy = sp.id
                   LEFT JOIN sod_violation sv ON sc.current_violation = sv.id
                   LEFT JOIN suser su ON sv.suser = su.collected_entity AND su.effective = 1 AND su.deleted = 0
                   LEFT JOIN saccount sa ON sv.saccount = sa.id
                   LEFT JOIN sod_condition ON sod_condition.policy = sc.policy
                   LEFT JOIN sod_condition_item ON sod_condition_item.condition = sod_condition.id
                   LEFT JOIN spermission spermission_1 ON spermission_1.unique_permission_id = sod_condition_item.unique_id AND spermission_1.effective = 1 AND spermission_1.deleted = 0
               LEFT JOIN role_policy ON role_policy.unique_policy_id = sod_condition_item.unique_id AND role_policy.deleted = 0) spv1 ON spv1.unique_user_id = users1.unique_user_id
    WHERE spv1.policy_name IS NOT NULL AND users1.effective = 1
    GROUP BY users1.unique_user_id, spv1.perm_id, users1.effective, spv1.unique_id, spv1.item_name, spv1.UNIQUE_ID) policy ON policy.unique_user_id = users.unique_user_id AND policy.unique_id = rpolicy.unique_policy_id
      WHERE users.effective = 1 AND users.deleted = 0 AND (rpolicy.deleted = 0 OR rpolicy.ID IS NULL)
      GROUP BY 
      users.id, 
      supervisor.display_name, 
      sci.violations, 
      rperm.policy, 
      rpolicy.name, 
      rpolicy.unique_policy_id, 
      rpolicy.description, 
      rpolicy.risk, 
      policy.policy_name, 
      users.display_name, 
      users.job_title, 
      users.department, 
      users.user_id, 
      users.first_name, 
      users.last_name, 
      users.risk, 
      users.unique_user_id

    UNION ALL
    SELECT DISTINCT
      users.id,
      users.display_name,
      users.job_title,
      STUFF((SELECT DISTINCT ' | ' + suser_email.email FROM suser suser1
        JOIN suser_email ON users.id = suser_email.suser AND suser1.effective = 1 AND suser1.deleted = 0
        WHERE suser1.id = users.id
        FOR XML PATH('')),1,3,'') email,
      users.department,
      users.user_id,
      users.first_name,
      users.last_name,
      users.risk,
      users.unique_user_id,
      supervisor.display_name supervisor,
      sci.violations,
        CASE
          WHEN sac.account_id IS NOT NULL THEN '6-ACCOUNT'
          ELSE '6-NO-ACCOUNT'
        END detail_type,
      NULL group_id,
      NULL group_name,
      NULL group_description,
      NULL permission_id,
      NULL permission_name,
      NULL permission_description,
      NULL permission_type,
      NULL permission_application_name,
      NULL permission_risk,
      NULL permission_root,
      NULL permission_policy_violated,
      NULL permission_entitlements,
      NULL brole_id,
      NULL brole_name,
      NULL brole_description,
      NULL brole_risk,
      NULL brole_update_time,
      NULL supervisee_id,
      NULL supervisee_display_name,
      NULL supervisee_email,
      NULL supervisee_department,
      NULL supervisee_job_title,
      NULL trole_policy_id,
      NULL trole_item_name,
      NULL trole_unique_policy_id,
      NULL trole_description,
      NULL trole_risk,
      NULL trole_policy_violated,
      sac.name account_name,
      sac.account_id,
      sac.risk account_risk,
      sac.description account_description,
      source.name account_application_name
  FROM suser_account_permission sap
      JOIN saccount sac ON sap.saccount = sac.id AND sac.effective = '1' AND sac.deleted = '0' AND sac.virtual = '0'
      JOIN data_source source ON sac.application = source.id
      RIGHT JOIN suser users ON sap.suser = users.id
      LEFT JOIN suser_email ON users.id = suser_email.suser
      LEFT JOIN suser supervisor ON supervisor.unique_user_id = users.supervisor_unique_id AND supervisor.effective = 1 AND supervisor.deleted = 0
    LEFT JOIN ( SELECT 
                     sci1.violator_name,
                     sci1.violator_id,
                     STUFF((SELECT ', ' + CAST(CONCAT(sci2.policy_name,(CONCAT(': SOD-', RIGHT(CONVERT(varchar, CONVERT(varbinary(4), sci2.id), 2), 4)))) AS nvarchar(MAX))
                     FROM (SELECT sc.id,
              CASE
                WHEN sv.suser IS NOT NULL THEN su.display_name
                ELSE sa.name
              END AS violator_name,
              CASE
                WHEN sv.suser IS NOT NULL THEN su.unique_user_id
                ELSE sa.unique_account_id
              END AS violator_id,
              sp.name AS policy_name
              FROM sod_case sc
            LEFT JOIN sod_policy sp ON sc.policy = sp.id
            LEFT JOIN sod_violation sv ON sc.current_violation = sv.id
            LEFT JOIN suser su ON sv.suser = su.id
            LEFT JOIN saccount sa ON sv.saccount = sa.id) sci2
                     WHERE sci2.violator_id = sci1.violator_id FOR XML PATH('')),1,2,'') AS violations
           FROM (SELECT sc.id,
              CASE
                WHEN sv.suser IS NOT NULL THEN su.display_name
                ELSE sa.name
              END AS violator_name,
              CASE
                WHEN sv.suser IS NOT NULL THEN su.unique_user_id
                ELSE sa.unique_account_id
              END AS violator_id,
              sp.name AS policy_name
              FROM sod_case sc
            LEFT JOIN sod_policy sp ON sc.policy = sp.id
            LEFT JOIN sod_violation sv ON sc.current_violation = sv.id
            LEFT JOIN suser su ON sv.suser = su.id
            LEFT JOIN saccount sa ON sv.saccount = sa.id)sci1
                 GROUP BY sci1.violator_name, sci1.violator_id) sci ON sci.violator_id = users.unique_user_id
      WHERE users.effective = 1 AND users.deleted = 0
      GROUP BY 
      users.id,
      supervisor.display_name, 
      sci.violations, 
      sac.account_id, 
      sac.name, 
      sac.risk, 
      sac.description, 
      source.name,
      users.display_name, 
      users.job_title, 
      users.department, 
      users.user_id, 
      users.first_name, 
      users.last_name, 
      users.risk, 
      users.unique_user_id      
  )
  SELECT 
    current_user_access.id,
    current_user_access.display_name,
    current_user_access.job_title,
    current_user_access.email,
    current_user_access.department,
    current_user_access.user_id user_cn,
    current_user_access.first_name,
    current_user_access.last_name,
    current_user_access.risk,
    current_user_access.unique_user_id,
    current_user_access.supervisor,
    current_user_access.violations,
    current_user_access.detail_type,
    current_user_access.group_id,
    current_user_access.group_name,
    current_user_access.group_description,
    current_user_access.permission_id,
    current_user_access.permission_name,
    current_user_access.permission_description,
    current_user_access.permission_type,
    current_user_access.permission_application_name,
    current_user_access.permission_risk,
    current_user_access.permission_root,
    current_user_access.permission_policy_violated,
    current_user_access.permission_entitlements,
    current_user_access.brole_id,
    current_user_access.brole_name,
    current_user_access.brole_description,
    current_user_access.brole_risk,
    current_user_access.brole_update_time,
    current_user_access.supervisee_id,
    current_user_access.supervisee_display_name,
    current_user_access.supervisee_email,
    current_user_access.supervisee_department,
    current_user_access.supervisee_job_title,
    current_user_access.trole_policy_id,
    current_user_access.trole_item_name,
    current_user_access.trole_unique_policy_id,
    current_user_access.trole_description,
    current_user_access.trole_risk,
    current_user_access.trole_policy_violated,
    current_user_access.account_name,
    current_user_access.account_id,
    current_user_access.account_risk,
    current_user_access.account_description,
    current_user_access.account_application_name
  FROM current_user_access
GO

CREATE VIEW auth_user_resource_v AS
SELECT
    br.name AS brole,
  br.unique_id AS unique_business_role_id,
  su.display_name AS user_name,
  su.unique_user_id,
  sp.name AS permission_name,
  sp.unique_permission_id,
  rp.name AS role_name,
  rp.unique_policy_id,
  ds.name AS application,
  ds.unique_id AS unique_application_id,
  aur.start_date,
  aur.end_date,
  aur.in_validity_period,
  aur.in_grace_period,
  aur.indirect_authorization,
  aur.auto_grant,
  aur.auto_revoke,
  aur.create_date,
  aur.deleted,
  aur.delete_date
FROM auth_user_resource aur
  JOIN user_resource ur ON aur.user_resource_id = ur.id
  JOIN business_role br ON aur.unique_business_role_id = br.unique_id AND br.approval_state = 'PUBLISHED'
  JOIN suser su ON su.unique_user_id = ur.unique_user_id AND su.effective = 1
  LEFT JOIN spermission sp ON sp.unique_permission_id = ur.unique_resource_id AND sp.effective = 1
  LEFT JOIN role_policy rp ON rp.unique_policy_id = ur.unique_resource_id AND rp.deleted = 0
  LEFT JOIN data_source ds ON CAST(ds.id AS NVARCHAR) = ur.unique_resource_id AND ds.deleted = 0
GO

CREATE VIEW role_perm_policy_v AS
SELECT DISTINCT
  policy.id,
  policy.name AS policy_name,
  policy.unique_policy_id,
  policy.description AS policy_description,
  policy.created_by,
  policy.changed_by,
  policy.creation_time,
  policy.update_time,
  policy.deleted,
  policy.last_detection_time,
  CASE
    WHEN policy.state = 'INACTIVE' AND ISNULL(policy.detection_status,'INACTIVE') = 'INACTIVE' THEN 'Inactive'
    WHEN policy.state = 'ACTIVE' AND policy.detection_status IN ('COMPLETED','RUNNING') THEN 'Active'
    WHEN policy.state = 'INACTIVE' AND policy.detection_status = 'INVALID' THEN 'Invalid from inactive'
    WHEN policy.state = 'INVALID' AND policy.detection_status = 'INVALID' THEN 'Invalid from active'
    ELSE NULL
  END AS current_state,
  policy.state AS actual_state,
  policy.detection_status,
  policy.risk_mode AS policy_risk_mode,
  policy.risk AS policy_risk,
  policy.cost AS policy_cost,
  (SELECT STUFF((SELECT ' | ' + powner.unique_owner_id
    FROM policy_owners_v powner
    LEFT JOIN role_policy rperm ON rperm.id = powner.role_policy
    WHERE powner.is_user = 1 AND rperm.id = policy.id
    GROUP BY powner.unique_owner_id
    FOR XML PATH('')), 1, 3, '') AS user_owner_id) AS user_owner_id,
  (SELECT STUFF((SELECT ' | ' + powner.unique_owner_id
    FROM policy_owners_v powner
    LEFT JOIN role_policy rperm ON rperm.id = powner.role_policy
    WHERE powner.is_user = 0 AND rperm.id = policy.id
    GROUP BY powner.unique_owner_id
    FOR XML PATH('')), 1, 3, '') AS group_owner_id) AS group_owner_id,
  (SELECT STUFF((SELECT ' | ' + powner.owner_name
    FROM policy_owners_v powner
    LEFT JOIN role_policy rperm ON rperm.id = powner.role_policy
    WHERE powner.is_user = 1 AND rperm.id = policy.id
    GROUP BY powner.owner_name
    FOR XML PATH('')), 1, 3, '') AS user_owner_id) AS user_owners,
  (SELECT STUFF((SELECT ' | ' + powner.owner_name
    FROM policy_owners_v powner
    LEFT JOIN role_policy rperm ON rperm.id = powner.role_policy
    WHERE powner.is_user = 0 AND rperm.id = policy.id
    GROUP BY powner.owner_name
    FOR XML PATH('')), 1, 3, '') AS user_owner_id) AS group_owners,
  perm.unique_permission_id,
  perm.name AS perm_name,
  perm_user.unique_user_id AS permission_user_id,
  perm_user.display_name AS permission_user_name
FROM
  role_policy policy
  LEFT JOIN policy_owner ON policy.id = policy_owner.role_policy
  LEFT JOIN role_permission role_perm ON  policy.id = role_perm.policy
  LEFT JOIN spermission perm ON role_perm.unique_permission_id = perm.unique_permission_id AND perm.effective = 1
  LEFT JOIN suser_account_permission_role role_user ON  role_user.role = policy.id
  LEFT JOIN suser perm_user ON  perm_user.id = role_user.suser
GO

GRANT SELECT ON current_user_access_v TO ig_report_role
GO

GRANT SELECT ON auth_user_resource_v TO ig_report_role
GO

GRANT SELECT ON role_perm_policy_v TO ig_report_role
GO