USE [igops]
GO

DROP VIEW IF EXISTS request_lifecycle_v
GO

DROP VIEW IF EXISTS perm_assignment_changes_v
GO

DROP VIEW IF EXISTS user_perm_snapshot_v
GO

CREATE VIEW request_lifecycle_v AS
 SELECT perm.name AS permissionname,
    perm.unique_permission_id,
    stepuser.display_name AS stepapprovername,
    stepuser.unique_user_id AS stepapproveruserid,
    rras.finish_time AS stepapprovaltime,
    stepuser2.display_name AS statussettername,
    stepuser2.unique_user_id AS statussetteruserid,
    rris.status_time AS statussettime,
    rr.id AS cartid,
    rri.id,
    rras.step AS stepapprovalorder,
    rr.request_status,
    rris.item_approval_status AS stepapproveordenystatus,
    role.name AS rolename,
    ds.name AS appname,
    requser.display_name AS requestusername,
    requser.unique_user_id AS requestuserid,
    rr.creation_time AS requesttime,
    apprvuser.display_name AS approveordenyusername,
    apprvuser.unique_user_id AS approveordenyuserid,
        CASE
            WHEN rras.id IS NULL THEN NULL
            ELSE rri.item_approval_status
        END AS approveordenystatus,
    rri.item_approval_status,
    rri.last_status_update_time AS approveordenytime,
    recipuser.display_name AS recipientusername,
    recipuser.unique_user_id AS recipientuserid,
    rri.add_item AS add_request,
    rri.reason AS item_reason,
    rr.reason AS request_reason,
    rras.finish_comment AS approval_comment,
    cia.change_item_status AS fulfillment_status,
    cia.item_comment AS fulfillment_comment,
    cia.last_updated AS fulfillment_date,
    fufusr.display_name AS fulfillerusername,
    fufusr.unique_user_id AS fulfilleruserid,
        CASE
            WHEN cri.fulfiller_type = 'GROUP' THEN sgrp.name
            WHEN cri.fulfiller_type = 'USER' THEN susr.display_name
            ELSE NULL
        END AS assigned_fulfiller
   FROM resource_request_item rri
     JOIN resource_request rr ON rr.id = rri.resource_request_id
     LEFT JOIN resource_request_item_step rris ON rris.resource_request_item_id = rri.id
     LEFT JOIN suser requser ON requser.unique_user_id = rr.unique_requester_id AND requser.effective = 1
     LEFT JOIN suser recipuser ON recipuser.unique_user_id = rri.unique_recipient_id AND recipuser.effective = 1
     LEFT JOIN spermission perm ON perm.unique_permission_id = rri.unique_item_id AND perm.effective = 1 AND rri.item_type = 'PERMISSION'
     LEFT JOIN role_policy role ON role.unique_policy_id = rri.unique_item_id AND role.deleted = 0 AND rri.item_type = 'TECHNICAL_ROLE'
     LEFT JOIN application app ON CAST(app.id AS NVARCHAR(255)) = rri.unique_item_id AND rri.item_type = 'APPLICATION'
     LEFT JOIN data_source ds ON ds.id = app.id AND ds.deleted = 0
     LEFT JOIN fulfillment_item_combined fic ON fic.id = rri.change_request_item_id
     LEFT JOIN change_request_item cri ON cri.id = fic.id
     LEFT JOIN change_item_action cia ON cia.id = cri.latest_action
     LEFT JOIN suser fufusr ON cia.fulfiller_id = fufusr.unique_user_id AND fufusr.effective = 1
     LEFT JOIN resource_request_approval_step rras ON rras.id = rris.approval_step_id
     LEFT JOIN suser stepuser ON stepuser.unique_user_id = rras.finished_by AND stepuser.effective = 1
     LEFT JOIN suser stepuser2 ON stepuser2.unique_user_id = rris.status_set_by AND stepuser2.effective = 1
     LEFT JOIN suser apprvuser ON apprvuser.unique_user_id = rri.status_changed_by AND apprvuser.effective = 1 AND rras.id IS NOT NULL
     LEFT JOIN sgroup sgrp ON sgrp.unique_group_id = cri.fulfiller AND sgrp.effective = 1
     LEFT JOIN suser susr ON susr.unique_user_id = cri.fulfiller AND susr.effective = 1
GO

CREATE VIEW user_perm_snapshot_v AS
  SELECT ds.unique_id AS appl_qid,
    ds.name AS appl_name,
    perm.unique_permission_id AS perm_qid,
    perm.name AS perm_name,
    perm.description,
    perm.type,
    root_perm.root_permission,
    assoc_perm.associated_permission,
    suser.display_name,
    suser.unique_user_id,
    suser.last_name AS user_last_name,
    suser.first_name AS user_first_name,
    suser.unique_user_id AS user_qid,
    effective_user.id AS effective_user_id,
    suser.dn AS user_dn,
    suser.collected_entity,
    suser.idm_tree_name AS user_idm_tree_name,
    snap.id AS snap_id,
    snap.timestamp AS snap_timestamp
   FROM spermission perm
    LEFT JOIN data_source ds ON perm.application = ds.id
    LEFT JOIN saccount_permission ap ON perm.collected_entity = ap.spermission
    LEFT JOIN saccount_user sau ON sau.saccount = ap.saccount
    JOIN suser ON sau.suser = suser.collected_entity
    JOIN suser effective_user ON suser.unique_user_id = effective_user.unique_user_id AND effective_user.effective = 1
    LEFT JOIN snapshot_sversions au_snap_sv ON sau.sversion = au_snap_sv.sversions
    LEFT JOIN snapshot snap ON au_snap_sv.snapshot = snap.id AND snap.type = 'PUBLICATION'
    LEFT JOIN (SELECT s_perm.unique_permission_id,
                s_perm.collected_entity,
                STUFF((SELECT DISTINCT ' • ' + perm2.name FROM spermission
                JOIN (SELECT DISTINCT se.entailed, se.entailing FROM spermission_entailment se) entail ON spermission.collected_entity = entail.entailed
                JOIN spermission perm2 ON perm2.collected_entity = entail.entailing AND perm2.deleted = 0
                WHERE spermission.unique_permission_id =  s_perm.unique_permission_id
                FOR XML PATH('')),1,3,'') root_permission
              FROM spermission s_perm
              JOIN spermission_entailment entail ON s_perm.collected_entity = entail.entailed
              JOIN spermission perm2 ON perm2.collected_entity = entail.entailing AND perm2.deleted = 0
              GROUP BY s_perm.unique_permission_id, s_perm.collected_entity) root_perm ON root_perm.unique_permission_id = perm.unique_permission_id
    LEFT JOIN (SELECT s_perm.unique_permission_id,
                s_perm.collected_entity,
                STUFF((SELECT DISTINCT ' • ' + perm3.name FROM spermission
                JOIN (SELECT DISTINCT se.entailed, se.entailing FROM spermission_entailment se) entail ON spermission.collected_entity = entail.entailing
                JOIN spermission perm3 ON perm3.collected_entity = entail.entailed AND perm3.deleted = 0
                WHERE spermission.unique_permission_id = s_perm.unique_permission_id
                FOR XML PATH('')),1,3,'') associated_permission
              FROM spermission s_perm
              JOIN spermission_entailment entail2 ON s_perm.collected_entity = entail2.entailing
              JOIN spermission perm3 ON perm3.collected_entity = entail2.entailed AND perm3.deleted = 0
              GROUP BY s_perm.unique_permission_id, s_perm.collected_entity) assoc_perm ON assoc_perm.unique_permission_id = perm.unique_permission_id 
  WHERE perm.type != 'BoundEntitlement' AND perm.type != 'ResourceParm' AND perm.type != 'BoundResourceParm' AND perm.merged = 0 AND snap.timestamp IS NOT NULL
GO

CREATE VIEW perm_assignment_changes_v AS
WITH dates AS(
  SELECT DISTINCT
    ups.snap_timestamp,
    (SELECT MAX(ups4.snap_timestamp) FROM user_perm_snapshot_v ups4 WHERE ups4.snap_timestamp < ups.snap_timestamp) AS prev_time,
    (SELECT MIN(ups4.snap_timestamp) FROM user_perm_snapshot_v ups4 WHERE ups4.snap_timestamp > ups.snap_timestamp) AS next_time
  FROM (SELECT DISTINCT user_perm_snapshot_v.snap_timestamp FROM user_perm_snapshot_v) ups
)
SELECT DISTINCT
  'ADDED' AS added_removed,
  ups.appl_qid,
  ups.appl_name,
  ups.perm_qid,
  ups.perm_name,
  ups.description,
  ups.type,
  ups.root_permission,
  ups.associated_permission,
  ups.display_name,
  ups.unique_user_id,
  ups.user_last_name,
  ups.user_first_name,
  ups.user_qid,
  ups.effective_user_id,
  ups.user_dn,
  ups.collected_entity,
  ups.user_idm_tree_name,
  ups.snap_id,
  ups.snap_timestamp AS add_remove_date
FROM user_perm_snapshot_v ups
  JOIN dates ON ups.snap_timestamp = dates.snap_timestamp
WHERE NOT EXISTS
  (SELECT
    1
  FROM user_perm_snapshot_v ups2
  WHERE ups2.snap_timestamp = dates.prev_time AND ups2.perm_qid =  ups.perm_qid AND ups2.unique_user_id = ups.unique_user_id)

UNION ALL

SELECT DISTINCT
  'REMOVED' AS added_removed,
  ups.appl_qid,
  ups.appl_name,
  ups.perm_qid,
  ups.perm_name,
  ups.description,
  ups.type,
  ups.root_permission,
  ups.associated_permission,
  ups.display_name,
  ups.unique_user_id,
  ups.user_last_name,
  ups.user_first_name,
  ups.user_qid,
  ups.effective_user_id,
  ups.user_dn,
  ups.collected_entity,
  ups.user_idm_tree_name,
  ups.snap_id,
  dates.next_time AS add_remove_date
FROM user_perm_snapshot_v ups
  JOIN dates ON ups.snap_timestamp = dates.snap_timestamp
WHERE ups.snap_timestamp != (SELECT MAX(ups4.snap_timestamp) FROM user_perm_snapshot_v ups4) 
AND NOT EXISTS
    (SELECT
      1
    FROM user_perm_snapshot_v ups2
    WHERE ups2.snap_timestamp = dates.next_time AND ups2.perm_qid =  ups.perm_qid AND ups2.unique_user_id = ups.unique_user_id)


GO

GRANT SELECT ON request_lifecycle_v TO ig_report_role
GO

GRANT SELECT ON user_perm_snapshot_v TO ig_report_role
GO

GRANT SELECT ON perm_assignment_changes_v TO ig_report_role
GO