USE [igops]
GO

DROP VIEW IF EXISTS access_review_details_v
GO

CREATE VIEW access_review_details_v AS
  SELECT DISTINCT
    item.id review_item_id,
    item.last_override_action_id last_override_action,
    instance.id review_instance_id,
    instance.status review_status,
    instance.start_date,
    instance.end_date,
    instance.valid_to_date,
    action.last_update,
    item.approved_time,
    instance.item_count,
    multi_owner.review_owners,
    multi_reviewer.reviewer_ids reviewer_id,
    CASE
      WHEN item.final_cert_by_unique_user_id IS NOT NULL THEN final_reviewer.display_name
      ELSE multi_reviewer.reviewers
    END reviewer_display_name,
    CASE
      WHEN item.final_cert_by_unique_user_id IS NOT NULL THEN  substring(final_reviewer.dn, 4,  charindex(',', final_reviewer.dn) - 4)
      ELSE multi_reviewer.user_cn
    END reviewer_cn,
    reassigned_by_history.reviewers reassigned_by_user_disp_name,
    reassigned_from_history.reviewers reassigned_from_user_disp_name,
    reassigned_by_history.reasons reassigned_reasons,
    reassigned_by_history.rev_comments reassigned_comments,
    reassigned_by_history.reassigned_date,
    defn.name review_name,
    defn.description,
    defn.review_type,
    defn.next_schedule_date,
    CASE
      WHEN item.item_type = 'ACCOUNT' AND usr.id IS NULL THEN shacc.suser
      ELSE usr.id
    END user_id,
    CASE
      WHEN item.item_type = 'ACCOUNT' AND usr.display_name IS NULL THEN shacc.display_name
      ELSE usr.display_name
    END user_display_name,
    CASE
      WHEN item.item_type = 'ACCOUNT' AND usr.display_name IS NULL THEN shacc.user_cn
      ELSE substring(usr.dn, 4, charindex(',', usr.dn) - 4)
    END user_cn,
    action.action reviewer_action,
    reason.name modify_reason_name,
    reason.id modify_reason_id,
    CASE
      WHEN fulfillment.review_item_id IS NOT NULL THEN 1
      ELSE 0
    END remove_completed,
    usr.unique_user_id,
    action.review_order,
    action.reviewer_comment,
    perm.id permission_id,
    perm.name permission_name,
    acct.id account_id,
    acct.account_id account_native_id,
    acct.name account_name,
    acct.relation_to_user_type account_relation,
    app.id application_id,
    app_ds.name application_name,
    item.final_decision,
    item.final_decision_time,
    role.id role_id,
    role.name role_name,
    item.item_type,
    brole.name brole_name,
    brole.id business_role_id,
    action.fulfillment_instructions,
    esc_reviewer.display_name esc_reviewer,
    sup.unique_user_id supervisor_id,
    sup.display_name supervisor_display_name
  FROM
    review_item item
    JOIN review_instance instance ON item.review_instance_id = instance.id AND instance.status NOT IN ('PREVIEW', 'PREVIEW_CANCELED', 'PREVIEW_STARTING')
     JOIN (SELECT
          review_instance.id inst_id,
          defn.id defn_id,
          CASE WHEN staged_defn.name IS NULL THEN defn.name ELSE staged_defn.name END AS name,
          CASE WHEN staged_defn.description IS NULL THEN defn.description ELSE staged_defn.description END AS description,
          CASE WHEN staged_defn.next_schedule_date IS NULL THEN defn.next_schedule_date ELSE staged_defn.next_schedule_date END AS next_schedule_date,
          defn.review_type,
          defn.deleted
        FROM review_definition defn
        JOIN review_instance ON defn.id = review_instance.review_definition
        LEFT JOIN review_definition staged_defn ON staged_defn.id = review_instance.staged_review_definition_id) defn ON instance.id = defn.inst_id
    LEFT JOIN suser usr ON usr.unique_user_id = item.unique_user_id AND usr.effective = 1
    LEFT JOIN suser sup ON sup.unique_user_id = item.unique_supervisor_id AND sup.effective = 1
    LEFT JOIN spermission perm ON perm.unique_permission_id = item.unique_permission_id AND perm.effective = 1
    LEFT JOIN saccount acct ON acct.unique_account_id = item.unique_account_id AND acct.effective = 1
    LEFT JOIN application app ON app.id =
      CASE
        WHEN perm.id IS NULL THEN acct.application
        ELSE perm.application
      END
    LEFT JOIN business_role brole ON brole.id = item.business_role_id
    LEFT JOIN role_policy role ON role.id = item.role_id
    LEFT JOIN data_source app_ds ON app_ds.id = app.id
    LEFT JOIN review_item_action action ON action.review_item_id = item.id AND action.action <> 'REASSIGNED'
    LEFT JOIN review_item_action_reason reason ON reason.id = action.reason_id
    LEFT JOIN reviewer_instance inst2 ON instance.id = inst2.review_instance_id AND inst2.type = 'RO'
    LEFT JOIN reviewer_instance inst3 ON instance.id = inst3.review_instance_id
    LEFT JOIN (SELECT DISTINCT
      item.id item_id,
      STUFF((SELECT ' | ' + CAST(multi_reviewers.id AS nvarchar(MAX)) FROM review_item item1
      LEFT JOIN (SELECT DISTINCT reviewer_instance_id, unique_user_id FROM reviewer_user_instance) rrui ON rrui.reviewer_instance_id = item1.curr_reviewer_instance_id
      LEFT JOIN suser multi_reviewers ON rrui.unique_user_id = multi_reviewers.unique_user_id AND multi_reviewers.effective = 1
      WHERE item1.id = item.id
      ORDER BY multi_reviewers.display_name FOR XML PATH('')), 1, 3, '') reviewer_ids,
      STUFF((SELECT  ' | ' + CAST(multi_reviewers.unique_user_id AS nvarchar(MAX)) FROM review_item item1
      LEFT JOIN (SELECT DISTINCT reviewer_instance_id, unique_user_id FROM reviewer_user_instance) rrui ON rrui.reviewer_instance_id = item1.curr_reviewer_instance_id
      LEFT JOIN suser multi_reviewers ON rrui.unique_user_id = multi_reviewers.unique_user_id AND multi_reviewers.effective = 1
      WHERE item1.id = item.id
      ORDER BY multi_reviewers.display_name FOR XML PATH('')),1,3,'') reviewer_unique_ids,
      STUFF((SELECT ' | ' + CAST(multi_reviewers.display_name AS nvarchar(MAX)) FROM review_item item1
      LEFT JOIN (SELECT DISTINCT reviewer_instance_id, unique_user_id FROM reviewer_user_instance) rrui ON rrui.reviewer_instance_id = item1.curr_reviewer_instance_id
      LEFT JOIN suser multi_reviewers ON rrui.unique_user_id = multi_reviewers.unique_user_id AND multi_reviewers.effective = 1
      WHERE item1.id = item.id
      ORDER BY multi_reviewers.display_name FOR XML PATH('')),1,3,'') reviewers,
      STUFF((SELECT ' | ' + CAST(substring(multi_reviewers.dn, 4, charindex(',', multi_reviewers.dn) - 4) AS nvarchar(MAX)) FROM review_item item1
      LEFT JOIN (SELECT DISTINCT reviewer_instance_id, unique_user_id FROM reviewer_user_instance) rrui ON rrui.reviewer_instance_id = item1.curr_reviewer_instance_id
      LEFT JOIN suser multi_reviewers ON rrui.unique_user_id = multi_reviewers.unique_user_id AND multi_reviewers.effective = 1
      WHERE item1.id = item.id
      ORDER BY multi_reviewers.display_name FOR XML PATH('')),1,3, '') user_cn
      FROM review_item item
      LEFT JOIN reviewer_user_instance rrui ON rrui.reviewer_instance_id = item.curr_reviewer_instance_id
      LEFT JOIN suser multi_reviewers ON rrui.unique_user_id = multi_reviewers.unique_user_id AND multi_reviewers.effective = 1
      GROUP BY item.id) multi_reviewer ON multi_reviewer.item_id = item.id
    LEFT JOIN suser final_reviewer ON item.final_cert_by_unique_user_id = final_reviewer.unique_user_id AND final_reviewer.effective = 1
    LEFT JOIN suser acting_reviewer ON action.acted_by_user_unique_id = acting_reviewer.unique_user_id AND acting_reviewer.effective = 1
    LEFT JOIN (SELECT
      ria.review_item_id,
      STUFF((SELECT ' | ' + usr.display_name FROM review_item_action ria_1
      LEFT JOIN reviewer_instance ri ON ri.id = ria_1.reviewer_instance_id
      LEFT JOIN suser usr ON ri.single_reviewer_unique_id = usr.unique_user_id AND usr.effective = 1
      WHERE ria_1.action = 'REASSIGNED' AND ria.review_item_id = ria_1.review_item_id
      GROUP BY ria_1.review_item_id, usr.display_name, ria_1.assigned ORDER BY ria_1.assigned FOR XML PATH('')),1,3,'') reviewers
      FROM  review_item_action ria
      LEFT JOIN reviewer_instance ri ON ri.id = ria.reviewer_instance_id
      LEFT JOIN suser usr ON ri.single_reviewer_unique_id = usr.unique_user_id AND usr.effective = 1
      WHERE ria.action = 'REASSIGNED'
      GROUP BY ria.review_item_id) reassigned_from_history ON reassigned_from_history.review_item_id = item.id
    LEFT JOIN (SELECT
      ria.review_item_id,
      STUFF((SELECT ' | ' + usr.display_name FROM review_item_action ria_1
      LEFT JOIN suser usr ON ria_1.reassigned_by_user_unique_id = usr.unique_user_id AND usr.effective = 1
      WHERE ria_1.action = 'REASSIGNED' AND ria.review_item_id = ria_1.review_item_id ORDER BY ria_1.assigned FOR XML PATH('')),1,3,'') reviewers,
      STUFF((SELECT ' | ' + COALESCE(reason.name, 'NULL_REASON') FROM review_item_action ria_1
      LEFT JOIN review_item_action_reason reason ON ria_1.reason_id = reason.id
      LEFT JOIN suser usr ON ria_1.reassigned_by_user_unique_id = usr.unique_user_id AND usr.effective = 1
      WHERE ria_1.action = 'REASSIGNED' AND ria.review_item_id = ria_1.review_item_id ORDER BY ria_1.assigned FOR XML PATH('')),1,3,'') reasons,
      STUFF((SELECT ' | ' + COALESCE(NULLIF(ria_1.reviewer_comment, ''), 'NULL_COMMENT')  FROM  review_item_action ria_1
      LEFT JOIN suser usr ON ria_1.reassigned_by_user_unique_id = usr.unique_user_id AND usr.effective = 1
      WHERE ria_1.action = 'REASSIGNED' AND ria.review_item_id = ria_1.review_item_id ORDER BY ria_1.assigned FOR XML PATH('')),1,3,'') rev_comments,
      STUFF((SELECT ' | ' + CAST(CONVERT(DATETIMEOFFSET(0), ria_1.assigned) AS nvarchar(MAX))  FROM  review_item_action ria_1
      LEFT JOIN suser usr ON ria_1.reassigned_by_user_unique_id = usr.unique_user_id AND usr.effective = 1
      WHERE ria_1.action = 'REASSIGNED' AND ria.review_item_id = ria_1.review_item_id ORDER BY ria_1.assigned FOR XML PATH('')),1,3,'') AS reassigned_date
      FROM  review_item_action ria
      LEFT JOIN review_item_action_reason reason ON ria.reason_id = reason.id
      LEFT JOIN suser usr ON ria.reassigned_by_user_unique_id = usr.unique_user_id AND usr.effective = 1
      WHERE ria.action = 'REASSIGNED'
      GROUP BY ria.review_item_id) reassigned_by_history ON reassigned_by_history.review_item_id = item.id
    LEFT JOIN shared_accounts_v shacc ON shacc.unique_account_id = acct.unique_account_id
    LEFT JOIN ( SELECT DISTINCT defn_1.name review_name,
      ri.id rid,
      STUFF((SELECT DISTINCT ' | ' + s.display_name FROM review_instance ri1
      JOIN review_definition defn_1 ON ri1.review_definition = defn_1.id
      LEFT JOIN reviewer_instance rev_owner ON ri1.id = rev_owner.review_instance_id AND rev_owner.type = 'RO'
      LEFT JOIN reviewer_user_instance rui ON rui.reviewer_instance_id = rev_owner.id
      LEFT JOIN suser s ON rui.unique_user_id = s.unique_user_id AND s.effective = 1
      WHERE ri1.ID = ri.id
      FOR XML PATH('')),1,3,'') review_owners
      FROM review_instance ri
      JOIN review_definition defn_1 ON ri.review_definition = defn_1.id
      LEFT JOIN reviewer_instance rev_owner ON ri.id = rev_owner.review_instance_id AND rev_owner.type = 'RO'
      LEFT JOIN reviewer_user_instance rui ON rui.reviewer_instance_id = rev_owner.id
      LEFT JOIN suser s ON rui.unique_user_id = s.unique_user_id AND s.effective = 1
      GROUP BY defn_1.name, ri.id) multi_owner ON multi_owner.rid = item.review_instance_id
    LEFT JOIN (SELECT
        esc_reviewers1.review_instance_id,
        STUFF((SELECT ' | ' + esc_reviewers.display_name FROM (SELECT DISTINCT
          u.display_name display_name,
          rri.review_instance_id
        FROM review_item ri
          JOIN reviewer_instance rri ON ri.review_instance_id = rri.review_instance_id AND rri.type = 'EO'
          JOIN reviewer_user_instance rriu ON rriu.reviewer_instance_id = rri.id
          JOIN suser u ON u.unique_user_id = rriu.unique_user_id AND u.effective = 1) esc_reviewers
        WHERE esc_reviewers.review_instance_id = esc_reviewers1.review_instance_id
        GROUP BY esc_reviewers.review_instance_id, esc_reviewers.display_name
        ORDER BY esc_reviewers.display_name FOR XML PATH('')),1,3,'') display_name
      FROM (SELECT DISTINCT
          u.display_name display_name,
          rri.review_instance_id
        FROM review_item ri
          JOIN reviewer_instance rri ON ri.review_instance_id = rri.review_instance_id AND rri.type = 'EO'
          JOIN reviewer_user_instance rriu ON rriu.reviewer_instance_id = rri.id
          JOIN suser u ON u.unique_user_id = rriu.unique_user_id AND u.effective = 1) esc_reviewers1
      GROUP BY esc_reviewers1.review_instance_id) esc_reviewer ON esc_reviewer.review_instance_id = item.review_instance_id
    LEFT JOIN ( SELECT review_item_id, action, change_item_status
      FROM change_request_item cri
      LEFT JOIN change_item_action cia ON cri.latest_action = cia.id
      WHERE cri.change_request_type LIKE 'REMOVE' + '%' AND action = 'FULFILLED' AND (change_item_status = 'VERIFIED' OR change_item_status = 'COMPLETED')) fulfillment ON item.id = fulfillment.review_item_id


  WHERE
    defn.deleted = 0
    AND (acting_reviewer.unique_user_id LIKE '%' + multi_reviewer.reviewer_unique_ids + '%' OR acting_reviewer.unique_user_id IS NULL
      AND (action.review_order =
      ( SELECT DISTINCT min(act.review_order) min
        FROM review_item_action act
        WHERE (act.action = 'PENDING' OR act.action = 'ESCALATE') AND act.review_item_id = item.id)
      OR action.acted_by_policy = 1)
      OR acting_reviewer.unique_user_id IS NOT NULL AND action.shared_queue = 1)
    AND (action.action <> 'PENDING' OR item.final_decision IS NULL)
GO

GRANT SELECT ON access_review_details_v TO ig_report_role
GO
