USE [igops]
GO

DROP VIEW IF EXISTS access_review_details_v
GO
DROP VIEW IF EXISTS access_review_summary_v
GO
DROP VIEW IF EXISTS business_role_hist_v
GO
DROP VIEW IF EXISTS reviewer_item_v
GO
DROP VIEW IF EXISTS role_parameter_v
GO

CREATE VIEW access_review_details_v AS
  SELECT DISTINCT
    item.id review_item_id,
    item.last_override_action_id last_override_action,
    instance.id review_instance_id,
    instance.status review_status,
    instance.start_date,
    instance.end_date,
    instance.valid_to_date,
    action.last_update,
    item.approved_time,
    instance.item_count,
    multi_owner.review_owners,
    multi_reviewer.reviewer_ids reviewer_id,
    CASE
      WHEN item.final_cert_by_unique_user_id IS NOT NULL THEN final_reviewer.display_name
      ELSE multi_reviewer.reviewers
    END reviewer_display_name,
    CASE
      WHEN item.final_cert_by_unique_user_id IS NOT NULL THEN  substring(final_reviewer.dn, 4,  charindex(',', final_reviewer.dn) - 4)
      ELSE multi_reviewer.user_cn
    END reviewer_cn,
    reassigned_by_history.reviewers reassigned_by_user_disp_name,
    reassigned_from_history.reviewers reassigned_from_user_disp_name,
    reassigned_by_history.reasons reassigned_reasons,
    reassigned_by_history.rev_comments reassigned_comments,
    reassigned_by_history.reassigned_date,
    defn.name review_name,
    defn.description,
    defn.review_type,
    defn.next_schedule_date,
    CASE
      WHEN item.item_type = 'ACCOUNT' AND usr.id IS NULL THEN shacc.suser
      ELSE usr.id
    END user_id,
    CASE
      WHEN item.item_type = 'ACCOUNT' AND usr.display_name IS NULL THEN shacc.display_name
      ELSE usr.display_name
    END user_display_name,
    CASE
      WHEN item.item_type = 'ACCOUNT' AND usr.display_name IS NULL THEN shacc.user_cn
      ELSE substring(usr.dn, 4, charindex(',', usr.dn) - 4)
    END user_cn,
    action.action reviewer_action,
    reason.name modify_reason_name,
    reason.id modify_reason_id,
    CASE
      WHEN fulfillment.review_item_id IS NOT NULL THEN 1
      ELSE 0
    END remove_completed,
    usr.unique_user_id,
    action.review_order,
    action.reviewer_comment,
    perm.id permission_id,
    perm.name permission_name,
    acct.id account_id,
    acct.account_id account_native_id,
    acct.name account_name,
    acct.relation_to_user_type account_relation,
    app.id application_id,
    app_ds.name application_name,
    item.final_decision,
    item.final_decision_time,
    role.id role_id,
    role.name role_name,
    item.item_type,
    brole.name brole_name,
    brole.id business_role_id,
    action.fulfillment_instructions,
    esc_reviewer.display_name esc_reviewer,
    sup.unique_user_id supervisor_id,
    sup.display_name supervisor_display_name
  FROM
    review_item item
    JOIN review_instance instance ON item.review_instance_id = instance.id AND instance.status NOT IN ('PREVIEW', 'PREVIEW_CANCELED', 'PREVIEW_STARTING')
     JOIN (SELECT
          review_instance.id inst_id,
          defn.id defn_id,
          CASE WHEN staged_defn.name IS NULL THEN defn.name ELSE staged_defn.name END AS name,
          CASE WHEN staged_defn.description IS NULL THEN defn.description ELSE staged_defn.description END AS description,
          CASE WHEN staged_defn.next_schedule_date IS NULL THEN defn.next_schedule_date ELSE staged_defn.next_schedule_date END AS next_schedule_date,
          defn.review_type,
          defn.deleted
        FROM review_definition defn
        JOIN review_instance ON defn.id = review_instance.review_definition
        LEFT JOIN review_definition staged_defn ON staged_defn.id = review_instance.staged_review_definition_id) defn ON instance.id = defn.inst_id
    LEFT JOIN suser usr ON usr.unique_user_id = item.unique_user_id AND usr.effective = 1
    LEFT JOIN suser sup ON sup.unique_user_id = item.unique_supervisor_id AND sup.effective = 1
    LEFT JOIN spermission perm ON perm.unique_permission_id = item.unique_permission_id AND perm.effective = 1
    LEFT JOIN saccount acct ON acct.unique_account_id = item.unique_account_id AND acct.effective = 1
    LEFT JOIN application app ON app.id =
      CASE
        WHEN perm.id IS NULL THEN acct.application
        ELSE perm.application
      END
    LEFT JOIN business_role brole ON brole.id = item.business_role_id
    LEFT JOIN role_policy role ON role.id = item.role_id
    LEFT JOIN data_source app_ds ON app_ds.id = app.id
    LEFT JOIN review_item_action action ON action.review_item_id = item.id AND action.action <> 'REASSIGNED'
    LEFT JOIN review_item_action_reason reason ON reason.id = action.reason_id
    LEFT JOIN reviewer_instance inst2 ON instance.id = inst2.review_instance_id AND inst2.type = 'RO'
    LEFT JOIN reviewer_instance inst3 ON instance.id = inst3.review_instance_id
    LEFT JOIN (SELECT DISTINCT
      item.id item_id,
      STUFF((SELECT ' | ' + CAST(multi_reviewers.id AS nvarchar(MAX)) FROM review_item item1
      LEFT JOIN (SELECT DISTINCT reviewer_instance_id, unique_user_id FROM reviewer_user_instance) rrui ON rrui.reviewer_instance_id = item1.curr_reviewer_instance_id
      LEFT JOIN suser multi_reviewers ON rrui.unique_user_id = multi_reviewers.unique_user_id AND multi_reviewers.effective = 1
      WHERE item1.id = item.id
      ORDER BY multi_reviewers.display_name FOR XML PATH('')), 1, 3, '') reviewer_ids,
      STUFF((SELECT  ' | ' + CAST(multi_reviewers.unique_user_id AS nvarchar(MAX)) FROM review_item item1
      LEFT JOIN (SELECT DISTINCT reviewer_instance_id, unique_user_id FROM reviewer_user_instance) rrui ON rrui.reviewer_instance_id = item1.curr_reviewer_instance_id
      LEFT JOIN suser multi_reviewers ON rrui.unique_user_id = multi_reviewers.unique_user_id AND multi_reviewers.effective = 1
      WHERE item1.id = item.id
      ORDER BY multi_reviewers.display_name FOR XML PATH('')),1,3,'') reviewer_unique_ids,
      STUFF((SELECT ' | ' + CAST(multi_reviewers.display_name AS nvarchar(MAX)) FROM review_item item1
      LEFT JOIN (SELECT DISTINCT reviewer_instance_id, unique_user_id FROM reviewer_user_instance) rrui ON rrui.reviewer_instance_id = item1.curr_reviewer_instance_id
      LEFT JOIN suser multi_reviewers ON rrui.unique_user_id = multi_reviewers.unique_user_id AND multi_reviewers.effective = 1
      WHERE item1.id = item.id
      ORDER BY multi_reviewers.display_name FOR XML PATH('')),1,3,'') reviewers,
      STUFF((SELECT ' | ' + CAST(substring(multi_reviewers.dn, 4, charindex(',', multi_reviewers.dn) - 4) AS nvarchar(MAX)) FROM review_item item1
      LEFT JOIN (SELECT DISTINCT reviewer_instance_id, unique_user_id FROM reviewer_user_instance) rrui ON rrui.reviewer_instance_id = item1.curr_reviewer_instance_id
      LEFT JOIN suser multi_reviewers ON rrui.unique_user_id = multi_reviewers.unique_user_id AND multi_reviewers.effective = 1
      WHERE item1.id = item.id
      ORDER BY multi_reviewers.display_name FOR XML PATH('')),1,3, '') user_cn
      FROM review_item item
      LEFT JOIN reviewer_user_instance rrui ON rrui.reviewer_instance_id = item.curr_reviewer_instance_id
      LEFT JOIN suser multi_reviewers ON rrui.unique_user_id = multi_reviewers.unique_user_id AND multi_reviewers.effective = 1
      GROUP BY item.id) multi_reviewer ON multi_reviewer.item_id = item.id
    LEFT JOIN suser final_reviewer ON item.final_cert_by_unique_user_id = final_reviewer.unique_user_id AND final_reviewer.effective = 1
    LEFT JOIN suser acting_reviewer ON action.acted_by_user_unique_id = acting_reviewer.unique_user_id AND acting_reviewer.effective = 1
    LEFT JOIN (SELECT
      ria.review_item_id,
      STUFF((SELECT ' | ' + usr.display_name FROM review_item_action ria_1
      LEFT JOIN reviewer_instance ri ON ri.id = ria_1.reviewer_instance_id
      LEFT JOIN suser usr ON ri.single_reviewer_unique_id = usr.unique_user_id AND usr.effective = 1
      WHERE ria_1.action = 'REASSIGNED' AND ria.review_item_id = ria_1.review_item_id
      GROUP BY ria_1.review_item_id, usr.display_name, ria_1.assigned ORDER BY ria_1.assigned FOR XML PATH('')),1,3,'') reviewers
      FROM  review_item_action ria
      LEFT JOIN reviewer_instance ri ON ri.id = ria.reviewer_instance_id
      LEFT JOIN suser usr ON ri.single_reviewer_unique_id = usr.unique_user_id AND usr.effective = 1
      WHERE ria.action = 'REASSIGNED'
      GROUP BY ria.review_item_id) reassigned_from_history ON reassigned_from_history.review_item_id = item.id
    LEFT JOIN (SELECT
      ria.review_item_id,
      STUFF((SELECT ' | ' + usr.display_name FROM review_item_action ria_1
      LEFT JOIN suser usr ON ria_1.reassigned_by_user_unique_id = usr.unique_user_id AND usr.effective = 1
      WHERE ria_1.action = 'REASSIGNED' AND ria.review_item_id = ria_1.review_item_id ORDER BY ria_1.assigned FOR XML PATH('')),1,3,'') reviewers,
      STUFF((SELECT ' | ' + COALESCE(reason.name, 'NULL_REASON') FROM review_item_action ria_1
      LEFT JOIN review_item_action_reason reason ON ria_1.reason_id = reason.id
      LEFT JOIN suser usr ON ria_1.reassigned_by_user_unique_id = usr.unique_user_id AND usr.effective = 1
      WHERE ria_1.action = 'REASSIGNED' AND ria.review_item_id = ria_1.review_item_id ORDER BY ria_1.assigned FOR XML PATH('')),1,3,'') reasons,
      STUFF((SELECT ' | ' + COALESCE(NULLIF(ria_1.reviewer_comment, ''), 'NULL_COMMENT')  FROM  review_item_action ria_1
      LEFT JOIN suser usr ON ria_1.reassigned_by_user_unique_id = usr.unique_user_id AND usr.effective = 1
      WHERE ria_1.action = 'REASSIGNED' AND ria.review_item_id = ria_1.review_item_id ORDER BY ria_1.assigned FOR XML PATH('')),1,3,'') rev_comments,
      STUFF((SELECT ' | ' + CAST(CONVERT(DATETIMEOFFSET(0), ria_1.assigned) AS nvarchar(MAX))  FROM  review_item_action ria_1
      LEFT JOIN suser usr ON ria_1.reassigned_by_user_unique_id = usr.unique_user_id AND usr.effective = 1
      WHERE ria_1.action = 'REASSIGNED' AND ria.review_item_id = ria_1.review_item_id ORDER BY ria_1.assigned FOR XML PATH('')),1,3,'') AS reassigned_date
      FROM  review_item_action ria
      LEFT JOIN review_item_action_reason reason ON ria.reason_id = reason.id
      LEFT JOIN suser usr ON ria.reassigned_by_user_unique_id = usr.unique_user_id AND usr.effective = 1
      WHERE ria.action = 'REASSIGNED'
      GROUP BY ria.review_item_id) reassigned_by_history ON reassigned_by_history.review_item_id = item.id
    LEFT JOIN shared_accounts_v shacc ON shacc.unique_account_id = acct.unique_account_id
    LEFT JOIN ( SELECT DISTINCT defn_1.name review_name,
      ri.id rid,
      STUFF((SELECT DISTINCT ' | ' + s.display_name FROM review_instance ri1
      JOIN review_definition defn_1 ON ri1.review_definition = defn_1.id
      LEFT JOIN reviewer_instance rev_owner ON ri1.id = rev_owner.review_instance_id AND rev_owner.type = 'RO'
      LEFT JOIN reviewer_user_instance rui ON rui.reviewer_instance_id = rev_owner.id
      LEFT JOIN suser s ON rui.unique_user_id = s.unique_user_id AND s.effective = 1
      WHERE ri1.ID = ri.id
      FOR XML PATH('')),1,3,'') review_owners
      FROM review_instance ri
      JOIN review_definition defn_1 ON ri.review_definition = defn_1.id
      LEFT JOIN reviewer_instance rev_owner ON ri.id = rev_owner.review_instance_id AND rev_owner.type = 'RO'
      LEFT JOIN reviewer_user_instance rui ON rui.reviewer_instance_id = rev_owner.id
      LEFT JOIN suser s ON rui.unique_user_id = s.unique_user_id AND s.effective = 1
      GROUP BY defn_1.name, ri.id) multi_owner ON multi_owner.rid = item.review_instance_id
    LEFT JOIN (SELECT
        esc_reviewers1.review_instance_id,
        STUFF((SELECT ' | ' + esc_reviewers.display_name FROM (SELECT DISTINCT
          u.display_name display_name,
          rri.review_instance_id
        FROM review_item ri
          JOIN reviewer_instance rri ON ri.review_instance_id = rri.review_instance_id AND rri.type = 'EO'
          JOIN reviewer_user_instance rriu ON rriu.reviewer_instance_id = rri.id
          JOIN suser u ON u.unique_user_id = rriu.unique_user_id AND u.effective = 1) esc_reviewers
        WHERE esc_reviewers.review_instance_id = esc_reviewers1.review_instance_id
        GROUP BY esc_reviewers.review_instance_id, esc_reviewers.display_name
        ORDER BY esc_reviewers.display_name FOR XML PATH('')),1,3,'') display_name
      FROM (SELECT DISTINCT
          u.display_name display_name,
          rri.review_instance_id
        FROM review_item ri
          JOIN reviewer_instance rri ON ri.review_instance_id = rri.review_instance_id AND rri.type = 'EO'
          JOIN reviewer_user_instance rriu ON rriu.reviewer_instance_id = rri.id
          JOIN suser u ON u.unique_user_id = rriu.unique_user_id AND u.effective = 1) esc_reviewers1
      GROUP BY esc_reviewers1.review_instance_id) esc_reviewer ON esc_reviewer.review_instance_id = item.review_instance_id
    LEFT JOIN ( SELECT review_item_id, action, change_item_status
      FROM change_request_item cri
      LEFT JOIN change_item_action cia ON cri.latest_action = cia.id
      WHERE cri.change_request_type LIKE 'REMOVE' + '%' AND action = 'FULFILLED' AND (change_item_status = 'VERIFIED' OR change_item_status = 'COMPLETED')) fulfillment ON item.id = fulfillment.review_item_id


  WHERE
    defn.deleted = 0
    AND (acting_reviewer.unique_user_id LIKE '%' + multi_reviewer.reviewer_unique_ids + '%' OR acting_reviewer.unique_user_id IS NULL
      AND (action.review_order =
      ( SELECT DISTINCT min(act.review_order) min
        FROM review_item_action act
        WHERE (act.action = 'PENDING' OR act.action = 'ESCALATE') AND act.review_item_id = item.id)
      OR action.acted_by_policy = 1)
      OR acting_reviewer.unique_user_id IS NOT NULL AND action.shared_queue = 1)
    AND (action.action <> 'PENDING' OR item.final_decision IS NULL)
GO

CREATE VIEW access_review_summary_v AS
  SELECT 
    instance.id AS instance_id,
    defn.name,
    defn.description,
    defn.review_type,
    defn.next_schedule_date,
    defn.unique_review_def_id,
    multi_owner.review_owners AS review_inst_owner_display_name,
    instance.status,
    instance.status_comment,
    instance.start_date,
    instance.expected_end_date,
    instance.end_date,
    instance.valid_to_date,
    instance.canceled_by,
    instance.task_count,
    instance.task_complete_count,
  CASE
      WHEN instance.task_complete_count = 0 THEN 0
      ELSE round(((instance.task_complete_count) / CAST(instance.task_count AS float) * 100), 1)
    END AS percent_complete,
    canceler.display_name AS canceled_by_name,
    instance.item_count,
    instance.item_complete_count,
    instances.reviewer_count,
    ( SELECT decisions.decision_count
           FROM ( SELECT inst.id AS instance,
                    it.final_decision AS decision,
                    count(it.final_decision) AS decision_count
                   FROM review_instance inst
                     LEFT JOIN review_item it ON it.review_instance_id = inst.id
                  GROUP BY inst.id, it.final_decision) decisions
          WHERE decisions.instance = instance.id AND decisions.decision = 'ASSIGN_USER') AS assign_count,
    ( SELECT decisions.decision_count
           FROM ( SELECT inst.id AS instance,
                    it.final_decision AS decision,
                    count(it.final_decision) AS decision_count
                   FROM review_instance inst
                     LEFT JOIN review_item it ON it.review_instance_id = inst.id
                  GROUP BY inst.id, it.final_decision) decisions
          WHERE decisions.instance = instance.id AND decisions.decision = 'KEEP') AS keep_count,
    ( SELECT decisions.decision_count
           FROM ( SELECT inst.id AS instance,
                    it.final_decision AS decision,
                    count(it.final_decision) AS decision_count
                   FROM review_instance inst
                     LEFT JOIN review_item it ON it.review_instance_id = inst.id
                  GROUP BY inst.id, it.final_decision) decisions
          WHERE decisions.instance = instance.id AND decisions.decision = 'REMOVE') AS remove_count,
    ( SELECT decisions.decision_count
           FROM ( SELECT inst.id AS instance,
                    it.final_decision AS decision,
                    count(it.final_decision) AS decision_count
                   FROM review_instance inst
                     LEFT JOIN review_item it ON it.review_instance_id = inst.id
                  GROUP BY inst.id, it.final_decision) decisions
          WHERE decisions.instance = instance.id AND decisions.decision IN ('MODIFY_ENTITY', 'MODIFY_ACCESS', 'MODIFY_SUPERVISOR')) AS modify_count
  FROM review_instance instance
    JOIN (SELECT
        review_instance.id inst_id,
        defn.id defn_id,
        CASE WHEN staged_defn.name IS NULL THEN defn.name ELSE staged_defn.name END AS name,
        CASE WHEN staged_defn.description IS NULL THEN defn.description ELSE staged_defn.description END description,
        defn.review_type,
        defn.next_schedule_date,
        defn.deleted,
        defn.unique_review_def_id
      FROM review_definition defn
        JOIN review_instance ON defn.id = review_instance.review_definition
        LEFT JOIN review_definition staged_defn ON staged_defn.id = review_instance.staged_review_definition_id) defn ON defn.inst_id = instance.id
    JOIN ( SELECT inst.id AS instance,
            count(DISTINCT rui.unique_user_id) AS reviewer_count
           FROM review_instance inst
             LEFT JOIN reviewer_instance ri ON ri.review_instance_id = inst.id
             LEFT JOIN reviewer_user_instance rui ON rui.reviewer_instance_id = ri.id
             WHERE ri.task_count > 0
          GROUP BY inst.id) instances ON instances.instance = instance.id
    LEFT JOIN suser canceler ON instance.canceled_by = canceler.unique_user_id AND canceler.effective = 1
    LEFT JOIN ( SELECT DISTINCT defn_1.name review_name,
        ri.id rid,
        STUFF((SELECT DISTINCT ' | ' + s.display_name FROM review_instance ri1
          JOIN review_definition defn_1 ON ri1.review_definition = defn_1.id
          LEFT JOIN reviewer_instance rev_owner ON ri1.id = rev_owner.review_instance_id AND rev_owner.type = 'RO'
          LEFT JOIN reviewer_user_instance rui ON rui.reviewer_instance_id = rev_owner.id
          LEFT JOIN suser s ON rui.unique_user_id = s.unique_user_id AND s.effective = 1 
      WHERE ri1.ID = ri.id
        FOR XML PATH('')),1,3,'') review_owners
        FROM review_instance ri
          JOIN review_definition defn_1 ON ri.review_definition = defn_1.id
          LEFT JOIN reviewer_instance rev_owner ON ri.id = rev_owner.review_instance_id AND rev_owner.type = 'RO'
          LEFT JOIN reviewer_user_instance rui ON rui.reviewer_instance_id = rev_owner.id
          LEFT JOIN suser s ON rui.unique_user_id = s.unique_user_id AND s.effective = 1
        GROUP BY defn_1.name, ri.id) multi_owner ON multi_owner.rid = instance.id
  WHERE defn.deleted = 0 AND instance.status NOT IN ('PREVIEW', 'PREVIEW_CANCELED', 'PREVIEW_STARTING')
GO

CREATE VIEW business_role_hist_v AS
  SELECT b_role.id AS business_role_id,
    b_role.unique_id AS business_role_unique_id,
    b_role.name,
    b_role.description,
    b_role.grace_period,
    b_role.risk,
    b_role.creation_time,
    b_role.update_time,
    b_role.approval_time,
    b_role.publish_time,
    user_owners.user_owners AS user_owners,
    group_owners.group_owners AS group_owners,
    b_role.approval_state,
    su1.display_name AS created_by,
    su2.display_name AS published_by,
    su3.display_name AS changed_by,
    su4.display_name AS approved_by,
    user_managers.user_managers AS user_managers,
    group_managers.group_managers AS group_managers,
    su5.display_name AS user_fulfiller,
    sgroup.name AS group_fulfiller,
    b_role.approval_note,
    CASE WHEN b_role.approval_policy IS NULL THEN (SELECT name FROM approval_policy WHERE global = 1 AND deleted = 0)
    ELSE ap.name END AS policy_name,
    
    CASE WHEN b_role.approval_policy IS NULL THEN (SELECT implicit FROM approval_policy WHERE global = 1 AND deleted = 0)
    ELSE ap.implicit END AS approval_not_required, 
    
    CASE WHEN b_role.approval_policy IS NULL THEN (SELECT description FROM approval_policy WHERE global = 1 AND deleted = 0)
    ELSE ap.description END AS policy_description, 
    
    CASE WHEN b_role.approval_policy IS NULL THEN (SELECT creation_time FROM approval_policy WHERE global = 1 AND deleted = 0)
    ELSE ap.creation_time END AS policy_creation_time,
    
    CASE WHEN b_role.approval_policy IS NULL THEN (SELECT update_time FROM approval_policy WHERE global = 1 AND deleted = 0)
    ELSE ap.update_time END AS last_policy_update_time,
    pol_users.policy_users AS policy_users,
    pol_groups.policy_groups AS policy_groups,
    br_crit.criteria AS membership_criteria,
    incl_users.included_users AS included_users,
    incl_groups.included_groups AS included_groups,
    excl_users.excluded_users AS excluded_users,
    excl_groups.excluded_groups AS excluded_groups,
    auth_perm.auth_permission,
    auth_tech_role.auth_tech_role AS auth_tech_role,
    auth_app.auth_application AS auth_application,
    incl_brole.bnames AS included_broles
   FROM business_role b_role
     LEFT JOIN suser su1 ON b_role.created_by = su1.unique_user_id AND su1.display_name IS NOT NULL AND su1.deleted = 0 AND su1.effective = 1
     LEFT JOIN suser su2 ON b_role.published_by = su2.unique_user_id AND su2.display_name IS NOT NULL AND su2.deleted = 0 AND su2.effective = 1
     LEFT JOIN suser su3 ON b_role.changed_by = su3.unique_user_id AND su3.display_name IS NOT NULL AND su3.deleted = 0 AND su3.effective = 1
     LEFT JOIN suser su4 ON b_role.approved_by = su4.unique_user_id AND su4.display_name IS NOT NULL AND su4.deleted = 0 AND su4.effective = 1
     JOIN auth_scope ON auth_scope.unique_business_role_id = b_role.unique_id
     LEFT JOIN auth_role_mapping ar_map ON ar_map.auth_scope = auth_scope.id AND ar_map.auth_role = 29
     LEFT JOIN ( SELECT DISTINCT auth_scope.unique_business_role_id,
  STUFF( ( SELECT ' | ' + CAST(manager_names.display_name AS nvarchar(MAX))
    FROM ( SELECT DISTINCT display_name, su.unique_user_id, auth_scope, auth_role
      FROM auth_role_mapping ar_map
      JOIN suser su ON ar_map.unique_user_id = su.unique_user_id AND su.display_name IS NOT NULL AND su.deleted = 0 AND su.effective = 1) manager_names
      WHERE  manager_names.auth_scope = auth_scope.id AND manager_names.unique_user_id IS NOT NULL AND manager_names.auth_role = 27
      ORDER BY manager_names.display_name FOR XML PATH('')), 1, 3, '') AS user_managers
  FROM auth_scope) user_managers ON user_managers.unique_business_role_id = b_role.unique_id
     LEFT JOIN ( SELECT DISTINCT auth_scope_1.unique_business_role_id,
  STUFF( ( SELECT ' | ' + CAST(group1_names.name AS nvarchar(MAX))
    FROM ( SELECT DISTINCT name, sgroup.unique_group_id, auth_scope, auth_role
      FROM auth_role_mapping ar_map_1
      JOIN sgroup ON ar_map_1.unique_group_id = sgroup.unique_group_id AND sgroup.name IS NOT NULL) group1_names
      WHERE group1_names.auth_scope = auth_scope_1.id AND group1_names.unique_group_id IS NOT NULL AND group1_names.auth_role = 27
      ORDER BY group1_names.name FOR XML PATH('')), 1, 3, '') AS group_managers
  FROM auth_scope auth_scope_1) group_managers ON group_managers.unique_business_role_id = b_role.unique_id
     LEFT JOIN suser su5 ON ar_map.unique_user_id = su5.unique_user_id AND su5.display_name IS NOT NULL AND su5.deleted = 0 AND su5.effective = 1
     LEFT JOIN sgroup ON ar_map.unique_group_id = sgroup.unique_group_id AND sgroup.name IS NOT NULL AND sgroup.deleted = 0 AND sgroup.effective = 1
     LEFT JOIN approval_policy ap ON b_role.approval_policy = ap.id
     LEFT JOIN business_role_memb_crit br_crit ON b_role.id = br_crit.business_role_id
LEFT JOIN ( SELECT incl_brole2.business_role_id,
          STUFF((SELECT ' | ' + CAST(incl_brole1.name AS nvarchar(MAX)) FROM (SELECT DISTINCT name, business_role_id from business_role_incl_brole) incl_brole1
          WHERE incl_brole1.business_role_id = incl_brole2.business_role_id
          ORDER BY incl_brole1.name FOR XML PATH('')), 1, 3, '') AS bnames
          FROM business_role_incl_brole incl_brole2
          GROUP BY incl_brole2.business_role_id) incl_brole ON incl_brole.business_role_id = b_role.id 

LEFT JOIN ( SELECT DISTINCT br_owner_1.business_role_id,
          STUFF((SELECT ' | ' + CAST(owner_names.display_name AS nvarchar(MAX)) FROM (SELECT DISTINCT display_name, business_role_id from business_role_owner br_owner_1 JOIN suser su ON br_owner_1.unique_owner_id = su.unique_user_id AND su.display_name IS NOT NULL AND su.deleted = 0 AND su.effective = 1) owner_names
                WHERE  owner_names.business_role_id = br_owner_1.business_role_id
                ORDER BY owner_names.display_name FOR XML PATH('')), 1, 3, '') AS user_owners
          FROM business_role_owner br_owner_1
        GROUP BY br_owner_1.business_role_id) user_owners ON user_owners.business_role_id = b_role.id

LEFT JOIN ( SELECT DISTINCT br_owner_1.business_role_id,
          STUFF((SELECT ' | ' + CAST(group_names.name AS nvarchar(MAX)) FROM (SELECT DISTINCT name, business_role_id from business_role_owner br_owner_1 JOIN sgroup sgroup_1 ON br_owner_1.unique_owner_id = sgroup_1.unique_group_id AND sgroup_1.name IS NOT NULL AND sgroup_1.deleted = 0 AND sgroup_1.effective = 1) group_names
                WHERE  group_names.business_role_id = br_owner_1.business_role_id
                ORDER BY group_names.name FOR XML PATH('')), 1, 3, '') AS group_owners
          FROM business_role_owner br_owner_1
        GROUP BY br_owner_1.business_role_id) group_owners ON group_owners.business_role_id = b_role.id

LEFT JOIN ( SELECT DISTINCT apmemb.approval_policy_id,
          STUFF((SELECT ' | ' + CAST(pol_user_names.display_name AS nvarchar(MAX)) FROM (SELECT DISTINCT display_name, approval_policy_id from approval_policy_memb apmemb JOIN suser su ON apmemb.unique_member_id = su.unique_user_id AND su.display_name IS NOT NULL AND su.deleted = 0 AND su.effective = 1) pol_user_names
                WHERE  pol_user_names.approval_policy_id = apmemb.approval_policy_id
                ORDER BY pol_user_names.display_name FOR XML PATH('')), 1, 3, '') AS policy_users
          FROM approval_policy_memb apmemb
        GROUP BY apmemb.approval_policy_id) pol_users ON pol_users.approval_policy_id = b_role.approval_policy


LEFT JOIN ( SELECT DISTINCT apmemb.approval_policy_id,
          STUFF((SELECT ' | ' + CAST(pol_groupname.name AS nvarchar(MAX)) FROM (SELECT DISTINCT name, approval_policy_id from approval_policy_memb apmemb JOIN sgroup sgroup_1 ON apmemb.unique_member_id = sgroup_1.unique_group_id AND sgroup_1.name IS NOT NULL AND sgroup_1.deleted = 0 AND sgroup_1.effective = 1) pol_groupname
                WHERE  pol_groupname.approval_policy_id = apmemb.approval_policy_id
                ORDER BY pol_groupname.name FOR XML PATH('')), 1, 3, '') AS policy_groups
          FROM approval_policy_memb apmemb
        GROUP BY apmemb.approval_policy_id) pol_groups ON pol_groups.approval_policy_id = b_role.approval_policy


LEFT JOIN ( SELECT DISTINCT br_incl_mem.business_role_id,
          STUFF((SELECT ' | ' + CAST(incl_users_names.display_name AS nvarchar(MAX)) FROM (SELECT DISTINCT display_name, business_role_id from business_role_incl_memb br_incl_mem JOIN suser su ON br_incl_mem.unique_member_id = su.unique_user_id AND su.display_name IS NOT NULL AND su.deleted = 0 AND su.effective = 1) incl_users_names
                WHERE  incl_users_names.business_role_id = br_incl_mem.business_role_id
                ORDER BY incl_users_names.display_name FOR XML PATH('')), 1, 3, '') AS included_users
          FROM business_role_incl_memb br_incl_mem
         GROUP BY br_incl_mem.business_role_id) incl_users ON incl_users.business_role_id = b_role.id


LEFT JOIN ( SELECT DISTINCT br_incl_mem.business_role_id,
          STUFF((SELECT ' | ' + CAST(incl_groupnames.name AS nvarchar(MAX)) FROM (SELECT DISTINCT sgroup_1.name, business_role_id from business_role_incl_memb br_incl_mem JOIN sgroup sgroup_1 ON br_incl_mem.unique_member_id = sgroup_1.unique_group_id AND sgroup_1.name IS NOT NULL AND sgroup_1.deleted = 0 AND sgroup_1.effective = 1) incl_groupnames
                WHERE  incl_groupnames.business_role_id = br_incl_mem.business_role_id
                ORDER BY incl_groupnames.name FOR XML PATH('')), 1, 3, '') AS included_groups
          FROM business_role_incl_memb br_incl_mem
        GROUP BY br_incl_mem.business_role_id) incl_groups ON incl_groups.business_role_id = b_role.id

LEFT JOIN ( SELECT DISTINCT br_excl_mem.business_role_id,
          STUFF((SELECT ' | ' + CAST(excl_user_names.display_name AS nvarchar(MAX)) FROM (SELECT DISTINCT display_name, business_role_id from business_role_excl_memb br_excl_mem JOIN suser ON br_excl_mem.unique_member_id = suser.unique_user_id AND suser.display_name IS NOT NULL AND suser.deleted = 0 AND suser.effective = 1) excl_user_names
                WHERE  excl_user_names.business_role_id = br_excl_mem.business_role_id
                ORDER BY excl_user_names.display_name FOR XML PATH('')), 1, 3, '') AS excluded_users
          FROM business_role_excl_memb br_excl_mem
        GROUP BY br_excl_mem.business_role_id) excl_users ON excl_users.business_role_id = b_role.id


LEFT JOIN ( SELECT DISTINCT br_excl_mem.business_role_id,
          STUFF((SELECT ' | ' + CAST(excl_group_names.name AS nvarchar(MAX)) FROM (SELECT DISTINCT sgroup_1.name, business_role_id from business_role_excl_memb br_excl_mem JOIN sgroup sgroup_1 ON br_excl_mem.unique_member_id = sgroup_1.unique_group_id AND sgroup_1.name IS NOT NULL AND sgroup_1.deleted = 0 AND sgroup_1.effective = 1) excl_group_names
                WHERE  excl_group_names.business_role_id = br_excl_mem.business_role_id
                ORDER BY excl_group_names.name FOR XML PATH('')), 1, 3, '') AS excluded_groups
          FROM business_role_excl_memb br_excl_mem
        GROUP BY br_excl_mem.business_role_id) excl_groups ON excl_groups.business_role_id = b_role.id


LEFT JOIN ( SELECT DISTINCT br_incl_item.business_role_id,
          STUFF((SELECT ' | ' + CAST(authperm_names.name AS nvarchar(MAX)) FROM (SELECT DISTINCT spermission.name, business_role_id from business_role_incl_item br_incl_item JOIN spermission ON br_incl_item.unique_item_id = spermission.unique_permission_id AND spermission.name IS NOT NULL AND spermission.deleted = 0 AND spermission.effective = 1) authperm_names
                WHERE  authperm_names.business_role_id = br_incl_item.business_role_id
                ORDER BY authperm_names.name FOR XML PATH('')), 1, 3, '') AS auth_permission
          FROM business_role_incl_item br_incl_item
        GROUP BY br_incl_item.business_role_id) auth_perm ON auth_perm.business_role_id = b_role.id

LEFT JOIN ( SELECT DISTINCT br_incl_item.business_role_id,
          STUFF((SELECT ' | ' + CAST(authtechrole_names.name AS nvarchar(MAX)) FROM (SELECT DISTINCT role_policy.name, business_role_id from business_role_incl_item br_incl_item JOIN role_policy ON br_incl_item.unique_item_id = role_policy.unique_policy_id AND role_policy.deleted = 0) authtechrole_names
                WHERE  authtechrole_names.business_role_id = br_incl_item.business_role_id
                ORDER BY authtechrole_names.name FOR XML PATH('')), 1, 3, '') AS auth_tech_role
          FROM business_role_incl_item br_incl_item
        GROUP BY br_incl_item.business_role_id) auth_tech_role ON auth_tech_role.business_role_id = b_role.id

LEFT JOIN ( SELECT DISTINCT br_incl_item.business_role_id,
          STUFF((SELECT ' | ' + CAST(authapp_names.name AS nvarchar(MAX)) FROM (SELECT DISTINCT data_source.name, business_role_id from business_role_incl_item br_incl_item JOIN data_source ON br_incl_item.unique_item_id = CAST(data_source.id AS nvarchar(MAX)) AND data_source.deleted = 0) authapp_names
                WHERE  authapp_names.business_role_id = br_incl_item.business_role_id
                ORDER BY authapp_names.name FOR XML PATH('')), 1, 3, '') AS auth_application
          FROM business_role_incl_item br_incl_item
        GROUP BY br_incl_item.business_role_id) auth_app ON auth_app.business_role_id = b_role.id


  WHERE b_role.approval_state = 'ARCHIVE' OR b_role.approval_state = 'PUBLISHED'
GO

CREATE VIEW reviewer_item_v AS
  SELECT item.id AS item_id,
        CASE
            WHEN rui.unique_user_id IS NULL THEN item.final_cert_by_unique_user_id
            ELSE rui.unique_user_id
        END AS unique_user_id,
    	defn.review_type
  FROM review_item item
     LEFT JOIN reviewer_user_instance rui ON rui.reviewer_instance_id = item.curr_reviewer_instance_id AND item.final_cert_by_unique_user_id IS NULL
     JOIN review_instance instance ON item.review_instance_id = instance.id
     JOIN review_definition defn ON instance.review_definition = defn.id
GO

CREATE VIEW role_parameter_v AS
  SELECT DISTINCT 
    role.id AS role_id,
    role.name AS role_name,
    role.unique_policy_id
  FROM role_policy role
  WHERE EXISTS ( 
    SELECT 1
    FROM review_item item
    WHERE item.role_id = role.id)
GO

GRANT SELECT ON access_review_details_v TO ig_report_role
GO
GRANT SELECT ON access_review_summary_v TO ig_report_role
GO
GRANT SELECT ON business_role_hist_v TO ig_report_role
GO
GRANT SELECT ON reviewer_item_v TO ig_report_role
GO
GRANT SELECT ON role_parameter_v TO ig_report_role
GO