BEGIN
  FOR k IN (SELECT null FROM all_synonyms WHERE owner = 'PUBLIC' and synonym_name = 'BUSINESS_ROLE_HIST_V') LOOP
    EXECUTE IMMEDIATE 'DROP PUBLIC SYNONYM BUSINESS_ROLE_HIST_V';
  END LOOP;
   FOR i IN (SELECT null FROM all_views WHERE owner = user and view_name = 'BUSINESS_ROLE_HIST_V') LOOP
    EXECUTE IMMEDIATE 'DROP VIEW BUSINESS_ROLE_HIST_V';
  END LOOP;
END;
/

CREATE VIEW business_role_hist_v AS
    SELECT b_role.id AS business_role_id,
    b_role.unique_id AS business_role_unique_id,
    b_role.name,
    b_role.description,
    b_role.grace_period,
    b_role.risk,
    b_role.creation_time,
    b_role.update_time,
    b_role.approval_time,
    b_role.publish_time,
    COALESCE(user_owners.user_owners, '') AS user_owners,
    COALESCE(group_owners.group_owners, '') AS group_owners,
    b_role.approval_state,
    su1.display_name AS created_by,
    su2.display_name AS published_by,
    su3.display_name AS changed_by,
    su4.display_name AS approved_by,
    COALESCE(user_manage.user_managers, '') AS user_managers,
    COALESCE(group_manage.group_managers, '') AS group_managers,
    su5.display_name AS user_fulfiller,
    sgroup.name AS group_fulfiller,
    b_role.approval_note,
    CASE WHEN b_role.approval_policy IS NULL THEN (SELECT name FROM approval_policy WHERE global = 1 AND deleted = 0)
    ELSE ap.name END AS policy_name,
    
    CASE WHEN b_role.approval_policy IS NULL THEN (SELECT implicit FROM approval_policy WHERE global = 1 AND deleted = 0)
    ELSE ap.implicit END AS approval_not_required, 
    
    CASE WHEN b_role.approval_policy IS NULL THEN (SELECT description FROM approval_policy WHERE global = 1 AND deleted = 0)
    ELSE ap.description END AS policy_description, 
    
    CASE WHEN b_role.approval_policy IS NULL THEN (SELECT creation_time FROM approval_policy WHERE global = 1 AND deleted = 0)
    ELSE ap.creation_time END AS policy_creation_time,
    
    CASE WHEN b_role.approval_policy IS NULL THEN (SELECT update_time FROM approval_policy WHERE global = 1 AND deleted = 0)
    ELSE ap.update_time END AS last_policy_update_time,
    COALESCE(pol_users.policy_users, '') AS policy_users,
    COALESCE(pol_groups.policy_groups, '') AS policy_groups,
    br_crit.criteria AS membership_criteria,
    COALESCE(incl_users.included_users, '') AS included_users,
    COALESCE(incl_groups.included_groups, '') AS included_groups,
    COALESCE(excl_users.excluded_users, '') AS excluded_users,
    COALESCE(excl_groups.excluded_groups, '') AS excluded_groups,
    auth_perm.auth_permission,
    COALESCE(auth_tech_role.auth_tech_role, '') AS auth_tech_role,
    COALESCE(auth_app.auth_application, '') AS auth_application,
    incl_brole.bnames AS included_broles
   FROM business_role b_role
     LEFT JOIN suser su1 ON b_role.created_by = su1.unique_user_id AND su1.display_name IS NOT NULL AND su1.deleted = 0 AND su1.effective = 1
     LEFT JOIN suser su2 ON b_role.published_by = su2.unique_user_id AND su2.display_name IS NOT NULL AND su2.deleted = 0 AND su2.effective = 1
     LEFT JOIN suser su3 ON b_role.changed_by = su3.unique_user_id AND su3.display_name IS NOT NULL AND su3.deleted = 0 AND su3.effective = 1
     LEFT JOIN suser su4 ON b_role.approved_by = su4.unique_user_id AND su4.display_name IS NOT NULL AND su4.deleted = 0 AND su4.effective = 1
     JOIN auth_scope ON auth_scope.unique_business_role_id = b_role.unique_id
     LEFT JOIN auth_role_mapping ar_map ON ar_map.auth_scope = auth_scope.id AND ar_map.auth_role = 29
     LEFT JOIN ( SELECT auth_scope.unique_business_role_id, 
        LISTAGG(suser.display_name , ' | ' ON OVERFLOW TRUNCATE) WITHIN GROUP(ORDER BY suser.display_name) AS user_managers
        FROM auth_role_mapping ar_map
        JOIN (SELECT DISTINCT unique_user_id, display_name, effective, deleted from suser) suser ON ar_map.unique_user_id = suser.unique_user_id AND suser.display_name IS NOT NULL AND suser.deleted = 0 AND suser.effective = 1
        JOIN auth_scope ON ar_map.auth_scope = auth_scope.id
        WHERE ar_map.unique_user_id IS NOT NULL AND ar_map.auth_role = 27
        GROUP BY auth_scope.unique_business_role_id) user_manage ON user_manage.unique_business_role_id = b_role.unique_id
     LEFT JOIN ( SELECT auth_scope_1.unique_business_role_id,
          LISTAGG(sgroup_1.name, ' | ' ON OVERFLOW TRUNCATE) WITHIN GROUP (ORDER BY sgroup_1.name) AS group_managers
          FROM auth_role_mapping ar_map_1
          JOIN (SELECT DISTINCT unique_group_id, name from sgroup) sgroup_1 ON ar_map_1.unique_group_id = sgroup_1.unique_group_id AND sgroup_1.name IS NOT NULL
          JOIN auth_scope auth_scope_1 ON ar_map_1.auth_scope = auth_scope_1.id
          WHERE ar_map_1.unique_group_id IS NOT NULL AND ar_map_1.auth_role = 27
          GROUP BY auth_scope_1.unique_business_role_id) group_manage ON group_manage.unique_business_role_id = b_role.unique_id
     LEFT JOIN suser su5 ON ar_map.unique_user_id = su5.unique_user_id AND su5.display_name IS NOT NULL AND su5.deleted = 0 AND su5.effective = 1
     LEFT JOIN sgroup ON ar_map.unique_group_id = sgroup.unique_group_id AND sgroup.name IS NOT NULL AND sgroup.deleted = 0 AND sgroup.effective = 1
     LEFT JOIN approval_policy ap ON b_role.approval_policy = ap.id
     LEFT JOIN business_role_memb_crit br_crit ON b_role.id = br_crit.business_role_id
      LEFT JOIN ( SELECT incl_brole2.business_role_id,
          LISTAGG(incl_brole2.name, ' | ' ON OVERFLOW TRUNCATE) WITHIN GROUP (ORDER BY incl_brole2.business_role_id) AS bnames
          FROM business_role_incl_brole incl_brole2
          GROUP BY incl_brole2.business_role_id) incl_brole ON incl_brole.business_role_id = b_role.id
     LEFT JOIN ( SELECT br_owner_1.business_role_id,
          LISTAGG(su.display_name, ' | ' ON OVERFLOW TRUNCATE) WITHIN GROUP (ORDER BY su.display_name) AS user_owners
          FROM business_role_owner br_owner_1
          JOIN (SELECT DISTINCT unique_user_id, display_name, effective, deleted from suser) su ON br_owner_1.unique_owner_id = su.unique_user_id AND su.display_name IS NOT NULL AND su.deleted = 0 AND su.effective = 1
          GROUP BY br_owner_1.business_role_id) user_owners ON user_owners.business_role_id = b_role.id


     LEFT JOIN ( SELECT br_owner_1.business_role_id,
          LISTAGG(sgroup_1.name, ' | ' ON OVERFLOW TRUNCATE) WITHIN GROUP (ORDER BY sgroup_1.name) AS group_owners
          FROM business_role_owner br_owner_1
          JOIN (SELECT DISTINCT unique_group_id, name, deleted, effective from sgroup) sgroup_1 ON br_owner_1.unique_owner_id = sgroup_1.unique_group_id AND sgroup_1.name IS NOT NULL AND sgroup_1.deleted = 0 AND sgroup_1.effective = 1
          GROUP BY br_owner_1.business_role_id) group_owners ON group_owners.business_role_id = b_role.id

     LEFT JOIN ( SELECT apmemb.approval_policy_id,
          LISTAGG(su.display_name, ' | ' ON OVERFLOW TRUNCATE) WITHIN GROUP (ORDER BY su.display_name) AS policy_users
          FROM approval_policy_memb apmemb
          JOIN (SELECT DISTINCT unique_user_id, display_name, deleted, effective from suser) su ON apmemb.unique_member_id = su.unique_user_id AND su.display_name IS NOT NULL AND su.deleted = 0 AND su.effective = 1
          WHERE apmemb.is_user = 1
          GROUP BY apmemb.approval_policy_id) pol_users ON pol_users.approval_policy_id = b_role.approval_policy

     LEFT JOIN ( SELECT apmemb.approval_policy_id,
          LISTAGG(sgroup_1.name, ' | ' ON OVERFLOW TRUNCATE) WITHIN GROUP(ORDER BY sgroup_1.name) AS policy_groups
          FROM approval_policy_memb apmemb
          JOIN (SELECT DISTINCT unique_group_id, name, deleted, effective from sgroup) sgroup_1 ON apmemb.unique_member_id = sgroup_1.unique_group_id AND sgroup_1.name IS NOT NULL AND sgroup_1.deleted = 0 AND sgroup_1.effective = 1
          WHERE apmemb.is_user = 0
          GROUP BY apmemb.approval_policy_id) pol_groups ON pol_groups.approval_policy_id = b_role.approval_policy

     LEFT JOIN ( SELECT br_incl_mem.business_role_id,
          LISTAGG(su.display_name, ' | ' ON OVERFLOW TRUNCATE) WITHIN GROUP (ORDER BY su.display_name) AS included_users
          FROM business_role_incl_memb br_incl_mem
          JOIN (SELECT DISTINCT unique_user_id, display_name, deleted, effective from suser) su ON br_incl_mem.unique_member_id = su.unique_user_id AND su.display_name IS NOT NULL AND su.deleted = 0 AND su.effective = 1
          WHERE br_incl_mem.is_user = 1
          GROUP BY br_incl_mem.business_role_id) incl_users ON incl_users.business_role_id = b_role.id

     LEFT JOIN ( SELECT br_incl_mem.business_role_id,
          LISTAGG(sgroup_1.name, ' | ' ON OVERFLOW TRUNCATE) WITHIN GROUP (ORDER BY sgroup_1.name) AS included_groups
          FROM business_role_incl_memb br_incl_mem
          JOIN (SELECT DISTINCT unique_group_id, name, deleted, effective from sgroup) sgroup_1 ON br_incl_mem.unique_member_id = sgroup_1.unique_group_id AND sgroup_1.name IS NOT NULL AND sgroup_1.deleted = 0 AND sgroup_1.effective = 1
          WHERE br_incl_mem.is_user = 0
          GROUP BY br_incl_mem.business_role_id) incl_groups ON incl_groups.business_role_id = b_role.id

    LEFT JOIN ( SELECT br_excl_mem.business_role_id,
          LISTAGG(suser.display_name, ' | ' ON OVERFLOW TRUNCATE) WITHIN GROUP (ORDER BY suser.display_name) AS excluded_users
          FROM business_role_excl_memb br_excl_mem
          JOIN (SELECT DISTINCT unique_user_id, display_name, deleted, effective from suser) suser ON br_excl_mem.unique_member_id = suser.unique_user_id AND suser.display_name IS NOT NULL AND suser.deleted = 0 AND suser.effective = 1
          WHERE br_excl_mem.is_user = 1
          GROUP BY br_excl_mem.business_role_id) excl_users ON excl_users.business_role_id = b_role.id

    LEFT JOIN ( SELECT br_excl_mem.business_role_id,
          LISTAGG(sgroup_1.name, ' | ' ON OVERFLOW TRUNCATE) WITHIN GROUP (ORDER BY sgroup_1.name) AS excluded_groups
          FROM business_role_excl_memb br_excl_mem
          JOIN (SELECT DISTINCT unique_group_id, name, deleted, effective from sgroup) sgroup_1 ON br_excl_mem.unique_member_id = sgroup_1.unique_group_id AND sgroup_1.name IS NOT NULL AND sgroup_1.deleted = 0 AND sgroup_1.effective = 1
          WHERE br_excl_mem.is_user = 0
          GROUP BY br_excl_mem.business_role_id) excl_groups ON excl_groups.business_role_id = b_role.id

     LEFT JOIN ( SELECT br_incl_item.business_role_id,
          LISTAGG(spermission.name, ' | ' ON OVERFLOW TRUNCATE) WITHIN GROUP (ORDER BY spermission.name) AS auth_permission
          FROM business_role_incl_item br_incl_item
          JOIN (SELECT DISTINCT unique_permission_id, name, deleted, effective from spermission) spermission ON br_incl_item.unique_item_id = spermission.unique_permission_id AND spermission.name IS NOT NULL AND spermission.deleted = 0 AND spermission.effective = 1
          WHERE br_incl_item.item_type = 'PERMISSION'
          GROUP BY br_incl_item.business_role_id) auth_perm ON auth_perm.business_role_id = b_role.id

     LEFT JOIN ( SELECT br_incl_item.business_role_id,
          LISTAGG(role_policy.name, ' | ' ON OVERFLOW TRUNCATE) WITHIN GROUP (ORDER BY role_policy.name) AS auth_tech_role
          FROM business_role_incl_item br_incl_item
          JOIN (SELECT DISTINCT unique_policy_id, name, deleted from role_policy) role_policy ON br_incl_item.unique_item_id = role_policy.unique_policy_id AND role_policy.deleted = 0
          WHERE br_incl_item.item_type = 'TECHNICAL_ROLE'
          GROUP BY br_incl_item.business_role_id) auth_tech_role ON auth_tech_role.business_role_id = b_role.id

     LEFT JOIN ( SELECT br_incl_item.business_role_id,
          LISTAGG(data_source.name, ' | ' ON OVERFLOW TRUNCATE) WITHIN GROUP (ORDER BY data_source.name) AS auth_application
          FROM business_role_incl_item br_incl_item
          JOIN (SELECT DISTINCT id, name, deleted from data_source) data_source ON br_incl_item.unique_item_id = data_source.id AND data_source.deleted = 0
          WHERE br_incl_item.item_type = 'APPLICATION'
          GROUP BY br_incl_item.business_role_id) auth_app ON auth_app.business_role_id = b_role.id
          
  WHERE b_role.approval_state = 'ARCHIVE' OR b_role.approval_state = 'PUBLISHED';


GRANT SELECT ON business_role_hist_v TO ig_report_role;
CREATE PUBLIC SYNONYM business_role_hist_v FOR igops.business_role_hist_v;