BEGIN
  FOR k IN (SELECT null FROM all_synonyms WHERE owner = 'PUBLIC' and synonym_name = 'CURRENT_USER_ACCESS_V') LOOP
    EXECUTE IMMEDIATE 'DROP PUBLIC SYNONYM CURRENT_USER_ACCESS_V';
  END LOOP;
   FOR i IN (SELECT null FROM all_views WHERE owner = user and view_name = 'CURRENT_USER_ACCESS_V') LOOP
    EXECUTE IMMEDIATE 'DROP VIEW CURRENT_USER_ACCESS_V';
  END LOOP;

  FOR k IN (SELECT null FROM all_synonyms WHERE owner = 'PUBLIC' and synonym_name = 'AUTH_USER_RESOURCE_V') LOOP
    EXECUTE IMMEDIATE 'DROP PUBLIC SYNONYM AUTH_USER_RESOURCE_V';
  END LOOP;
   FOR i IN (SELECT null FROM all_views WHERE owner = user and view_name = 'AUTH_USER_RESOURCE_V') LOOP
    EXECUTE IMMEDIATE 'DROP VIEW AUTH_USER_RESOURCE_V';
  END LOOP;

  FOR k IN (SELECT null FROM all_synonyms WHERE owner = 'PUBLIC' and synonym_name = 'ROLE_PERM_POLICY_V') LOOP
    EXECUTE IMMEDIATE 'DROP PUBLIC SYNONYM ROLE_PERM_POLICY_V';
  END LOOP;
   FOR i IN (SELECT null FROM all_views WHERE owner = user and view_name = 'ROLE_PERM_POLICY_V') LOOP
    EXECUTE IMMEDIATE 'DROP VIEW ROLE_PERM_POLICY_V';
  END LOOP;
END;
/

CREATE VIEW current_user_access_v AS
  WITH  trole_users AS(
    SELECT
      role_user.policy,
      suser.unique_user_id
    FROM role_user
      JOIN suser ON role_user.suser = suser.id
) ,current_user_access AS(
  SELECT DISTINCT 
        users.id,
        users.display_name,
        users.job_title,
        suser_email.email email,
        users.department,
        users.user_id,
        users.first_name,
        users.last_name,
        users.risk,
        users.unique_user_id,
        supervisor.display_name supervisor,
        sci.violations,
        CASE
          WHEN sgroup.group_id IS NOT NULL THEN '1-GROUP'
          ELSE '1-NO-GROUP'
        END detail_type,
        sgroup.group_id group_id,
        sgroup.name group_name,
        sgroup.short_description group_description,
        NULL permission_id,
        NULL permission_name,
        NULL permission_description,
        NULL permission_type,
        NULL permission_application_name,
        NULL permission_risk,
        NULL permission_root,
        NULL permission_policy_violated,
        NULL permission_entitlements,
        NULL brole_id,
        NULL brole_name,
        NULL brole_description,
        NULL brole_risk,
        NULL brole_update_time,
        NULL supervisee_id,
        NULL supervisee_display_name,
        NULL supervisee_email,
        NULL supervisee_department,
        NULL supervisee_job_title,
        NULL trole_policy_id,
        NULL trole_item_name,
        NULL trole_unique_policy_id,
        NULL trole_description,
        NULL trole_risk,
        NULL trole_policy_violated,
        NULL account_name,
        NULL account_id,
        NULL account_risk,
        NULL account_description,
        NULL account_application_name
        FROM suser users
        LEFT JOIN sgroup_user direct ON users.collected_entity = direct.suser
        LEFT JOIN sgroup_indirect_suser_v indirect ON users.id = indirect.id AND direct.sgroup = indirect.child_id
        LEFT JOIN sgroup ON direct.sgroup = sgroup.id OR indirect.parent_id = sgroup.id
        LEFT JOIN (SELECT DISTINCT
                      suser1.id suser_id,
                      LISTAGG((TO_CHAR(suser_email1.email)), ' | ') WITHIN GROUP(ORDER BY suser_email1.email) email
                   FROM suser suser1
                  JOIN suser_email suser_email1 ON suser1.id = suser_email1.suser AND suser1.effective = 1 AND suser1.deleted = 0
                   GROUP BY suser1.id) suser_email ON users.id = suser_email.suser_id
        LEFT JOIN suser supervisor ON users.supervisor_unique_id = supervisor.unique_user_id AND supervisor.effective = 1 AND supervisor.deleted = 0
        LEFT JOIN ( SELECT 
                        CASE 
                            WHEN sv.suser IS NOT NULL THEN su.display_name
                            ELSE sa.name
                          END violator_name,
                          CASE
                            WHEN sv.suser IS NOT NULL THEN su.unique_user_id
                            ELSE sa.unique_account_id
                          END violator_id,
                       LISTAGG(CONCAT(TO_CHAR(sp.name),CONCAT(': SOD-', to_char(sc.id,'FM00XX'))), ', ') WITHIN GROUP (ORDER BY su.unique_user_id, sa.unique_account_id) violations                      
                       FROM sod_case sc
                       LEFT JOIN sod_policy sp ON sc.policy = sp.id
                       LEFT JOIN sod_violation sv ON sc.current_violation = sv.id
                       LEFT JOIN suser su ON sv.suser = su.id
                       LEFT JOIN saccount sa ON sv.saccount = sa.id 
                       GROUP BY  
                          CASE 
                            WHEN sv.suser IS NOT NULL THEN su.display_name
                            ELSE sa.name
                          END,
                          CASE
                            WHEN sv.suser IS NOT NULL THEN su.unique_user_id
                            ELSE sa.unique_account_id
                          END) sci ON users.unique_user_id = sci.violator_id
      WHERE users.effective = 1 AND users.deleted = 0
      GROUP BY
        users.id,
        users.display_name,
        users.job_title,
        suser_email.email,
        users.department,
        users.user_id,
        users.first_name,
        users.last_name,
        users.risk,
        users.unique_user_id,
        supervisor.display_name,
        sci.violations,
        CASE
          WHEN sgroup.group_id IS NOT NULL THEN '1-GROUP'
          ELSE '1-NO-GROUP'
        END,
        sgroup.group_id,
        sgroup.name,
        sgroup.short_description

       UNION ALL
        SELECT
        users.id,
        users.display_name,
        users.job_title,
        suser_email.email,
        users.department,
        users.user_id,
        users.first_name,
        users.last_name,
        users.risk,
        users.unique_user_id,
        supervisor.display_name supervisor,
        sci.violations,
          CASE
            WHEN perm.id IS NOT NULL THEN '2-PERMISSION'
            ELSE '2-NO-PERM'
          END detail_type,
        NULL group_id,
        NULL group_name,
        NULL group_description,
        perm.id permission_id,
        perm.name permission_name,
        perm.description permission_description,
        perm.type permission_type,
        perm.application_name permission_application_name,
        perm.risk permission_risk,
        sav.permission permission_root,
        spv.policy_name permission_policy_violated,
        ent.permission_entitlements,
        NULL brole_id,
        NULL brole_name,
        NULL brole_description,
        NULL brole_risk,
        NULL brole_update_time,
        NULL supervisee_id,
        NULL supervisee_display_name,
        NULL supervisee_email,
        NULL supervisee_department,
        NULL supervisee_job_title,
        NULL trole_policy_id,
        NULL trole_item_name,
        NULL trole_unique_policy_id,
        NULL trole_description,
        NULL trole_risk,
        NULL trole_policy_violated,
        NULL account_name,
        NULL account_id,
        NULL account_risk,
        NULL account_description,
        NULL account_application_name
      FROM suser users
        LEFT JOIN 
            (SELECT
              perm.id, 
              sap.suser, perm.name, 
              perm.description, 
              perm.type, 
              data_source.name application_name, 
              perm.risk 
             FROM spermission perm JOIN data_source ON perm.application = data_source.id 
              JOIN suser_account_permission sap ON perm.id = sap.spermission 
              WHERE (exclude_from_catalog = 0 OR exclude_from_catalog IS NULL) AND effective = 1) perm ON users.id = perm.suser
              
        LEFT JOIN 
            (SELECT 
              perm.name,
              perm.id,
              LISTAGG((TO_CHAR(ent.associated_permission)), ' | ') WITHIN GROUP (ORDER BY ent.associated_permission) permission_entitlements
            FROM spermission perm
              LEFT JOIN (SELECT DISTINCT perm.id, perm_assoc.name associated_permission FROM spermission_entailment perm_entail
              LEFT JOIN spermission perm ON perm.collected_entity = perm_entail.entailing
              LEFT JOIN spermission perm_assoc ON perm_assoc.collected_entity = perm_entail.entailed
            WHERE perm.deleted = '0' AND perm.effective = '1' AND perm_assoc.deleted = '0' AND perm_assoc.effective = '1') ent ON ent.id = perm.id    
              GROUP BY  perm.name, perm.id) ent on ent.id = perm.id
        LEFT JOIN spermission perm1 ON perm1.id = perm.id AND perm1.effective = 1 AND perm1.deleted = 0
        
        LEFT JOIN 
            (SELECT DISTINCT
                perm.name permission,
                perm.id,
                perm_entail.entailed associated_permission_id
            FROM spermission_entailment perm_entail
            LEFT JOIN spermission perm ON perm.collected_entity = perm_entail.entailing
            LEFT JOIN spermission perm_assoc ON perm_assoc.collected_entity = perm_entail.entailed
             WHERE perm.deleted = 0 AND perm.effective = 1 AND perm_assoc.deleted = 0 AND perm_assoc.effective = 1) sav ON sav.associated_permission_id = perm1.collected_entity
             
        LEFT JOIN 
            (SELECT 
                sv.suser,
                sp.name policy_name, 
                sod_condition_item.unique_id
             FROM sod_policy sp
             LEFT JOIN sod_violation sv ON sp.id = sv.policy
             LEFT JOIN sod_condition ON sod_condition.policy = sp.id
             LEFT JOIN sod_condition_item ON sod_condition_item.condition = sod_condition.id) spv ON spv.unique_id = perm1.unique_permission_id AND users.collected_entity = spv.suser
        LEFT JOIN (SELECT DISTINCT
                      suser1.id suser_id,
                      LISTAGG((TO_CHAR(suser_email1.email)), ' | ') WITHIN GROUP(ORDER BY suser_email1.email) email
                   FROM suser suser1
                  JOIN suser_email suser_email1 ON suser1.id = suser_email1.suser AND suser1.effective = 1 AND suser1.deleted = 0
                   GROUP BY suser1.id) suser_email ON users.id = suser_email.suser_id
                   
        LEFT JOIN suser supervisor ON supervisor.unique_user_id = users.supervisor_unique_id AND supervisor.effective = 1 AND supervisor.deleted = 0
        
        LEFT JOIN ( SELECT 
                      CASE 
                        WHEN sv.suser IS NOT NULL THEN su.display_name
                        ELSE sa.name
                      END violator_name,
                      CASE 
                          WHEN sv.suser IS NOT NULL THEN su.unique_user_id
                          ELSE sa.unique_account_id
                      END violator_id,
                      LISTAGG(CONCAT(TO_CHAR(sp.name),CONCAT(': SOD-', to_char(sc.id,'FM00XX'))), ', ') WITHIN GROUP (ORDER BY su.unique_user_id, sa.unique_account_id) violations
                   FROM sod_case sc
                   LEFT JOIN sod_policy sp ON sc.policy = sp.id
                   LEFT JOIN sod_violation sv ON sc.current_violation = sv.id
                   LEFT JOIN suser su ON sv.suser = su.id
                   LEFT JOIN saccount sa ON sv.saccount = sa.id
                  GROUP BY 
                       CASE 
                        WHEN sv.suser IS NOT NULL THEN su.display_name
                        ELSE sa.name
                      END, 
                       CASE 
                          WHEN sv.suser IS NOT NULL THEN su.unique_user_id
                          ELSE sa.unique_account_id
                      END) sci ON sci.violator_id = users.unique_user_id  
                      
                      
      WHERE users.effective = 1 AND users.deleted = 0
      GROUP BY 
          users.id, 
          users.display_name, 
          users.job_title,
          suser_email.email,
          users.department, 
          users.user_id, 
          users.first_name, 
          users.last_name, 
          users.risk, 
          users.unique_user_id, 
          supervisor.display_name, 
          sci.violations, 
          perm.id, 
          perm.name, 
          perm.description, 
          perm.type, 
          perm.application_name, 
          perm.risk, 
          sav.permission, 
          spv.policy_name,
          ent.permission_entitlements
   
      UNION ALL
      SELECT 
        users.id,
        users.display_name,
        users.job_title,
        LISTAGG((TO_CHAR(suser_email.email)), ' | ') WITHIN GROUP(ORDER BY suser_email.email) email,
        users.department,
        users.user_id,
        users.first_name,
        users.last_name,
        users.risk,
        users.unique_user_id,
        supervisor.display_name supervisor,
        sci.violations,
          CASE
            WHEN br.id IS NOT NULL THEN '3-BROLE'
            ELSE '3-NO-BROLE'
          END detail_type,
        NULL group_id,
        NULL group_name,
        NULL group_description,
        NULL permission_id,
        NULL permission_name,
        NULL permission_description,
        NULL permission_type,
        NULL permission_application_name,
        NULL permission_risk,
        NULL permission_root,
        NULL permission_policy_violated,
        NULL permission_entitlements,
        br.id brole_id,
        br.name brole_name,
        br.description brole_description,
        br.risk brole_risk,
        br.update_time brole_update_time,
        NULL supervisee_id,
        NULL supervisee_display_name,
        NULL supervisee_email,
        NULL supervisee_department,
        NULL supervisee_job_title,
        NULL trole_policy_id,
        NULL trole_item_name,
        NULL trole_unique_policy_id,
        NULL trole_description,
        NULL trole_risk,
        NULL trole_policy_violated,
        NULL account_name,
        NULL account_id,
        NULL account_risk,
        NULL account_description,
        NULL account_application_name
      FROM business_role_member brm
        LEFT JOIN business_role br ON brm.unique_business_role_id = br.unique_id AND br.approval_state = 'PUBLISHED'
        RIGHT JOIN suser users ON users.unique_user_id = brm.unique_user_id
        LEFT JOIN suser_email ON users.id = suser_email.suser
        LEFT JOIN suser supervisor ON supervisor.unique_user_id = users.supervisor_unique_id AND supervisor.effective = 1 AND supervisor.deleted = 0
        LEFT JOIN 
              ( SELECT 
                CASE 
                    WHEN sv.suser IS NOT NULL THEN su.display_name
                    ELSE sa.name
                END violator_name,
                CASE 
                    WHEN sv.suser IS NOT NULL THEN su.unique_user_id
                    ELSE sa.unique_account_id
                END violator_id,
                LISTAGG(CONCAT(TO_CHAR(sp.name),CONCAT(': SOD-', to_char(sc.id,'FM00XX'))), ', ') WITHIN GROUP (ORDER BY su.unique_user_id, sa.unique_account_id) violations
               FROM sod_case sc
               LEFT JOIN sod_policy sp ON sc.policy = sp.id
               LEFT JOIN sod_violation sv ON sc.current_violation = sv.id
               LEFT JOIN suser su ON sv.suser = su.id
               LEFT JOIN saccount sa ON sv.saccount = sa.id
                GROUP BY 
                  CASE 
                        WHEN sv.suser IS NOT NULL THEN su.display_name
                        ELSE sa.name
                      END, 
                       CASE 
                          WHEN sv.suser IS NOT NULL THEN su.unique_user_id
                          ELSE sa.unique_account_id
                      END
                  ) sci ON sci.violator_id = users.unique_user_id  
      WHERE users.effective = 1 AND users.deleted = 0 and brm.deleted = 0
      GROUP BY 
        users.id, 
        users.display_name, 
        users.job_title, 
        users.department, 
        users.user_id, 
        users.first_name, 
        users.last_name, 
        users.risk, 
        users.unique_user_id,  
        supervisor.display_name, 
        sci.violations, 
        br.id, 
        br.name, 
        br.description, 
        br.risk, 
        br.update_time
     
      UNION ALL
      SELECT 
        users.id,
        users.display_name,
        users.job_title,
        LISTAGG((TO_CHAR(suser_email.email)), ' | ') WITHIN GROUP(ORDER BY suser_email.email) email,
        users.department,
        users.user_id,
        users.first_name,
        users.last_name,
        users.risk,
        users.unique_user_id,
        supervisor.display_name supervisor,
        sci.violations,
          CASE
            WHEN supervisee.unique_user_id IS NOT NULL THEN '4-SUPERVISEE' 
            ELSE '4-NO-SUPERVISEE'
          END detail_type,
        NULL group_id,
        NULL group_name,
        NULL group_description,
        NULL permission_id,
        NULL permission_name,
        NULL permission_description,
        NULL permission_type,
        NULL permission_application_name,
        NULL permission_risk,
        NULL permission_root,
        NULL permission_policy_violated,
        NULL permission_entitlements,
        NULL brole_id,
        NULL brole_name,
        NULL brole_description,
        NULL brole_risk,
        NULL brole_update_time,
        supervisee.unique_user_id supervisee_id,
        supervisee.display_name supervisee_display_name,
        supervisee_email.email  supervisee_email,
        supervisee.department supervisee_department,
        supervisee.job_title supervisee_job_title,
        NULL trole_policy_id,
        NULL trole_item_name,
        NULL trole_unique_policy_id,
        NULL trole_description,
        NULL trole_risk,
        NULL trole_policy_violated,
        NULL account_name,
        NULL account_id,
        NULL account_risk,
        NULL account_description,
        NULL account_application_name
      FROM suser users
        LEFT JOIN suser supervisee ON supervisee.supervisor_unique_id = users.unique_user_id AND supervisee.deleted = '0' AND supervisee.effective = '1'
        LEFT JOIN suser supervisor ON supervisor.unique_user_id = users.supervisor_unique_id AND supervisor.effective = 1 AND supervisor.deleted = 0
        LEFT JOIN suser_email ON users.id = suser_email.suser
      LEFT JOIN 
          (SELECT
              suser.id suser,
              suser.display_name,
              LISTAGG((TO_CHAR(suser_email.email)), ' | ') WITHIN GROUP(ORDER BY suser_email.email) email
          FROM suser
          JOIN suser_email ON suser.id = suser_email.suser AND suser.effective = '1' AND suser.deleted = '0'   
          GROUP BY suser.id, suser.display_name) supervisee_email ON supervisee.id = supervisee_email.suser
      LEFT JOIN (
                SELECT 
                  CASE 
                    WHEN sv.suser IS NOT NULL THEN su.display_name
                    ELSE sa.name
                  END violator_name,
                  CASE 
                    WHEN sv.suser IS NOT NULL THEN su.unique_user_id
                    ELSE sa.unique_account_id
                  END violator_id,
                  LISTAGG(CONCAT(TO_CHAR(sp.name),CONCAT(': SOD-', to_char(sc.id,'FM00XX'))), ', ') WITHIN GROUP (ORDER BY su.unique_user_id, sa.unique_account_id) violations
                FROM sod_case sc
                 LEFT JOIN sod_policy sp ON sc.policy = sp.id
                 LEFT JOIN sod_violation sv ON sc.current_violation = sv.id
                 LEFT JOIN suser su ON sv.suser = su.id
                 LEFT JOIN saccount sa ON sv.saccount = sa.id
                GROUP BY 
                      CASE 
                        WHEN sv.suser IS NOT NULL THEN su.display_name
                        ELSE sa.name
                      END, 
                       CASE 
                          WHEN sv.suser IS NOT NULL THEN su.unique_user_id
                          ELSE sa.unique_account_id
                      END) sci ON sci.violator_id = users.unique_user_id  
      WHERE users.effective = 1 AND users.deleted = 0
      GROUP BY 
        users.id,
        users.display_name,
        users.job_title,
        users.department,
        users.user_id,
        users.first_name,
        users.last_name,
        users.risk,
        users.unique_user_id,
        supervisor.display_name, 
        sci.violations, 
        supervisee.unique_user_id, 
        supervisee.display_name, 
        supervisee_email.email, 
        supervisee.department, 
        supervisee.job_title

      UNION ALL
      SELECT DISTINCT
        users.id,
        users.display_name,
        users.job_title,
        LISTAGG((TO_CHAR(suser_email.email)), ' | ') WITHIN GROUP(ORDER BY suser_email.email) email,
        users.department,
        users.user_id,
        users.first_name,
        users.last_name,
        users.risk,
        users.unique_user_id,
        supervisor.display_name supervisor,
        sci.violations,
          CASE
            WHEN rperm.policy IS NOT NULL THEN '5-TROLE'
            ELSE '5-NO-TROLE'
          END detail_type,
        NULL group_id,
        NULL group_name,
        NULL group_description,
        NULL permission_id,
        NULL permission_name,
        NULL permission_description,
        NULL permission_type,
        NULL permission_application_name,
        NULL permission_risk,
        NULL permission_root,
        NULL permission_policy_violated,
        NULL permission_entitlements,
        NULL brole_id,
        NULL brole_name,
        NULL brole_description,
        NULL brole_risk,
        NULL brole_update_time,
        NULL supervisee_id,
        NULL supervisee_display_name,
        NULL supervisee_email,
        NULL supervisee_department,
        NULL supervisee_job_title,
        rperm.policy trole_policy_id,
        rpolicy.name trole_item_name,
        rpolicy.unique_policy_id trole_unique_policy_id,
        rpolicy.description trole_description,
        rpolicy.risk trole_risk,
        spv.violations trole_policy_violated,
        NULL account_name,
        NULL account_id,
        NULL account_risk,
        NULL account_description,
        NULL account_application_name
      FROM suser users
        LEFT JOIN trole_users ruser ON users.unique_user_id = ruser.unique_user_id
        LEFT JOIN role_policy rpolicy ON rpolicy.id = ruser.policy AND rpolicy.deleted = 0
        LEFT JOIN role_permission rperm ON ruser.policy = rperm.policy
        LEFT JOIN sod_condition_item sod ON sod.unique_id = rpolicy.unique_policy_id AND users.id = sod.id
        LEFT JOIN sod_condition ON sod.condition = sod_condition.id
        LEFT JOIN sod_policy ON sod_condition.policy = sod_policy.id
        LEFT JOIN suser_email ON users.id = suser_email.suser
        LEFT JOIN suser supervisor ON supervisor.unique_user_id = users.supervisor_unique_id AND supervisor.effective = 1 AND supervisor.deleted = 0
        LEFT JOIN ( SELECT 
                       sci1.violator_name,
                       sci1.violator_id,
                       listagg(TO_CHAR(CONCAT(sci1.policy_name, CONCAT(': SOD-', to_char(sci1.id, 'FM00XX') ))), ', ') WITHIN GROUP(ORDER BY sci1.violator_name) AS violations
                   FROM 
                   (
                      SELECT
                      sc.id as id,
                      su.display_name AS violator_name,
                      su.unique_user_id AS violator_id,
                      sp.name as policy_name
                  FROM SOD_CASE sc
                      LEFT JOIN SOD_POLICY sp ON sc.policy = sp.id
                      LEFT JOIN SOD_VIOLATION sv ON sc.current_violation = sv.id
                      LEFT JOIN SUSER su ON sv.suser = su.id
                   )
                   sci1
                   GROUP BY sci1.violator_name, sci1.violator_id) sci ON users.unique_user_id = sci.violator_id
      LEFT JOIN ( SELECT sc.id AS case_id,
            CASE
                WHEN sv.suser IS NOT NULL THEN su.display_name
                ELSE sa.name
            END AS violator_name,
            CASE
                WHEN sod_condition_item.type = 'PERMISSION' THEN spermission_1.name
                ELSE role_policy.name
            END AS item_name,
            su.unique_user_id,
            sp.name as policy_name,
      sod_condition_item.unique_id,
      LISTAGG(CONCAT(TO_CHAR(sp.name),CONCAT(': SOD-', to_char(sc.id,'FM00XX'))), ', ') WITHIN GROUP (ORDER BY su.display_name) violations
       FROM sod_case sc
         LEFT JOIN sod_policy sp ON sc.policy = sp.id
         LEFT JOIN sod_violation sv ON sc.current_violation = sv.id
         LEFT JOIN suser su ON sv.suser = su.collected_entity AND su.effective = 1 AND su.deleted = 0
         LEFT JOIN saccount sa ON sv.saccount = sa.id
         LEFT JOIN sod_condition ON sod_condition.policy = sc.policy
         LEFT JOIN sod_condition_item ON sod_condition_item.condition = sod_condition.id
         LEFT JOIN spermission spermission_1 ON spermission_1.unique_permission_id = sod_condition_item.unique_id AND spermission_1.effective = 1 AND spermission_1.deleted = 0
         LEFT JOIN role_policy ON role_policy.unique_policy_id = sod_condition_item.unique_id AND role_policy.deleted = 0
         GROUP BY sc.id, sv.suser, su.display_name, sa.name, sod_condition_item.type, spermission_1.name, role_policy.name, su.unique_user_id, sod_condition_item.unique_id, sp.name ) spv ON spv.unique_user_id = users.unique_user_id AND rpolicy.unique_policy_id = spv.unique_id
        WHERE users.effective = 1 AND users.deleted = 0 AND (rpolicy.deleted = 0 OR rpolicy.ID IS NULL)
        GROUP BY 
          users.id,
          users.display_name,
          users.job_title,
          users.department,
          users.user_id,
          users.first_name,
          users.last_name,
          users.risk,
          users.unique_user_id,
          supervisor.display_name, 
          sci.violations, 
          rperm.policy, 
          rpolicy.name, 
          rpolicy.unique_policy_id, 
          rpolicy.description, 
          rpolicy.risk, 
          spv.violations
          
      UNION ALL
      SELECT DISTINCT
        users.id,
        users.display_name,
        users.job_title,
        suser_email.email email,
        users.department,
        users.user_id,
        users.first_name,
        users.last_name,
        users.risk,
        users.unique_user_id,
        supervisor.display_name supervisor,
        sci.violations,
          CASE
            WHEN sac.account_id IS NOT NULL THEN '6-ACCOUNT'
            ELSE '6-NO-ACCOUNT'
          END detail_type,
        NULL group_id,
        NULL group_name,
        NULL group_description,
        NULL permission_id,
        NULL permission_name,
        NULL permission_description,
        NULL permission_type,
        NULL permission_application_name,
        NULL permission_risk,
        NULL permission_root,
        NULL permission_policy_violated,
        NULL permission_entitlements,
        NULL brole_id,
        NULL brole_name,
        NULL brole_description,
        NULL brole_risk,
        NULL brole_update_time,
        NULL supervisee_id,
        NULL supervisee_display_name,
        NULL supervisee_email,
        NULL supervisee_department,
        NULL supervisee_job_title,
        NULL trole_policy_id,
        NULL trole_item_name,
        NULL trole_unique_policy_id,
        NULL trole_description,
        NULL trole_risk,
        NULL trole_policy_violated,
        sac.name account_name,
        sac.account_id,
        sac.risk account_risk,
        sac.description account_description,
        source.name account_application_name
      FROM suser_account_permission sap
        JOIN saccount sac ON sap.saccount = sac.id AND sac.effective = '1' AND sac.deleted = '0' AND sac.virtual = '0'
        JOIN data_source source ON sac.application = source.id
        RIGHT JOIN suser users ON sap.suser = users.id
        LEFT JOIN (SELECT DISTINCT
                      suser1.id suser_id,
                      LISTAGG((TO_CHAR(suser_email1.email)), ' | ') WITHIN GROUP(ORDER BY suser_email1.email) email
                   FROM suser suser1
                  JOIN suser_email suser_email1 ON suser1.id = suser_email1.suser AND suser1.effective = 1 AND suser1.deleted = 0
                   GROUP BY suser1.id) suser_email ON users.id = suser_email.suser_id
        LEFT JOIN suser supervisor ON supervisor.unique_user_id = users.supervisor_unique_id AND supervisor.effective = 1 AND supervisor.deleted = 0
        LEFT JOIN ( SELECT 
                      CASE WHEN sv.suser IS NOT NULL THEN su.display_name
                        ELSE sa.name
                        END violator_name,
                      CASE WHEN sv.suser IS NOT NULL THEN su.unique_user_id
                        ELSE sa.unique_account_id
                        END violator_id,
                      LISTAGG(CONCAT(TO_CHAR(sp.name),CONCAT(': SOD-', to_char(sc.id,'FM00XX'))), ', ') WITHIN GROUP (ORDER BY su.unique_user_id, sa.unique_account_id) violations
                   FROM sod_case sc
                   LEFT JOIN sod_policy sp ON sc.policy = sp.id
                   LEFT JOIN sod_violation sv ON sc.current_violation = sv.id
                   LEFT JOIN suser su ON sv.suser = su.id
                   LEFT JOIN saccount sa ON sv.saccount = sa.id
        GROUP BY
                CASE WHEN sv.suser IS NOT NULL THEN su.display_name
                ELSE sa.name
                END, 
                CASE WHEN sv.suser IS NOT NULL THEN su.unique_user_id
                ELSE sa.unique_account_id
                END) sci ON sci.violator_id = users.unique_user_id  
        WHERE users.effective = 1 AND users.deleted = 0
        GROUP BY 
          users.id,
          users.display_name,
          users.job_title,
          suser_email.email,
          users.department,
          users.user_id,
          users.first_name,
          users.last_name,
          users.risk,
          users.unique_user_id,
          supervisor.display_name, 
          sci.violations,
          sac.name,
          sac.account_id,
          sac.risk,
          sac.description,
          source.name
    )
   SELECT 
    current_user_access.id,
    current_user_access.display_name,
    current_user_access.job_title,
    current_user_access.email,
    current_user_access.department,
    current_user_access.user_id user_cn,
    current_user_access.first_name,
    current_user_access.last_name,
    current_user_access.risk,
    current_user_access.unique_user_id,
    current_user_access.supervisor,
    current_user_access.violations,
    current_user_access.detail_type,
    current_user_access.group_id,
    current_user_access.group_name,
    current_user_access.group_description,
    current_user_access.permission_id,
    current_user_access.permission_name,
    current_user_access.permission_description,
    current_user_access.permission_type,
    current_user_access.permission_application_name,
    current_user_access.permission_risk,
    current_user_access.permission_root,
    current_user_access.permission_policy_violated,
    current_user_access.permission_entitlements,
    current_user_access.brole_id,
    current_user_access.brole_name,
    current_user_access.brole_description,
    current_user_access.brole_risk,
    current_user_access.brole_update_time,
    current_user_access.supervisee_id,
    current_user_access.supervisee_display_name,
    current_user_access.supervisee_email,
    current_user_access.supervisee_department,
    current_user_access.supervisee_job_title,
    current_user_access.trole_policy_id,
    current_user_access.trole_item_name,
    current_user_access.trole_unique_policy_id,
    current_user_access.trole_description,
    current_user_access.trole_risk,
    current_user_access.trole_policy_violated,
    current_user_access.account_name,
    current_user_access.account_id,
    current_user_access.account_risk,
    current_user_access.account_description,
    current_user_access.account_application_name
    FROM current_user_access;

CREATE VIEW auth_user_resource_v AS 
SELECT
  br.name AS brole,
  br.unique_id AS unique_business_role_id,
  su.display_name AS user_name,
  su.unique_user_id,
  sp.name AS permission_name,
  sp.unique_permission_id,
  rp.name AS role_name,
  rp.unique_policy_id,
  ds.name AS application,
  ds.unique_id AS unique_application_id,
  aur.start_date,
  aur.end_date,
  aur.in_validity_period,
  aur.in_grace_period,
  aur.indirect_authorization,
  aur.auto_grant,
  aur.auto_revoke,
  aur.create_date,
  aur.deleted,
  aur.delete_date
FROM auth_user_resource aur
  JOIN user_resource ur ON aur.user_resource_id = ur.id
  JOIN business_role br ON aur.unique_business_role_id = br.unique_id AND br.approval_state = 'PUBLISHED'
  JOIN suser su ON su.unique_user_id = ur.unique_user_id AND su.effective = 1
  LEFT JOIN spermission sp ON sp.unique_permission_id = ur.unique_resource_id AND sp.effective = 1
  LEFT JOIN role_policy rp ON rp.unique_policy_id = ur.unique_resource_id AND rp.deleted = 0
  LEFT JOIN data_source ds ON (CAST(ds.id AS NVARCHAR2(255)) = ur.unique_resource_id OR ds.id = sp.application) AND ds.deleted = 0;

CREATE VIEW role_perm_policy_v AS
SELECT DISTINCT 
  policy.id,
  policy.name policy_name,
  policy.unique_policy_id,
  policy.description policy_description,
  policy.created_by,
  policy.changed_by,
  policy.creation_time,
  policy.update_time,
  policy.deleted,
  policy.last_detection_time,
  CASE
    WHEN policy.state = 'INACTIVE' AND policy.detection_status IS NULL OR policy.detection_status = 'INACTIVE' THEN 'Inactive'
    WHEN policy.state = 'ACTIVE' AND policy.detection_status = 'COMPLETED' OR policy.detection_status = 'RUNNING' THEN 'Active'
    WHEN policy.state = 'INACTIVE' AND policy.detection_status = 'INVALID' THEN 'Invalid from inactive'
    WHEN policy.state = 'INVALID' AND policy.detection_status = 'INVALID' THEN 'Invalid from active'
    ELSE NULL
  END current_state,
  policy.state actual_state,
  policy.detection_status,
  TO_CHAR(policy.risk_mode) policy_risk_mode,
  policy.risk policy_risk,
  policy.cost policy_cost,
  (SELECT DISTINCT (listagg(TO_CHAR(unique_owner_id),' | ')WITHIN GROUP (ORDER BY unique_owner_id)) user_owner_id
    FROM (SELECT DISTINCT powner.unique_owner_id 
      FROM policy_owners_v powner 
      LEFT OUTER JOIN role_policy rperm ON rperm.id = powner.role_policy  
      WHERE powner.is_user = 1 AND policy.id = rperm.id) subquery ) user_owner_id,
  (SELECT DISTINCT (listagg(TO_CHAR(unique_owner_id),' | ') WITHIN GROUP (ORDER BY unique_owner_id)) group_owner_id
    FROM (SELECT DISTINCT powner.unique_owner_id 
      FROM policy_owners_v powner 
      LEFT OUTER JOIN role_policy rperm ON rperm.id = powner.role_policy  
      WHERE powner.is_user = 0 AND policy.id = rperm.id) subquery2 ) group_owner_id,
  (SELECT DISTINCT (listagg(TO_CHAR(owner_name),' | ')WITHIN GROUP (ORDER BY owner_name)) user_owners
    FROM (SELECT DISTINCT powner.owner_name 
      FROM policy_owners_v powner 
      LEFT OUTER JOIN role_policy rperm ON rperm.id = powner.role_policy 
      WHERE powner.is_user = 1 AND policy.id = rperm.id) subquery ) user_owners,
  (SELECT DISTINCT (listagg(TO_CHAR(owner_name),' | ') WITHIN GROUP (ORDER BY owner_name)) group_owners
    FROM (SELECT DISTINCT powner.owner_name 
      FROM policy_owners_v powner 
      LEFT OUTER JOIN role_policy rperm ON rperm.id = powner.role_policy 
      WHERE powner.is_user = 0 AND policy.id = rperm.id) subquery2) group_owners,
  perm.unique_permission_id,
  perm.name perm_name,
  perm_user.unique_user_id permission_user_id,
  perm_user.display_name permission_user_name
FROM 
  role_policy policy
  LEFT JOIN policy_owner ON policy.id = policy_owner.role_policy
  LEFT JOIN role_permission role_perm ON policy.id = role_perm.policy
  LEFT JOIN spermission perm ON role_perm.unique_permission_id = perm.unique_permission_id AND perm.effective = 1
  LEFT JOIN suser_account_permission_role role_user ON role_user.role = policy.id
  LEFT JOIN suser perm_user ON perm_user.id = role_user.suser;

GRANT SELECT ON igops.current_user_access_v TO ig_report_role;
CREATE PUBLIC SYNONYM current_user_access_v FOR igops.current_user_access_v;

GRANT SELECT ON igops.auth_user_resource_v TO ig_report_role;
CREATE PUBLIC SYNONYM auth_user_resource_v FOR igops.auth_user_resource_v;

GRANT SELECT ON igops.role_perm_policy_v TO ig_report_role;
CREATE PUBLIC SYNONYM role_perm_policy_v FOR igops.role_perm_policy_v;