DROP VIEW IF EXISTS current_user_access_v;

DROP VIEW IF EXISTS auth_user_resource_v;

DROP VIEW IF EXISTS role_perm_policy_v;

CREATE VIEW current_user_access_v AS
  WITH  trole_users AS(
    SELECT
      role_user.policy,
      suser.unique_user_id
    FROM role_user
      JOIN suser ON role_user.suser = suser.id),
  current_user_access AS(
    SELECT DISTINCT 
    users.id,
    users.display_name,
    users.job_title,
    string_agg(distinct suser_email.email, ' | ' order by suser_email.email) AS email,
    users.department,
    users.user_id,
    users.first_name,
    users.last_name,
    users.risk,
    users.unique_user_id,
    supervisor.display_name AS supervisor,
    sci.violations,
    CASE
    WHEN sgroup.group_id IS NOT NULL THEN '1-GROUP'
    ELSE '1-NO-GROUP'
    END AS detail_type,
    sgroup.group_id AS group_id,
    sgroup.name AS group_name,
    sgroup.short_description AS group_description,
    NULL::bigint AS permission_id,
    NULL AS permission_name,
    NULL AS permission_description,
    NULL AS permission_type,
    NULL AS permission_application_name,
    NULL::bigint AS permission_risk,
    NULL AS permission_root,
    NULL AS permission_policy_violated,
    NULL AS permission_entitlements,
    NULL::bigint AS brole_id,
    NULL AS brole_name,
    NULL AS brole_description,
    NULL::bigint AS brole_risk,
    NULL::timestamp with time zone AS brole_update_time,
    NULL AS supervisee_id,
    NULL AS supervisee_display_name,
    NULL AS supervisee_email,
    NULL AS supervisee_department,
    NULL AS supervisee_job_title,
    NULL::bigint AS trole_policy_id,
    NULL AS trole_item_name,
    NULL AS trole_unique_policy_id,
    NULL AS trole_description,
    NULL::bigint AS trole_risk,
    NULL AS trole_policy_violated,
    NULL AS account_name,
    NULL AS account_id,
    NULL::bigint AS account_risk,
    NULL AS account_description,
    NULL AS account_application_name
  FROM suser users
  LEFT JOIN sgroup_user direct ON direct.suser = users.collected_entity
  LEFT JOIN sgroup_indirect_suser_v indirect ON users.id = indirect.id AND direct.sgroup = indirect.child_id
  LEFT JOIN sgroup ON sgroup.id = direct.sgroup OR sgroup.id = indirect.parent_id
  LEFT JOIN suser_email ON users.id = suser_email.suser
  LEFT JOIN suser supervisor ON supervisor.unique_user_id = users.supervisor_unique_id AND supervisor.effective = true AND supervisor.deleted = false
  LEFT JOIN ( SELECT 
                CASE 
                  WHEN sv.suser IS NOT NULL THEN su.display_name
                  ELSE sa.name
                END AS violator_name,
                CASE 
                  WHEN sv.suser IS NOT NULL THEN su.unique_user_id
                  ELSE sa.unique_account_id
                END AS violator_id,
                string_agg(concat(sp.name, concat(': SOD-', lpad(UPPER(to_hex(sc.id))::character varying::text, 4, '0'::text))),
                        ', ' ORDER BY su.unique_user_id, sa.unique_account_id) AS violations
              FROM sod_case sc
              LEFT JOIN sod_policy sp ON sc.policy = sp.id
              LEFT JOIN sod_violation sv ON sc.current_violation = sv.id
              LEFT JOIN suser su ON sv.suser = su.id
              LEFT JOIN saccount sa ON sv.saccount = sa.id
              GROUP BY violator_name, violator_id) sci ON sci.violator_id = users.unique_user_id
  WHERE users.effective = true AND users.deleted = false
  GROUP BY users.id, supervisor.display_name, sci.violations,  sgroup.group_id, sgroup.name, sgroup.short_description
     
  UNION ALL

  SELECT
    users.id,
    users.display_name,
    users.job_title,
    string_agg(distinct suser_email.email, ' | ' ORDER BY suser_email.email) AS email,
    users.department,
    users.user_id,
    users.first_name,
    users.last_name,
    users.risk,
    users.unique_user_id,
    supervisor.display_name AS supervisor,
    sci.violations,
    CASE
      WHEN perm.permission_id IS NOT NULL THEN '2-PERMISSION'
      ELSE '2-NO-PERM'
    END AS detail_type,
    NULL::character varying AS group_id,
    NULL::character varying AS group_name,
    NULL::character varying AS group_description,
    perm.permission_id,
    perm.name AS permission_name,
    perm.description AS permission_description,
    perm.type AS permission_type,
    perm.application_name AS permission_application_name,
    perm.risk AS permission_risk,
    sav.permission AS permission_root,
    spv.policy_name AS permission_policy_violated,
    string_agg(ent.associated_permission, ' | ' ORDER BY (ent.associated_permission)) AS permission_entitlements,
    NULL::bigint AS brole_id,
    NULL AS brole_name,
    NULL AS brole_description,
    NULL::bigint AS brole_risk,
    NULL::timestamp with time zone AS brole_update_time,
    NULL AS supervisee_id,
    NULL AS supervisee_display_name,
    NULL AS supervisee_email,
    NULL AS supervisee_department,
    NULL AS supervisee_job_title,
    NULL::bigint AS trole_policy_id,
    NULL AS trole_item_name,
    NULL AS trole_unique_policy_id,
    NULL AS trole_description,
    NULL::bigint AS trole_risk,
    NULL AS trole_policy_violated,
    NULL AS account_name,
    NULL AS account_id,
    NULL::bigint AS account_risk,
    NULL AS account_description,
    NULL AS account_application_name
  FROM suser users
  LEFT JOIN (SELECT perm.id permission_id, sap.suser, perm.name, perm.description, perm.type, data_source.name application_name, perm.risk 
            FROM spermission perm JOIN data_source ON perm.application = data_source.id 
            JOIN suser_account_permission sap ON perm.id = sap.spermission 
            WHERE (exclude_from_catalog = false OR exclude_from_catalog IS NULL) AND effective = true) perm ON users.id = perm.suser
  LEFT JOIN (SELECT DISTINCT perm.id, perm_assoc.name AS associated_permission 
            FROM spermission_entailment perm_entail
            LEFT JOIN spermission perm ON perm.collected_entity = perm_entail.entailing
            LEFT JOIN spermission perm_assoc ON perm_assoc.collected_entity = perm_entail.entailed
            WHERE ((((perm.deleted = false) AND (perm.effective = true)) AND (perm_assoc.deleted = false)) AND (perm_assoc.effective = true))) ent ON ent.id = perm.permission_id
  LEFT JOIN spermission perm1 ON perm1.id = perm.permission_id AND perm1.effective = true AND perm1.deleted = false
  LEFT JOIN (SELECT DISTINCT perm.name permission, perm.id, perm_entail.entailed AS associated_permission_id
            FROM spermission_entailment perm_entail
            LEFT JOIN spermission perm ON perm.collected_entity = perm_entail.entailing
            LEFT JOIN spermission perm_assoc ON perm_assoc.collected_entity = perm_entail.entailed
            WHERE perm.deleted = false AND perm.effective = true AND perm_assoc.deleted = false AND perm_assoc.effective = true) sav ON sav.associated_permission_id = perm1.collected_entity
  LEFT JOIN (SELECT sv.suser, sp.name policy_name, sod_condition_item.unique_id
            FROM sod_policy sp
            LEFT JOIN sod_violation sv ON sp.id = sv.policy 
            LEFT JOIN sod_condition ON sod_condition.policy = sp.id
            LEFT JOIN sod_condition_item ON sod_condition_item.condition = sod_condition.id) spv ON spv.unique_id = perm1.unique_permission_id AND users.collected_entity = spv.suser
  LEFT JOIN (SELECT DISTINCT suser1.id AS suser_id,
              string_agg(suser_email1.email, ' | ' ORDER BY (suser_email1.email)) AS email
            FROM suser suser1
            JOIN suser_email suser_email1 ON suser1.id = suser_email1.suser AND suser1.effective = true AND suser1.deleted = false
            GROUP BY suser1.id) suser_email ON users.id = suser_email.suser_id
  LEFT JOIN suser supervisor ON supervisor.unique_user_id = users.supervisor_unique_id AND supervisor.effective = true AND supervisor.deleted = false
  LEFT JOIN ( SELECT 
              CASE 
                WHEN sv.suser IS NOT NULL THEN su.display_name
                ELSE sa.name
              END AS violator_name,
              CASE 
                WHEN sv.suser IS NOT NULL THEN su.unique_user_id
                ELSE sa.unique_account_id
              END AS violator_id,
              string_agg(concat(sp.name, concat(': SOD-', lpad(UPPER(to_hex(sc.id))::character varying::text, 4, '0'::text))),
                ', ' ORDER BY su.unique_user_id, sa.unique_account_id) AS violations
            FROM sod_case sc
            LEFT JOIN sod_policy sp ON sc.policy = sp.id
            LEFT JOIN sod_violation sv ON sc.current_violation = sv.id
            LEFT JOIN suser su ON sv.suser = su.id
            LEFT JOIN saccount sa ON sv.saccount = sa.id
            GROUP BY violator_name, violator_id) sci ON sci.violator_id = users.unique_user_id  
  WHERE users.effective = true AND users.deleted = false
  GROUP BY users.id, users.display_name, users.job_title, suser_email.email, users.department, users.user_id, users.first_name, users.last_name, users.risk, users.unique_user_id, supervisor.display_name, sci.violations, perm.permission_id, perm.name, perm.description, perm.type, perm.application_name, perm.risk, sav.permission, spv.policy_name

  UNION ALL

SELECT 
    users.id,
    users.display_name,
    users.job_title,
    string_agg(distinct suser_email.email, ' | ' order by suser_email.email) AS email,
    users.department,
    users.user_id,
    users.first_name,
    users.last_name,
    users.risk,
    users.unique_user_id,
    supervisor.display_name AS supervisor,
    sci.violations,
    CASE
    WHEN br.id IS NOT NULL THEN '3-BROLE'
    ELSE '3-NO-BROLE'
    END AS detail_type,
    NULL::character varying AS group_id,
    NULL AS group_name,
    NULL AS group_description,
    NULL::bigint AS permission_id,
    NULL AS permission_name,
    NULL AS permission_description,
    NULL AS permission_type,
    NULL AS permission_application_name,
    NULL::bigint AS permission_risk,
    NULL AS permission_root,
    NULL AS permission_policy_violated,
    NULL AS permission_entitlements,
    br.id AS brole_id,
    br.name AS brole_name,
    br.description AS brole_description,
    br.risk AS brole_risk,
    br.update_time AS brole_update_time,
    NULL AS supervisee_id,
    NULL AS supervisee_display_name,
    NULL AS supervisee_email,
    NULL AS supervisee_department,
    NULL AS supervisee_job_title,
    NULL::bigint AS trole_policy_id,
    NULL AS trole_item_name,
    NULL AS trole_unique_policy_id,
    NULL AS trole_description,
    NULL::bigint AS trole_risk,
    NULL AS trole_policy_violated,
    NULL AS account_name,
    NULL AS account_id,
    NULL::bigint AS account_risk,
    NULL AS account_description,
    NULL AS account_application_name
  FROM business_role_member brm
  LEFT JOIN business_role br ON brm.unique_business_role_id = br.unique_id AND br.approval_state = 'PUBLISHED'
  RIGHT JOIN suser users ON users.unique_user_id = brm.unique_user_id
  LEFT JOIN suser_email ON users.id = suser_email.suser
  LEFT JOIN suser supervisor ON supervisor.unique_user_id = users.supervisor_unique_id AND supervisor.effective = true AND supervisor.deleted = false
  LEFT JOIN ( SELECT 
              CASE 
                WHEN sv.suser IS NOT NULL THEN su.display_name
                ELSE sa.name
              END AS violator_name,
              CASE 
                WHEN sv.suser IS NOT NULL THEN su.unique_user_id
                ELSE sa.unique_account_id
              END AS violator_id,
              string_agg(concat(sp.name, concat(': SOD-', lpad(UPPER(to_hex(sc.id))::character varying::text, 4, '0'::text))),
                ', ' ORDER BY su.unique_user_id, sa.unique_account_id) AS violations
              FROM sod_case sc
              LEFT JOIN sod_policy sp ON sc.policy = sp.id
              LEFT JOIN sod_violation sv ON sc.current_violation = sv.id
              LEFT JOIN suser su ON sv.suser = su.id
              LEFT JOIN saccount sa ON sv.saccount = sa.id
              GROUP BY violator_name, violator_id) sci ON sci.violator_id = users.unique_user_id  
  WHERE users.effective = true AND users.deleted = false and brm.deleted = false
  GROUP BY users.id, supervisor.display_name, sci.violations, br.id, br.name, br.description, br.risk, br.update_time
     
  UNION ALL

  SELECT 
    users.id,
    users.display_name,
    users.job_title,
    string_agg(distinct suser_email.email, ' | ' order by suser_email.email) AS email,
    users.department,
    users.user_id,
    users.first_name,
    users.last_name,
    users.risk,
    users.unique_user_id,
    supervisor.display_name AS supervisor,
    sci.violations,
    CASE
    WHEN supervisee.unique_user_id IS NOT NULL THEN '4-SUPERVISEE' 
    ELSE '4-NO-SUPERVISEE'
    END AS detail_type,
    NULL::character varying AS group_id,
    NULL AS group_name,
    NULL AS group_description,
    NULL::bigint AS permission_id,
    NULL AS permission_name,
    NULL AS permission_description,
    NULL AS permission_type,
    NULL AS permission_application_name,
    NULL::bigint AS permission_risk,
    NULL AS permission_root,
    NULL AS permission_policy_violated,
    NULL AS permission_entitlements,
    NULL::bigint AS brole_id,
    NULL AS brole_name,
    NULL AS brole_description,
    NULL::bigint AS brole_risk,
    NULL::timestamp with time zone AS brole_update_time,
    supervisee.unique_user_id AS supervisee_id,
    supervisee.display_name AS supervisee_display_name,
    supervisee_email.email  supervisee_email,
    supervisee.department AS supervisee_department,
    supervisee.job_title AS supervisee_job_title,
    NULL::bigint AS trole_policy_id,
    NULL AS trole_item_name,
    NULL AS trole_unique_policy_id,
    NULL AS trole_description,
    NULL::bigint AS trole_risk,
    NULL AS trole_policy_violated,
    NULL AS account_name,
    NULL AS account_id,
    NULL::bigint AS account_risk,
    NULL AS account_description,
    NULL AS account_application_name
  FROM suser users
  LEFT JOIN suser supervisee ON supervisee.supervisor_unique_id = users.unique_user_id AND supervisee.deleted = '0' AND supervisee.effective = '1'
  LEFT JOIN suser supervisor ON supervisor.unique_user_id = users.supervisor_unique_id AND supervisor.effective = true AND supervisor.deleted = false
  LEFT JOIN suser_email ON users.id = suser_email.suser
  LEFT JOIN (SELECT suser.id suser, suser.display_name,
              string_agg(suser_email.email, ' | ' order by suser_email.email) As email
            FROM suser
            JOIN suser_email ON suser.id = suser_email.suser AND suser.effective = '1' AND suser.deleted = '0'   
            GROUP BY suser.id, suser.display_name) supervisee_email ON supervisee.id = supervisee_email.suser
  LEFT JOIN ( SELECT 
                CASE 
                  WHEN sv.suser IS NOT NULL THEN su.display_name
                  ELSE sa.name
                END AS violator_name,
                CASE 
                  WHEN sv.suser IS NOT NULL THEN su.unique_user_id
                  ELSE sa.unique_account_id
                END AS violator_id,
                string_agg(concat(sp.name, concat(': SOD-', lpad(UPPER(to_hex(sc.id))::character varying::text, 4, '0'::text))),
                  ', ' ORDER BY su.unique_user_id, sa.unique_account_id) AS violations
            FROM sod_case sc
            LEFT JOIN sod_policy sp ON sc.policy = sp.id
            LEFT JOIN sod_violation sv ON sc.current_violation = sv.id
            LEFT JOIN suser su ON sv.suser = su.id
            LEFT JOIN saccount sa ON sv.saccount = sa.id
            GROUP BY violator_name, violator_id) sci ON sci.violator_id = users.unique_user_id  
  WHERE users.effective = true AND users.deleted = false
  GROUP BY users.id, supervisor.display_name, sci.violations, supervisee.unique_user_id, supervisee.display_name, supervisee_email.email, supervisee.department, supervisee.job_title

  UNION ALL

  SELECT DISTINCT
    users.id,
    users.display_name,
    users.job_title,
    string_agg(distinct suser_email.email, ' | ' order by suser_email.email) AS email,
    users.department,
    users.user_id,
    users.first_name,
    users.last_name,
    users.risk,
    users.unique_user_id,
    supervisor.display_name AS supervisor,
    sci.violations,
    CASE
      WHEN rperm.policy IS NOT NULL THEN '5-TROLE'
      ELSE '5-NO-TROLE'
    END AS detail_type,
    NULL::character varying AS group_id,
    NULL::character varying AS group_name,
    NULL::character varying AS group_description,
    NULL::bigint AS permission_id,
    NULL AS permission_name,
    NULL AS permission_description,
    NULL AS permission_type,
    NULL AS permission_application_name,
    NULL::bigint AS permission_risk,
    NULL AS permission_root,
    NULL AS permission_policy_violated,
    NULL AS permission_entitlements,
    NULL::bigint AS brole_id,
    NULL AS brole_name,
    NULL AS brole_description,
    NULL::bigint AS brole_risk,
    NULL::timestamp with time zone AS brole_update_time,
    NULL AS supervisee_id,
    NULL AS supervisee_display_name,
    NULL AS supervisee_email,
    NULL AS supervisee_department,
    NULL AS supervisee_job_title,
    rperm.policy AS trole_policy_id,
    rpolicy.name AS trole_item_name,
    rpolicy.unique_policy_id AS trole_unique_policy_id,
    rpolicy.description AS trole_description,
    rpolicy.risk AS trole_risk,
    spv.violations trole_policy_violated,
    NULL AS account_name,
    NULL AS account_id,
    NULL::bigint AS account_risk,
    NULL AS account_description,
    NULL AS account_application_name
  FROM suser users
  LEFT JOIN trole_users ruser ON users.unique_user_id = ruser.unique_user_id
  LEFT JOIN role_policy rpolicy ON rpolicy.id = ruser.policy
  LEFT JOIN role_permission rperm ON ruser.policy = rperm.policy
  LEFT JOIN sod_condition_item sod ON sod.unique_id = rpolicy.unique_policy_id AND users.id = sod.id
  LEFT JOIN sod_condition ON sod.condition = sod_condition.id
  LEFT JOIN sod_policy ON sod_condition.policy = sod_policy.id
  LEFT JOIN suser_email ON users.id = suser_email.suser
  LEFT JOIN suser supervisor ON supervisor.unique_user_id = users.supervisor_unique_id AND supervisor.effective = true AND supervisor.deleted = false
  LEFT JOIN ( SELECT 
                CASE 
                  WHEN sv.suser IS NOT NULL THEN su.display_name
                  ELSE sa.name
                END AS violator_name,
                CASE 
                  WHEN sv.suser IS NOT NULL THEN su.unique_user_id
                  ELSE sa.unique_account_id
                END AS violator_id,
                string_agg(concat(sp.name, concat(': SOD-', lpad(UPPER(to_hex(sc.id))::character varying::text, 4, '0'::text))),
                ', ' ORDER BY su.unique_user_id, sa.unique_account_id) AS violations
              FROM sod_case sc
              LEFT JOIN sod_policy sp ON sc.policy = sp.id
              LEFT JOIN sod_violation sv ON sc.current_violation = sv.id
              LEFT JOIN suser su ON sv.suser = su.id
              LEFT JOIN saccount sa ON sv.saccount = sa.id
              GROUP BY violator_name, violator_id) sci ON sci.violator_id = users.unique_user_id  
  LEFT JOIN (SELECT sc.id AS case_id,
              CASE
                WHEN sv.suser IS NOT NULL THEN su.display_name
                ELSE sa.name
              END AS violator_name,
              CASE
                WHEN sod_condition_item.type::text = 'PERMISSION'::text THEN spermission_1.name
                ELSE role_policy.name
              END AS item_name,
              sp.name AS policy_name,
              su.unique_user_id,
              sod_condition_item.unique_id,
              string_agg(concat(sp.name, concat(': SOD-', lpad(upper(to_hex(sc.id)), 4, '0'::text))), ', '::text ORDER BY su.display_name) AS violations
             FROM sod_case sc
             LEFT JOIN sod_policy sp ON sc.policy = sp.id
             LEFT JOIN sod_violation sv ON sc.current_violation = sv.id
             LEFT JOIN suser su ON sv.suser = su.collected_entity AND su.effective = true AND su.deleted = false
             LEFT JOIN saccount sa ON sv.saccount = sa.id
             LEFT JOIN sod_condition ON sod_condition.policy = sc.policy
             LEFT JOIN sod_condition_item ON sod_condition_item.condition = sod_condition.id
             LEFT JOIN spermission spermission_1 ON spermission_1.unique_permission_id = sod_condition_item.unique_id AND spermission_1.effective = true AND spermission_1.deleted = false
             LEFT JOIN role_policy ON role_policy.unique_policy_id = sod_condition_item.unique_id AND role_policy.deleted = false
             GROUP BY sc.id, sv.suser, su.display_name, sa.name, sod_condition_item.type, spermission_1.name, role_policy.name, su.unique_user_id, sod_condition_item.unique_id, sp.name) spv ON spv.unique_user_id = users.unique_user_id AND spv.unique_id =  rpolicy.unique_policy_id
  WHERE users.effective = true AND users.deleted = false AND (rpolicy.deleted = false OR rpolicy.ID IS NULL)
  GROUP BY users.id, supervisor.display_name, sci.violations, rperm.policy, rpolicy.name, rpolicy.unique_policy_id, rpolicy.description, rpolicy.risk, sod_policy.name, spv.violations
 
  UNION ALL

  SELECT DISTINCT
    users.id,
    users.display_name,
    users.job_title,
    string_agg(distinct suser_email.email, ' | ' order by suser_email.email) AS email,
    users.department,
    users.user_id,
    users.first_name,
    users.last_name,
    users.risk,
    users.unique_user_id,
    supervisor.display_name AS supervisor,
    sci.violations,
    CASE
        WHEN sac.account_id IS NOT NULL THEN '6-ACCOUNT'
        ELSE '6-NO-ACCOUNT'
    END AS detail_type,
    NULL::character varying AS group_id,
    NULL::character varying AS group_name,
    NULL::character varying AS group_description,
    NULL::bigint AS permission_id,
    NULL AS permission_name,
    NULL AS permission_description,
    NULL AS permission_type,
    NULL AS permission_application_name,
    NULL::bigint AS permission_risk,
    NULL AS permission_root,
    NULL AS permission_policy_violated,
    NULL AS permission_entitlements,
    NULL::bigint AS brole_id,
    NULL AS brole_name,
    NULL AS brole_description,
    NULL::bigint AS brole_risk,
    NULL::timestamp with time zone AS brole_update_time,
    NULL AS supervisee_id,
    NULL AS supervisee_display_name,
    NULL AS supervisee_email,
    NULL AS supervisee_department,
    NULL AS supervisee_job_title,
    NULL::bigint AS trole_policy_id,
    NULL AS trole_item_name,
    NULL AS trole_unique_policy_id,
    NULL AS trole_description,
    NULL::bigint AS trole_risk,
    NULL AS trole_policy_violated,
    sac.name AS account_name,
    sac.account_id,
    sac.risk AS account_risk,
    sac.description AS account_description,
    source.name AS account_application_name
  FROM suser_account_permission sap
  JOIN saccount sac ON sap.saccount = sac.id AND sac.effective = '1' AND sac.deleted = '0' AND sac.virtual = '0'
  JOIN data_source source ON sac.application = source.id
  RIGHT JOIN suser users ON sap.suser = users.id
  LEFT JOIN suser_email ON users.id = suser_email.suser
  LEFT JOIN suser supervisor ON supervisor.unique_user_id = users.supervisor_unique_id AND supervisor.effective = true AND supervisor.deleted = false
  LEFT JOIN (SELECT 
                CASE 
                  WHEN sv.suser IS NOT NULL THEN su.display_name
                  ELSE sa.name
                END AS violator_name,
                CASE 
                  WHEN sv.suser IS NOT NULL THEN su.unique_user_id
                  ELSE sa.unique_account_id
                END AS violator_id,
                string_agg(concat(sp.name, concat(': SOD-', lpad(UPPER(to_hex(sc.id))::character varying::text, 4, '0'::text))),
                  ', ' ORDER BY su.unique_user_id, sa.unique_account_id) AS violations
              FROM sod_case sc
              LEFT JOIN sod_policy sp ON sc.policy = sp.id
              LEFT JOIN sod_violation sv ON sc.current_violation = sv.id
              LEFT JOIN suser su ON sv.suser = su.id
              LEFT JOIN saccount sa ON sv.saccount = sa.id
              GROUP BY violator_name, violator_id) sci ON sci.violator_id = users.unique_user_id  
  WHERE users.effective = true AND users.deleted = false
  GROUP BY users.id, supervisor.display_name, sci.violations, sac.account_id, sac.name, sac.risk, sac.description, source.name
     
)
  SELECT 
    current_user_access.id,
    current_user_access.display_name,
    current_user_access.job_title,
    current_user_access.email,
    current_user_access.department,
    current_user_access.user_id AS user_cn,
    current_user_access.first_name,
    current_user_access.last_name,
    current_user_access.risk,
    current_user_access.unique_user_id,
    current_user_access.supervisor,
    current_user_access.violations,
    current_user_access.detail_type,
    current_user_access.group_id,
    current_user_access.group_name,
    current_user_access.group_description,
    current_user_access.permission_id,
    current_user_access.permission_name,
    current_user_access.permission_description,
    current_user_access.permission_type,
    current_user_access.permission_application_name,
    current_user_access.permission_risk,
    current_user_access.permission_root,
    current_user_access.permission_policy_violated,
    current_user_access.permission_entitlements,
    current_user_access.brole_id,
    current_user_access.brole_name,
    current_user_access.brole_description,
    current_user_access.brole_risk,
    current_user_access.brole_update_time,
    current_user_access.supervisee_id,
    current_user_access.supervisee_display_name,
    current_user_access.supervisee_email,
    current_user_access.supervisee_department,
    current_user_access.supervisee_job_title,
    current_user_access.trole_policy_id,
    current_user_access.trole_item_name,
    current_user_access.trole_unique_policy_id,
    current_user_access.trole_description,
    current_user_access.trole_risk,
    current_user_access.trole_policy_violated,
    current_user_access.account_name,
    current_user_access.account_id,
    current_user_access.account_risk,
    current_user_access.account_description,
    current_user_access.account_application_name
  FROM current_user_access;

CREATE VIEW auth_user_resource_v AS 
SELECT
	br.name AS brole,
	br.unique_id AS unique_business_role_id,
	su.display_name AS user_name,
	su.unique_user_id,
	sp.name AS permission_name,
	sp.unique_permission_id,
	rp.name AS role_name,
	rp.unique_policy_id,
	ds.name AS application,
	ds.unique_id AS unique_application_id,
	aur.start_date,
	aur.end_date,
	aur.in_validity_period,
	aur.in_grace_period,
	aur.indirect_authorization,
	aur.auto_grant,
	aur.auto_revoke,
	aur.create_date,
	aur.deleted,
	aur.delete_date
FROM auth_user_resource aur
	JOIN user_resource ur ON aur.user_resource_id = ur.id
	JOIN business_role br ON aur.unique_business_role_id::text = br.unique_id::text AND br.approval_state::text = 'PUBLISHED'::text
	JOIN suser su ON su.unique_user_id::text = ur.unique_user_id::text AND su.effective = true
	LEFT JOIN spermission sp ON sp.unique_permission_id::text = ur.unique_resource_id::text AND sp.effective = true
	LEFT JOIN role_policy rp ON rp.unique_policy_id::text = ur.unique_resource_id::text AND rp.deleted = false
	LEFT JOIN data_source ds ON (ds.id::text = ur.unique_resource_id::text OR ds.id = sp.application) AND ds.deleted = false;

CREATE VIEW role_perm_policy_v AS
SELECT DISTINCT 
  policy.id,
  policy.name AS policy_name,
  policy.unique_policy_id,
  policy.description AS policy_description,
  policy.created_by,
  policy.changed_by,
  policy.creation_time,
  policy.update_time,
  policy.deleted,
  policy.last_detection_time,
  CASE
    WHEN policy.state = 'INACTIVE' AND policy.detection_status IS NULL OR policy.detection_status = 'INACTIVE' THEN 'Inactive'
    WHEN policy.state = 'ACTIVE' AND policy.detection_status = 'COMPLETED' OR policy.detection_status = 'RUNNING' THEN 'Active'
    WHEN policy.state = 'INACTIVE' AND policy.detection_status = 'INVALID' THEN 'Invalid from inactive'
    WHEN policy.state = 'INVALID' AND policy.detection_status = 'INVALID' THEN 'Invalid from active'
    ELSE NULL
  END AS current_state,
  policy.state AS actual_state,
  policy.detection_status,
  policy.risk_mode AS policy_risk_mode,
  policy.risk AS policy_risk,
  policy.cost AS policy_cost,
  ( SELECT DISTINCT string_agg(subquery.unique_owner_id, ' | ') AS user_owner_id
    FROM ( SELECT DISTINCT powner.unique_owner_id
      FROM policy_owners_v powner
      LEFT JOIN role_policy rperm ON rperm.id = powner.role_policy
      WHERE powner.is_user = true AND policy.id = rperm.id) subquery) AS user_owner_id,
  ( SELECT DISTINCT string_agg(subquery2.unique_owner_id, ' | ') AS group_owner_id
    FROM ( SELECT DISTINCT powner.unique_owner_id
      FROM policy_owners_v powner
      LEFT JOIN role_policy rperm ON rperm.id = powner.role_policy
      WHERE powner.is_user = false AND policy.id = rperm.id) subquery2) AS group_owner_id,
  ( SELECT DISTINCT string_agg(subquery.owner_name, ' | ') AS user_owners
    FROM ( SELECT DISTINCT powner.owner_name
      FROM policy_owners_v powner
      LEFT JOIN role_policy rperm ON rperm.id = powner.role_policy
      WHERE powner.is_user = true AND policy.id = rperm.id) subquery) AS user_owners,
  ( SELECT DISTINCT string_agg(subquery2.owner_name, ' | ') AS group_owners
    FROM ( SELECT DISTINCT powner.owner_name
      FROM policy_owners_v powner
      LEFT JOIN role_policy rperm ON rperm.id = powner.role_policy
      WHERE powner.is_user = false AND policy.id = rperm.id) subquery2) AS group_owners,
  perm.unique_permission_id,
  perm.name AS perm_name,
  perm_user.unique_user_id AS permission_user_id,
  perm_user.display_name AS permission_user_name
FROM 
  role_policy policy
  LEFT JOIN policy_owner ON policy.id = policy_owner.role_policy
  LEFT JOIN role_permission role_perm ON policy.id = role_perm.policy
  LEFT JOIN spermission perm ON role_perm.unique_permission_id = perm.unique_permission_id AND perm.effective = true
  LEFT JOIN suser_account_permission_role role_user ON role_user.role = policy.id
  LEFT JOIN suser perm_user ON perm_user.id = role_user.suser;


ALTER TABLE current_user_access_v
  OWNER TO igops;
GRANT ALL ON TABLE current_user_access_v TO igops;
GRANT SELECT ON TABLE current_user_access_v TO ig_report_role;

ALTER TABLE auth_user_resource_v
  OWNER TO igops;
GRANT ALL ON TABLE auth_user_resource_v TO igops;
GRANT SELECT ON TABLE auth_user_resource_v TO ig_report_role;

ALTER TABLE role_perm_policy_v
  OWNER TO igops;
GRANT ALL ON TABLE role_perm_policy_v TO igops;
GRANT SELECT ON TABLE role_perm_policy_v TO ig_report_role;