DROP VIEW IF EXISTS risk_score_calculation_v;

DROP VIEW IF EXISTS risk_score_overview_v;

DROP VIEW IF EXISTS risk_score_schedule_v;

CREATE VIEW risk_score_calculation_v AS 
 SELECT rfscfg.operation,
    rfscfg.lower_limit,
    rfscfg.upper_limit,
    rfscfg.factor_weight,
    rfs.base_rf_score,
    rfs.ranged_rf_score,
    rfs.weighted_rf_score,
    rfs.calculation_time,
    rfscfg.rfsc_version,
    rfscfg.update_time,
    rfscfg.creation_time,
    rfscfg.deleted,
    rfscfg.id,
    rfscfg.factor_attr_key,
    rfscfg.risk_condition,
    rfscfg.factor_unique_id,
        CASE
            WHEN substr(rfscfg.factor_unique_id::text, 1, 1) = 'G'::text THEN 'GOVERNANCE'::text
            WHEN substr(rfscfg.factor_unique_id::text, 1, 1) = 'U'::text THEN 'USER'::text
            WHEN substr(rfscfg.factor_unique_id::text, 1, 1) = 'A'::text THEN 'APPLICATION'::text
            ELSE NULL::text
        END AS entity_type
   FROM risk_factor_score_cfg rfscfg
     RIGHT JOIN ( SELECT r.base_rf_score,
            r.ranged_rf_score,
            r.weighted_rf_score,
            r.rf_unique_id,
            r.rfs_cfg_id,
            r.calculation_time,
            r.entity_type
           FROM ( SELECT risk_factor_score.rf_unique_id,
                    risk_factor_score.calculation_time,
                    risk_factor_score.base_rf_score,
                    risk_factor_score.ranged_rf_score,
                    risk_factor_score.weighted_rf_score,
                    risk_factor_score.rfs_cfg_id,
                    risk_factor_score.entity_type,
                    rank() OVER (PARTITION BY risk_factor_score.rf_unique_id ORDER BY risk_factor_score.calculation_time DESC) AS rnk
                   FROM risk_factor_score) r
          WHERE r.rnk = 1) rfs ON rfs.rfs_cfg_id = rfscfg.id
  WHERE rfscfg.deleted = false;

CREATE VIEW risk_score_overview_v AS 
 SELECT cfg.entity_type,
    cfg.risk_score_type,
    to_char(cfg.update_time, 'DD-MON-YY HH12:MI'::text) AS updated_time_mod,
    cfg.update_time,
    ct.calculation_time,
    cfg.id,
    cfg.ops_sequence,
    rs.risk_score
   FROM risk_score_cfg cfg
     LEFT JOIN ( SELECT rs_1.risk_score,
            rs_1.calculation_time,
            rs_1.entity_type,
            rs_1.entity_unique_id
           FROM ( SELECT risk_score.risk_score,
                    risk_score.calculation_time,
                    risk_score.entity_type,
                    risk_score.entity_unique_id,
                    rank() OVER (PARTITION BY risk_score.entity_unique_id ORDER BY risk_score.calculation_time DESC) AS rnk
                   FROM risk_score
                  WHERE risk_score.entity_type::text = 'GOVERNANCE'::text) rs_1
          WHERE rs_1.rnk = 1) rs ON rs.entity_type::text = cfg.entity_type::text
     JOIN ( SELECT risk_score.entity_type,
            max(risk_score.calculation_time) AS calculation_time
           FROM risk_score
          GROUP BY risk_score.entity_type) ct ON ct.entity_type::text = cfg.entity_type::text;

CREATE VIEW risk_score_schedule_v AS
SELECT 
  active,
    name,
    publish,
    unique_id,
    job_trigger,
    job_schedule_kind,
    entity_type,
    last_day_of_month,
    repeat,
    start_date,
    time_zone_id,
  description,
  end_date,
  interval,
  interval_unit,
  jt.cron_spec
FROM job_schedule js
JOIN job_trigger jt ON jt.qs_trigger_name = js.qs_job_name
WHERE job_schedule_kind = 'RISK_SCORING';

ALTER TABLE risk_score_schedule_v
  OWNER TO igops;
GRANT ALL ON TABLE risk_score_schedule_v TO igops;
GRANT SELECT ON TABLE risk_score_schedule_v TO ig_report_role;

ALTER TABLE risk_score_overview_v
  OWNER TO igops;
GRANT ALL ON TABLE risk_score_overview_v TO igops;
GRANT SELECT ON TABLE risk_score_overview_v TO ig_report_role;

ALTER TABLE risk_score_calculation_v
  OWNER TO igops;
GRANT ALL ON TABLE risk_score_calculation_v TO igops;
GRANT SELECT ON TABLE risk_score_calculation_v TO ig_report_role;