USE [igops]
GO

DROP VIEW IF EXISTS business_role_v
GO

CREATE VIEW business_role_v AS
  SELECT b_role.id AS business_role_id,
    b_role.unique_id,
    b_role.name,
    b_role.description,
    b_role.grace_period,
    b_role.risk,
    b_role.update_time,
    user_owners.user_owners AS user_owners,
    group_owners.group_owners AS group_owners,
    b_role.approval_state,
    su1.display_name AS created_by,
    su2.display_name AS published_by,
    su3.display_name AS changed_by,
    su4.display_name AS approved_by,
    user_managers.user_managers AS user_managers,
    group_managers.group_managers AS group_managers,
    su5.display_name AS user_fulfiller,
    sgroup.name AS group_fulfiller,
    b_role.approval_note,
    CASE WHEN b_role.approval_policy IS NULL THEN (SELECT name FROM approval_policy WHERE global = 1 AND deleted = 0)
    ELSE ap.name END AS policy_name,
    
    CASE WHEN b_role.approval_policy IS NULL THEN (SELECT implicit FROM approval_policy WHERE global = 1 AND deleted = 0)
    ELSE ap.implicit END AS approval_not_required, 
    
    CASE WHEN b_role.approval_policy IS NULL THEN (SELECT description FROM approval_policy WHERE global = 1 AND deleted = 0)
    ELSE ap.description END AS policy_description, 
    
    CASE WHEN b_role.approval_policy IS NULL THEN (SELECT creation_time FROM approval_policy WHERE global = 1 AND deleted = 0)
    ELSE ap.creation_time END AS policy_creation_time,
    
    CASE WHEN b_role.approval_policy IS NULL THEN (SELECT update_time FROM approval_policy WHERE global = 1 AND deleted = 0)
    ELSE ap.update_time END AS last_policy_update_time,
    pol_users.policy_users AS policy_users,
    pol_groups.policy_groups AS policy_groups,
    br_crit.criteria AS membership_criteria,
    incl_users.included_users AS included_users,
    incl_groups.included_groups AS included_groups,
    excl_users.excluded_users AS excluded_users,
    excl_groups.excluded_groups AS excluded_groups,
    auth_perm.auth_permission,
    auth_tech_role.auth_tech_role AS auth_tech_role,
    auth_app.auth_application AS auth_application,
    incl_brole.bnames AS included_broles,
    b_role.last_detect_time
   FROM business_role b_role
     LEFT JOIN suser su1 ON b_role.created_by = su1.unique_user_id AND su1.display_name IS NOT NULL AND su1.deleted = 0 AND su1.effective = 1
     LEFT JOIN suser su2 ON b_role.published_by = su2.unique_user_id AND su2.display_name IS NOT NULL AND su2.deleted = 0 AND su2.effective = 1
     LEFT JOIN suser su3 ON b_role.changed_by = su3.unique_user_id AND su3.display_name IS NOT NULL AND su3.deleted = 0 AND su3.effective = 1
     LEFT JOIN suser su4 ON b_role.approved_by = su4.unique_user_id AND su4.display_name IS NOT NULL AND su4.deleted = 0 AND su4.effective = 1
     JOIN auth_scope ON auth_scope.unique_business_role_id = b_role.unique_id
     LEFT JOIN auth_role_mapping ar_map ON ar_map.auth_scope = auth_scope.id AND ar_map.auth_role = 29
     LEFT JOIN ( SELECT DISTINCT auth_scope.unique_business_role_id,
  STUFF( ( SELECT ' | ' + CAST(manager_names.display_name AS nvarchar(MAX))
    FROM ( SELECT DISTINCT display_name, su.unique_user_id, auth_scope, auth_role
      FROM auth_role_mapping ar_map
      JOIN suser su ON ar_map.unique_user_id = su.unique_user_id AND su.display_name IS NOT NULL AND su.deleted = 0 AND su.effective = 1) manager_names
      WHERE  manager_names.auth_scope = auth_scope.id AND manager_names.unique_user_id IS NOT NULL AND manager_names.auth_role = 27
      ORDER BY manager_names.display_name FOR XML PATH('')), 1, 3, '') AS user_managers
  FROM auth_scope) user_managers ON user_managers.unique_business_role_id = b_role.unique_id
     LEFT JOIN ( SELECT DISTINCT auth_scope_1.unique_business_role_id,
  STUFF( ( SELECT ' | ' + CAST(group1_names.name AS nvarchar(MAX))
    FROM ( SELECT DISTINCT name, sgroup.unique_group_id, auth_scope, auth_role
      FROM auth_role_mapping ar_map_1
      JOIN sgroup ON ar_map_1.unique_group_id = sgroup.unique_group_id AND sgroup.name IS NOT NULL) group1_names
      WHERE group1_names.auth_scope = auth_scope_1.id AND group1_names.unique_group_id IS NOT NULL AND group1_names.auth_role = 27
      ORDER BY group1_names.name FOR XML PATH('')), 1, 3, '') AS group_managers
  FROM auth_scope auth_scope_1) group_managers ON group_managers.unique_business_role_id = b_role.unique_id
     LEFT JOIN suser su5 ON ar_map.unique_user_id = su5.unique_user_id AND su5.display_name IS NOT NULL AND su5.deleted = 0 AND su5.effective = 1
     LEFT JOIN sgroup ON ar_map.unique_group_id = sgroup.unique_group_id AND sgroup.name IS NOT NULL AND sgroup.deleted = 0 AND sgroup.effective = 1
     LEFT JOIN approval_policy ap ON b_role.approval_policy = ap.id
     LEFT JOIN business_role_memb_crit br_crit ON b_role.id = br_crit.business_role_id
LEFT JOIN ( SELECT incl_brole2.business_role_id,
          STUFF((SELECT ' | ' + CAST(incl_brole1.name AS nvarchar(MAX)) FROM (SELECT DISTINCT name, business_role_id from business_role_incl_brole) incl_brole1
          WHERE incl_brole1.business_role_id = incl_brole2.business_role_id
          ORDER BY incl_brole1.name FOR XML PATH('')), 1, 3, '') AS bnames
          FROM business_role_incl_brole incl_brole2
          GROUP BY incl_brole2.business_role_id) incl_brole ON incl_brole.business_role_id = b_role.id 

LEFT JOIN ( SELECT DISTINCT br_owner_1.business_role_id,
          STUFF((SELECT ' | ' + CAST(owner_names.display_name AS nvarchar(MAX)) FROM (SELECT DISTINCT display_name, business_role_id from business_role_owner br_owner_1 JOIN suser su ON br_owner_1.unique_owner_id = su.unique_user_id AND su.display_name IS NOT NULL AND su.deleted = 0 AND su.effective = 1) owner_names
                WHERE  owner_names.business_role_id = br_owner_1.business_role_id
                ORDER BY owner_names.display_name FOR XML PATH('')), 1, 3, '') AS user_owners
          FROM business_role_owner br_owner_1
        GROUP BY br_owner_1.business_role_id) user_owners ON user_owners.business_role_id = b_role.id

LEFT JOIN ( SELECT DISTINCT br_owner_1.business_role_id,
          STUFF((SELECT ' | ' + CAST(group_names.name AS nvarchar(MAX)) FROM (SELECT DISTINCT name, business_role_id from business_role_owner br_owner_1 JOIN sgroup sgroup_1 ON br_owner_1.unique_owner_id = sgroup_1.unique_group_id AND sgroup_1.name IS NOT NULL AND sgroup_1.deleted = 0 AND sgroup_1.effective = 1) group_names
                WHERE  group_names.business_role_id = br_owner_1.business_role_id
                ORDER BY group_names.name FOR XML PATH('')), 1, 3, '') AS group_owners
          FROM business_role_owner br_owner_1
        GROUP BY br_owner_1.business_role_id) group_owners ON group_owners.business_role_id = b_role.id

LEFT JOIN ( SELECT DISTINCT apmemb.approval_policy_id,
          STUFF((SELECT ' | ' + CAST(pol_user_names.display_name AS nvarchar(MAX)) FROM (SELECT DISTINCT display_name, approval_policy_id from approval_policy_memb apmemb JOIN suser su ON apmemb.unique_member_id = su.unique_user_id AND su.display_name IS NOT NULL AND su.deleted = 0 AND su.effective = 1) pol_user_names
                WHERE  pol_user_names.approval_policy_id = apmemb.approval_policy_id
                ORDER BY pol_user_names.display_name FOR XML PATH('')), 1, 3, '') AS policy_users
          FROM approval_policy_memb apmemb
        GROUP BY apmemb.approval_policy_id) pol_users ON pol_users.approval_policy_id = b_role.approval_policy


LEFT JOIN ( SELECT DISTINCT apmemb.approval_policy_id,
          STUFF((SELECT ' | ' + CAST(pol_groupname.name AS nvarchar(MAX)) FROM (SELECT DISTINCT name, approval_policy_id from approval_policy_memb apmemb JOIN sgroup sgroup_1 ON apmemb.unique_member_id = sgroup_1.unique_group_id AND sgroup_1.name IS NOT NULL AND sgroup_1.deleted = 0 AND sgroup_1.effective = 1) pol_groupname
                WHERE  pol_groupname.approval_policy_id = apmemb.approval_policy_id
                ORDER BY pol_groupname.name FOR XML PATH('')), 1, 3, '') AS policy_groups
          FROM approval_policy_memb apmemb
        GROUP BY apmemb.approval_policy_id) pol_groups ON pol_groups.approval_policy_id = b_role.approval_policy


LEFT JOIN ( SELECT DISTINCT br_incl_mem.business_role_id,
          STUFF((SELECT ' | ' + CAST(incl_users_names.display_name AS nvarchar(MAX)) FROM (SELECT DISTINCT display_name, business_role_id from business_role_incl_memb br_incl_mem JOIN suser su ON br_incl_mem.unique_member_id = su.unique_user_id AND su.display_name IS NOT NULL AND su.deleted = 0 AND su.effective = 1) incl_users_names
                WHERE  incl_users_names.business_role_id = br_incl_mem.business_role_id
                ORDER BY incl_users_names.display_name FOR XML PATH('')), 1, 3, '') AS included_users
          FROM business_role_incl_memb br_incl_mem
         GROUP BY br_incl_mem.business_role_id) incl_users ON incl_users.business_role_id = b_role.id


LEFT JOIN ( SELECT DISTINCT br_incl_mem.business_role_id,
          STUFF((SELECT ' | ' + CAST(incl_groupnames.name AS nvarchar(MAX)) FROM (SELECT DISTINCT sgroup_1.name, business_role_id from business_role_incl_memb br_incl_mem JOIN sgroup sgroup_1 ON br_incl_mem.unique_member_id = sgroup_1.unique_group_id AND sgroup_1.name IS NOT NULL AND sgroup_1.deleted = 0 AND sgroup_1.effective = 1) incl_groupnames
                WHERE  incl_groupnames.business_role_id = br_incl_mem.business_role_id
                ORDER BY incl_groupnames.name FOR XML PATH('')), 1, 3, '') AS included_groups
          FROM business_role_incl_memb br_incl_mem
        GROUP BY br_incl_mem.business_role_id) incl_groups ON incl_groups.business_role_id = b_role.id

LEFT JOIN ( SELECT DISTINCT br_excl_mem.business_role_id,
          STUFF((SELECT ' | ' + CAST(excl_user_names.display_name AS nvarchar(MAX)) FROM (SELECT DISTINCT display_name, business_role_id from business_role_excl_memb br_excl_mem JOIN suser ON br_excl_mem.unique_member_id = suser.unique_user_id AND suser.display_name IS NOT NULL AND suser.deleted = 0 AND suser.effective = 1) excl_user_names
                WHERE  excl_user_names.business_role_id = br_excl_mem.business_role_id
                ORDER BY excl_user_names.display_name FOR XML PATH('')), 1, 3, '') AS excluded_users
          FROM business_role_excl_memb br_excl_mem
        GROUP BY br_excl_mem.business_role_id) excl_users ON excl_users.business_role_id = b_role.id


LEFT JOIN ( SELECT DISTINCT br_excl_mem.business_role_id,
          STUFF((SELECT ' | ' + CAST(excl_group_names.name AS nvarchar(MAX)) FROM (SELECT DISTINCT sgroup_1.name, business_role_id from business_role_excl_memb br_excl_mem JOIN sgroup sgroup_1 ON br_excl_mem.unique_member_id = sgroup_1.unique_group_id AND sgroup_1.name IS NOT NULL AND sgroup_1.deleted = 0 AND sgroup_1.effective = 1) excl_group_names
                WHERE  excl_group_names.business_role_id = br_excl_mem.business_role_id
                ORDER BY excl_group_names.name FOR XML PATH('')), 1, 3, '') AS excluded_groups
          FROM business_role_excl_memb br_excl_mem
        GROUP BY br_excl_mem.business_role_id) excl_groups ON excl_groups.business_role_id = b_role.id


LEFT JOIN ( SELECT DISTINCT br_incl_item.business_role_id,
          STUFF((SELECT ' | ' + CAST(authperm_names.name AS nvarchar(MAX)) FROM (SELECT DISTINCT spermission.name, business_role_id from business_role_incl_item br_incl_item JOIN spermission ON br_incl_item.unique_item_id = spermission.unique_permission_id AND spermission.name IS NOT NULL AND spermission.deleted = 0 AND spermission.effective = 1) authperm_names
                WHERE  authperm_names.business_role_id = br_incl_item.business_role_id
                ORDER BY authperm_names.name FOR XML PATH('')), 1, 3, '') AS auth_permission
          FROM business_role_incl_item br_incl_item
        GROUP BY br_incl_item.business_role_id) auth_perm ON auth_perm.business_role_id = b_role.id

LEFT JOIN ( SELECT DISTINCT br_incl_item.business_role_id,
          STUFF((SELECT ' | ' + CAST(authtechrole_names.name AS nvarchar(MAX)) FROM (SELECT DISTINCT role_policy.name, business_role_id from business_role_incl_item br_incl_item JOIN role_policy ON br_incl_item.unique_item_id = role_policy.unique_policy_id AND role_policy.deleted = 0) authtechrole_names
                WHERE  authtechrole_names.business_role_id = br_incl_item.business_role_id
                ORDER BY authtechrole_names.name FOR XML PATH('')), 1, 3, '') AS auth_tech_role
          FROM business_role_incl_item br_incl_item
        GROUP BY br_incl_item.business_role_id) auth_tech_role ON auth_tech_role.business_role_id = b_role.id

LEFT JOIN ( SELECT DISTINCT br_incl_item.business_role_id,
          STUFF((SELECT ' | ' + CAST(authapp_names.name AS nvarchar(MAX)) FROM (SELECT DISTINCT data_source.name, business_role_id from business_role_incl_item br_incl_item JOIN data_source ON br_incl_item.unique_item_id = CAST(data_source.id AS nvarchar(MAX)) AND data_source.deleted = 0) authapp_names
                WHERE  authapp_names.business_role_id = br_incl_item.business_role_id
                ORDER BY authapp_names.name FOR XML PATH('')), 1, 3, '') AS auth_application
          FROM business_role_incl_item br_incl_item
        GROUP BY br_incl_item.business_role_id) auth_app ON auth_app.business_role_id = b_role.id


  WHERE b_role.approval_state != 'ARCHIVE'

GO

GRANT SELECT ON business_role_v TO ig_report_role
GO