USE [igops]
GO

DROP VIEW IF EXISTS permission_assignments_v
GO

DROP VIEW IF EXISTS assignment_attribute_def_v
GO

CREATE VIEW permission_assignments_v AS
WITH recent_collections AS (
    SELECT 
        MAX(id) OVER  (PARTITION BY data_collector ORDER BY data_collector) as col
    FROM data_collection 
        )

SELECT 
    usr.display_name AS user_display_name,
    usr.unique_user_id,
    perm.name AS permission,
    perm.unique_permission_id,
    perm.description AS permission_description,
    perm.risk AS permission_risk,
    perm.type AS permission_type,
    ds.name AS application,
    ds.unique_id as unique_application_id,
    roles.role_names AS role_name,
        CASE
        WHEN ur.is_authorized = 1 THEN 1
        ELSE 0
    END AS is_authorized,
    req.display_name AS requester_display_name,
    req.unique_user_id AS requester_unique_id,
    app.display_name AS approver_display_name,
    app.unique_user_id AS approver_unique_id,
    rri.reason AS request_item_reason,
    ri.reason AS request_reason,
    rrap.finish_comment AS approver_comment,
    cph.start_time,
    cph.end_time,
    cph.revocable,
    cph.value,
    cph.generic1,
    cph.generic2,
    cph.generic3,
    cph.generic4,
    cph.generic5,
    cph.generic6,
        COALESCE(
        (SELECT
            MIN(final_snap.snap_timestamp)
        FROM user_perm_snapshot_v final_snap
        WHERE final_snap.unique_user_id = usr.unique_user_id AND final_snap.perm_qid = perm.unique_permission_id
            AND final_snap.snap_timestamp >
            (
            SELECT 
                MAX(snap.snap_timestamp)
            FROM user_perm_snapshot_v snap
            WHERE snap.unique_user_id = usr.unique_user_id
                AND
                snap.snap_timestamp not in 
                (SELECT snap1.snap_timestamp FROM user_perm_snapshot_v snap1 WHERE snap1.perm_qid = perm.unique_permission_id AND snap1.unique_user_id = usr.unique_user_id))),
        (SELECT
            MIN(final_snap.snap_timestamp)
        FROM user_perm_snapshot_v final_snap
        WHERE final_snap.unique_user_id = usr.unique_user_id AND final_snap.perm_qid = perm.unique_permission_id)
    ) AS held_since
FROM 
    (SELECT * FROM spermission WHERE (exclude_from_catalog = 0 OR exclude_from_catalog IS NULL) AND effective = 1) perm
    LEFT JOIN data_source ds ON ds.id = perm.APPLICATION
    JOIN (SELECT DISTINCT suser, spermission FROM suser_account_permission) sap ON perm.id = sap.spermission
    JOIN suser usr ON usr.id = sap.suser AND usr.effective  = 1
    LEFT JOIN user_resource ur ON ur.unique_user_id = usr.unique_user_id AND ur.unique_resource_id = perm.unique_permission_id AND ur.deleted = 0
    LEFT JOIN resource_request_item rri ON rri.unique_item_id = perm.unique_permission_id AND rri.unique_recipient_id = usr.unique_user_id AND rri.item_approval_status = 'APPROVED' AND rri.add_item = 1 
            AND rri.last_status_update_time = (SELECT MAX(rri2.last_status_update_time) FROM resource_request_item rri2 WHERE rri2.unique_item_id = rri.unique_item_id AND rri2.unique_recipient_id = rri.unique_recipient_id)
    LEFT JOIN resource_request ri ON ri.id = rri.resource_request_id
    LEFT JOIN resource_request_approval_step rrap ON rrap.resource_request_id = ri.id
    LEFT JOIN suser req ON ri.unique_requester_id = req.unique_user_id AND req.effective = 1 AND req.deleted = 0
    LEFT JOIN suser app ON rri.status_changed_by = app.unique_user_id AND app.effective = 1 AND app.deleted = 0
    LEFT JOIN ( SELECT STUFF((SELECT ' | ' + CAST(name AS nvarchar(MAX)) 
                    FROM role_policy
                    GROUP BY name 
                    FOR XML PATH('')),1,3,'')role_names,
                    su.unique_user_id,
                    rperm.unique_permission_id
                FROM role_user ru
                    JOIN role_permission rperm ON rperm.policy = ru.policy
                    JOIN role_policy rp ON ru.policy = rp.id AND rp.state = 'ACTIVE' AND rp.deleted = 0
                    JOIN suser su ON su.id = ru.suser
                WHERE ru.deleted = 0
                GROUP BY su.unique_user_id, rperm.unique_permission_id) roles ON roles.unique_user_id = usr.unique_user_id AND roles.unique_permission_id = perm.unique_permission_id
    LEFT JOIN cpermission_holder cph ON cph.holder_link = usr.dn AND cph.unique_permission_id = perm.unique_permission_id AND (cph.data_collection IN (SELECT * FROM recent_collections))
GO

CREATE VIEW assignment_attribute_def_v AS
  SELECT DISTINCT 
    attribute_definition.id,
    attribute_definition.allow_static_mapping,
    attribute_definition.attribute_data_type,
    attribute_definition.attribute_key,
    attribute_definition.attribute_state,
    attribute_definition.attribute_type,
    attribute_definition.curatable,
    attribute_definition.entity_type,
    attribute_definition.length,
    attribute_definition.listable,
    attribute_definition.multi_valued,
    attribute_definition.quick_info,
    attribute_definition.quick_searchable,
    attribute_definition.required,
    attribute_definition.searchable,
    attribute_definition.sortable,
    attribute_definition.truncatable,
    attribute_definition.type_ahead_searchable,
    attribute_definition.localized_display_name,
    llm.label_map AS attribute_name,
    attribute_definition.joinable,
    attribute_definition.target_entity,
    attribute_definition.enum_type,
    attribute_definition.composition,
    attribute_definition.excluded_from_curation,
    attribute_definition.excluded_from_search,
    attribute_definition.advanced_searchable,
    attribute_definition.review_criteria,
    attribute_definition.business_role_criteria,
    attribute_definition.excluded_from_quick_search,
    attribute_definition.excluded_from_sorting
   FROM attribute_definition
     LEFT JOIN localized_label_map llm ON llm.localized_label = attribute_definition.localized_display_name
  WHERE attribute_definition.attribute_key LIKE 'generic%' AND attribute_definition.entity_type = 'PERMISSION_TO_HOLDERS' AND llm.label_map_key = 'en'
GO

GRANT SELECT ON permission_assignments_v TO ig_report_role
GO

GRANT SELECT ON assignment_attribute_def_v TO ig_report_role
GO