BEGIN
  FOR k IN (SELECT null FROM all_synonyms WHERE owner = 'PUBLIC' and synonym_name = 'AUTO_CHANGE_STATUS_V') LOOP
    EXECUTE IMMEDIATE 'DROP PUBLIC SYNONYM AUTO_CHANGE_STATUS_V';
  END LOOP;
   FOR i IN (SELECT null FROM all_views WHERE owner = user and view_name = 'AUTO_CHANGE_STATUS_V') LOOP
    EXECUTE IMMEDIATE 'DROP VIEW AUTO_CHANGE_STATUS_V';
  END LOOP;
END;
/

CREATE VIEW auto_change_status_v AS
SELECT DISTINCT
    cri.change_request_date,
    acs.unique_business_role_id,
    br.name AS business_role,
    cri.item_type,
    cri.change_request_type AS task,
    su.unique_user_id,
    su.display_name AS username,
    perm.unique_permission_id,
    perm.name AS permission,
    ds.unique_id AS application_id,
    ds.name AS application,
    cia.change_item_status AS request_status,
    CASE
        WHEN cia.change_item_status IN ('PENDING','RETRY','ERROR') THEN NULL
        ELSE cia.last_updated
    END AS fulfillmenttime,
    cri.fulfiller_type,
    COALESCE(fufusr.display_name, fic.fulfiller_name) AS fulfillerusername,
    CASE
        WHEN cri.fulfiller_type = 'GROUP' THEN sgrp.name
        WHEN cri.fulfiller_type = 'USER' THEN susr.display_name
        ELSE NULL
    END AS assigned_fulfiller
   FROM change_request_item cri
     JOIN auto_change_request acr ON cri.auto_change_request_id = acr.id
     JOIN auto_change_set acs ON acr.auto_change_set_id = acs.id
     JOIN business_role br ON acs.unique_business_role_id = br.unique_id
     JOIN suser su ON cri.user_uid = su.unique_user_id AND su.effective = 1
     LEFT JOIN spermission perm ON cri.permission = perm.unique_permission_id AND perm.effective = 1
     LEFT JOIN saccount acct ON cri.account = acct.unique_account_id AND acct.effective = 1
     LEFT JOIN data_source ds ON ds.id = cri.application
     LEFT JOIN change_item_action cia ON cia.id = cri.latest_action
     LEFT JOIN suser fufusr ON cia.fulfiller_id = fufusr.unique_user_id AND fufusr.effective = 1
     LEFT JOIN access_request_item ari ON cia.id = ari.id
     LEFT JOIN resource_request_item rri ON ari.unique_item_id = rri.unique_item_id
     LEFT JOIN fulfillment_item_combined fic ON fic.id = rri.change_request_item_id
     LEFT JOIN resource_request rr ON rr.id = rri.resource_request_id
     LEFT JOIN sgroup sgrp ON sgrp.unique_group_id = cri.fulfiller AND sgrp.effective = 1
     LEFT JOIN suser susr ON susr.unique_user_id = cri.fulfiller AND susr.effective = 1;

GRANT SELECT ON igops.auto_change_status_v TO ig_report_role;
CREATE PUBLIC SYNONYM auto_change_status_v FOR igops.auto_change_status_v;