BEGIN
  FOR k IN (SELECT null FROM all_synonyms WHERE owner = 'PUBLIC' and synonym_name = 'ACCESS_REVIEW_SUMMARY_V') LOOP
    EXECUTE IMMEDIATE 'DROP PUBLIC SYNONYM ACCESS_REVIEW_SUMMARY_V';
  END LOOP;
   FOR i IN (SELECT null FROM all_views WHERE owner = user and view_name = 'ACCESS_REVIEW_SUMMARY_V') LOOP
    EXECUTE IMMEDIATE 'DROP VIEW ACCESS_REVIEW_SUMMARY_V';
  END LOOP;
END;
/

CREATE VIEW access_review_summary_v AS 
 SELECT 
    instance.id instance_id,
    defn.name,
    defn.description,
    defn.review_type,
    defn.next_schedule_date,
    defn.unique_review_def_id,
    multi_owner.review_owners review_inst_owner_display_name,
    instance.status,
    instance.status_comment,
    instance.start_date,
    instance.expected_end_date,
    instance.end_date,
    instance.valid_to_date,
    instance.canceled_by,
    instance.task_count,
    instance.task_complete_count,
    CASE
      WHEN instance.task_complete_count = 0 THEN 0
      ELSE round((instance.task_complete_count / instance.task_count) * 100, 1)
    END AS percent_complete,
    canceler.display_name canceled_by_name,
    instance.item_count,
    instance.item_complete_count,
    instances.reviewer_count,
    (SELECT decisions.decision_count
      FROM (SELECT inst.id  instance,
          it.final_decision  decision,
          COUNT(it.final_decision) decision_count
        FROM review_instance inst
          LEFT JOIN review_item it ON it.review_instance_id = inst.id
        GROUP BY inst.id, it.final_decision) decisions
      WHERE decisions.instance = instance.id AND decisions.decision = 'ASSIGN_USER')  assign_count,
    (SELECT decisions.decision_count
      FROM ( SELECT inst.id AS instance,
          it.final_decision decision,
          COUNT(it.final_decision) AS decision_count
        FROM review_instance inst
          LEFT JOIN review_item it ON it.review_instance_id = inst.id
        GROUP BY inst.id, it.final_decision) decisions
      WHERE decisions.instance = instance.id AND decisions.decision = 'KEEP') AS keep_count,
    (SELECT decisions.decision_count
      FROM ( SELECT inst.id AS instance,
          it.final_decision decision,
          COUNT(it.final_decision) decision_count
        FROM review_instance inst
          LEFT JOIN review_item it ON it.review_instance_id = inst.id
        GROUP BY inst.id, it.final_decision) decisions
      WHERE decisions.instance = instance.id AND decisions.decision = 'REMOVE')  remove_count,
    (SELECT decisions.decision_count
      FROM ( SELECT inst.id AS instance,
          it.final_decision decision,
          COUNT(it.final_decision) decision_count
        FROM review_instance inst
          LEFT JOIN review_item it ON it.review_instance_id = inst.id
        GROUP BY inst.id, it.final_decision) decisions
      WHERE decisions.instance = instance.id AND decisions.decision IN ('MODIFY_ENTITY', 'MODIFY_ACCESS', 'MODIFY_SUPERVISOR'))  modify_count
  FROM review_instance instance
    JOIN (SELECT
        review_instance.id inst_id,
        defn.id defn_id,
        CASE WHEN staged_defn.name IS NULL THEN defn.name ELSE staged_defn.name END AS name,
        CASE WHEN staged_defn.description IS NULL THEN defn.description ELSE staged_defn.description END description,
        defn.review_type,
        defn.next_schedule_date,
        defn.deleted,
        defn.unique_review_def_id
      FROM review_definition defn
        JOIN review_instance ON defn.id = review_instance.review_definition
        LEFT JOIN review_definition staged_defn ON staged_defn.id = review_instance.staged_review_definition_id) defn ON defn.inst_id = instance.id
    JOIN (SELECT inst.id AS instance,
          COUNT(DISTINCT rui.unique_user_id)  reviewer_count
         FROM review_instance inst
           LEFT JOIN reviewer_instance ri ON ri.review_instance_id = inst.id
           LEFT JOIN reviewer_user_instance rui ON rui.reviewer_instance_id = ri.id
           WHERE ri.task_count > 0
        GROUP BY inst.id) instances ON instances.instance = instance.id
    LEFT JOIN suser canceler ON instance.canceled_by = canceler.unique_user_id AND canceler.effective = 1
    LEFT JOIN ( SELECT DISTINCT 
        defn_1.name review_name,
        ri.id AS rid,
        LISTAGG((TO_CHAR(s.display_name)), ' | ') WITHIN GROUP (ORDER BY s.display_name) AS review_owners
        FROM review_instance ri
          JOIN review_definition defn_1 ON ri.review_definition = defn_1.id
          LEFT JOIN reviewer_instance rev_owner_1 ON ri.id = rev_owner_1.review_instance_id AND rev_owner_1.type = 'RO'
          LEFT JOIN reviewer_user_instance rui ON rui.reviewer_instance_id = rev_owner_1.id
          LEFT JOIN suser s ON rui.unique_user_id = s.unique_user_id AND s.effective=1
        GROUP BY defn_1.name, ri.id) multi_owner ON multi_owner.rid = instance.id
  WHERE defn.deleted = 0 AND instance.status NOT IN ('PREVIEW', 'PREVIEW_CANCELED', 'PREVIEW_STARTING');

GRANT SELECT ON igops.access_review_summary_v TO ig_report_role;
CREATE PUBLIC SYNONYM access_review_summary_v FOR igops.access_review_summary_v;