DROP VIEW IF EXISTS approval_request_policy_v;

DROP VIEW IF EXISTS request_item_psod_v;

DROP VIEW IF EXISTS request_lifecycle_v;

DROP VIEW IF EXISTS resource_request_v;

CREATE VIEW approval_request_policy_v AS 
 SELECT aas.policy_id,
    arap.unique_policy_id,
    arap.name,
    aas.step_sequence,
    aas.approver_type,
    aas.escalation_period,
    aas.escalator_type,
    aas.timeout_period,
    aas.timeout_action,
    aas.id
 FROM access_approval_step aas
 	JOIN access_req_approval_policy arap ON aas.policy_id = arap.id;

CREATE VIEW request_item_psod_v AS 
 SELECT rri.id,
    rr.request_status,
    rri.item_type AS req_item_type,
    rri.unique_item_id AS req_item_unique_id,
    perm.name AS permission_name,
    role.name AS role_name,
    ds.name AS app_name,
    requser.display_name AS request_user_name,
    requser.unique_user_id AS request_user_id,
    rri.last_status_update_time AS appr_deny_time,
    recipuser.display_name AS recipient_user_name,
    recipuser.unique_user_id AS recipient_user_id,
    sod.name AS psod_policy_name,
    sod.unique_policy_id AS psod_policy_id,
    sod.id AS psod_id,
    sod.deleted AS policy_status,
    sod.creation_time,
    sod.description AS policy_description,
    sod.detection_status,
    sod.state,
    sod.update_time,
    change.display_name AS changed_by,
    creater.display_name AS created_by,
    pati.decision AS psod_appr_status,
    pati.decision_date AS psod_appr_status_time,
    pati.decided_by AS psod_appr_id,
    psodappuser.display_name AS psod_appr_name,
    policy_owners.owners
   FROM resource_request_item rri
     JOIN resource_request rr ON rr.id = rri.resource_request_id
     LEFT JOIN resource_request_item_step rris ON rris.resource_request_item_id = rri.id
     LEFT JOIN suser requser ON requser.unique_user_id::text = rr.unique_requester_id::text AND requser.effective = true
     LEFT JOIN suser recipuser ON recipuser.unique_user_id::text = rri.unique_recipient_id::text AND recipuser.effective = true
     LEFT JOIN spermission perm ON perm.unique_permission_id::text = rri.unique_item_id::text AND perm.effective = true AND rri.item_type::text = 'PERMISSION'::text
     LEFT JOIN role_policy role ON role.unique_policy_id::text = rri.unique_item_id::text AND role.deleted = false AND rri.item_type::text = 'TECHNICAL_ROLE'::text
     LEFT JOIN application app ON app.id::character varying(255)::text = rri.unique_item_id::text AND rri.item_type::text = 'APPLICATION'::text
     LEFT JOIN data_source ds ON ds.id = app.id AND ds.deleted = false
     LEFT JOIN change_request_item cri ON cri.id = rri.change_request_item_id
     LEFT JOIN change_item_action cia ON cia.change_request_item = cri.id
     LEFT JOIN suser fufusr ON cri.fulfiller::text = fufusr.unique_user_id::text AND fufusr.effective = true
     LEFT JOIN resource_request_approval_step rras ON rras.id = rris.approval_step_id
     LEFT JOIN suser stepuser ON stepuser.unique_user_id::text = rras.finished_by::text AND stepuser.effective = true
     LEFT JOIN suser stepuser2 ON stepuser2.unique_user_id::text = rris.status_set_by::text AND stepuser2.effective = true
     LEFT JOIN suser apprvuser ON apprvuser.unique_user_id::text = rri.status_changed_by::text AND apprvuser.effective = true AND rras.id IS NOT NULL
     LEFT JOIN psodv_approval_task_item pati ON pati.resource_request_item_id = rri.id
     LEFT JOIN psodv_approval_task pat ON pat.id = pati.psodv_approval_task_id
     LEFT JOIN suser psodsubuser ON psodsubuser.unique_user_id::text = pat.completed_by::text AND psodsubuser.effective = true
     LEFT JOIN suser psodappuser ON psodappuser.unique_user_id::text = pati.decided_by::text AND psodappuser.effective = true
     LEFT JOIN sod_policy sod ON sod.id = pat.policy_id
     LEFT JOIN suser change ON change.unique_user_id::text = sod.changed_by::text AND change.deleted = false AND change.effective = true
     LEFT JOIN suser creater ON creater.unique_user_id::text = sod.created_by::text AND creater.deleted = false AND creater.effective = true
     LEFT JOIN ( SELECT policy_owners_v.sod_policy,
            string_agg(policy_owners_v.owner_name::text, ' | '::text) AS owners
           FROM policy_owners_v
          GROUP BY policy_owners_v.sod_policy) policy_owners ON policy_owners.sod_policy = sod.id;

CREATE VIEW request_lifecycle_v AS 
 SELECT perm.name AS permissionname,
    perm.unique_permission_id,
    stepuser.display_name AS stepapprovername,
    stepuser.unique_user_id AS stepapproveruserid,
    rras.finish_time AS stepapprovaltime,
    stepuser2.display_name AS statussettername,
    stepuser2.unique_user_id AS statussetteruserid,
    rris.status_time AS statussettime,
    rr.id AS cartid,
    rri.id,
    rras.step AS stepapprovalorder,
    rr.request_status,
    rris.item_approval_status AS stepapproveordenystatus,
    role.name AS rolename,
    ds.name AS appname,
    requser.display_name AS requestusername,
    requser.unique_user_id AS requestuserid,
    rr.creation_time AS requesttime,
    apprvuser.display_name AS approveordenyusername,
    apprvuser.unique_user_id AS approveordenyuserid,
        CASE
            WHEN rras.id IS NULL THEN NULL::character varying
            ELSE rri.item_approval_status
        END AS approveordenystatus,
    rri.item_approval_status,
    rri.last_status_update_time AS approveordenytime,
    recipuser.display_name AS recipientusername,
    recipuser.unique_user_id AS recipientuserid,
    rri.add_item AS add_request,
    rri.reason AS item_reason,
    rr.reason AS request_reason,
    rras.finish_comment AS approval_comment,
    cia.change_item_status AS fulfillment_status,
    cia.item_comment AS fulfillment_comment,
    cia.last_updated AS fulfillment_date,
    fufusr.display_name AS fulfillerusername,
    fufusr.unique_user_id AS fulfilleruserid,
        CASE
            WHEN cri.fulfiller_type::text = 'GROUP'::text THEN sgrp.name
            WHEN cri.fulfiller_type::text = 'USER'::text THEN susr.display_name
            ELSE NULL::character varying
        END AS assigned_fulfiller
   FROM resource_request_item rri
     JOIN resource_request rr ON rr.id = rri.resource_request_id
     LEFT JOIN resource_request_item_step rris ON rris.resource_request_item_id = rri.id
     LEFT JOIN suser requser ON requser.unique_user_id::text = rr.unique_requester_id::text AND requser.effective = true
     LEFT JOIN suser recipuser ON recipuser.unique_user_id::text = rri.unique_recipient_id::text AND recipuser.effective = true
     LEFT JOIN spermission perm ON perm.unique_permission_id::text = rri.unique_item_id::text AND perm.effective = true AND rri.item_type::text = 'PERMISSION'::text
     LEFT JOIN role_policy role ON role.unique_policy_id::text = rri.unique_item_id::text AND role.deleted = false AND rri.item_type::text = 'TECHNICAL_ROLE'::text
     LEFT JOIN application app ON app.id::character varying(255)::text = rri.unique_item_id::text AND rri.item_type::text = 'APPLICATION'::text
     LEFT JOIN data_source ds ON ds.id = app.id AND ds.deleted = false
     LEFT JOIN fulfillment_item_combined fic ON fic.id = rri.change_request_item_id
     LEFT JOIN change_request_item cri ON cri.id = fic.id
     LEFT JOIN change_item_action cia ON cia.id = cri.latest_action
     LEFT JOIN suser fufusr ON cia.fulfiller_id::text = fufusr.unique_user_id::text AND fufusr.effective = true
     LEFT JOIN resource_request_approval_step rras ON rras.id = rris.approval_step_id
     LEFT JOIN suser stepuser ON stepuser.unique_user_id::text = rras.finished_by::text AND stepuser.effective = true
     LEFT JOIN suser stepuser2 ON stepuser2.unique_user_id::text = rris.status_set_by::text AND stepuser2.effective = true
     LEFT JOIN suser apprvuser ON apprvuser.unique_user_id::text = rri.status_changed_by::text AND apprvuser.effective = true AND rras.id IS NOT NULL
     LEFT JOIN sgroup sgrp ON sgrp.unique_group_id::text = cri.fulfiller::text AND sgrp.effective = true
     LEFT JOIN suser susr ON susr.unique_user_id::text = cri.fulfiller::text AND susr.effective = true;

CREATE VIEW resource_request_v AS 
 SELECT rreq.unique_requester_id,
    rreq.current_approval_step,
    rreq.change_set_id,
    rreq.reason,
    rreq.request_status,
    rreq.approval_policy_id,
    rreq.creation_time,
    rreq.approval_start_time,
    rreq.change_set_start_time,
    rreq.last_update_time,
    rreq.id,
    rreq.approval_strategy_json,
    rreq.request_node_json,
    rreq.psodv_approval_start_time
   FROM resource_request rreq
     JOIN suser su ON su.unique_user_id::text = rreq.unique_requester_id::text AND su.effective = true;

ALTER TABLE approval_request_policy_v
  OWNER TO igops;
GRANT ALL ON TABLE approval_request_policy_v TO igops;
GRANT SELECT ON TABLE approval_request_policy_v TO ig_report_role;

ALTER TABLE request_item_psod_v
  OWNER TO igops;
GRANT ALL ON TABLE request_item_psod_v TO igops;
GRANT SELECT ON TABLE request_item_psod_v TO ig_report_role;

ALTER TABLE request_lifecycle_v
  OWNER TO igops;
GRANT ALL ON TABLE request_lifecycle_v TO igops;
GRANT SELECT ON TABLE request_lifecycle_v TO ig_report_role;

ALTER TABLE resource_request_v
  OWNER TO igops;
GRANT ALL ON TABLE resource_request_v TO igops;
GRANT SELECT ON TABLE resource_request_v TO ig_report_role;