DROP VIEW IF EXISTS reassignment_by_delegation_v;

CREATE VIEW reassignment_by_delegation_v AS
SELECT
  ri.review_instance_id,
  defn.defn_id AS definition_id,
  defn.name AS review_name,
  defn.review_type,
  defn.description,
  inst.start_date,
  inst.end_date,
  inst.valid_to_date,
  ri.approved_time,
  defn.next_schedule_date,
  inst.status,
  multi_owner.review_owners,
  ri.id AS item_id,
  inst.task_count AS total_items,
  ri.final_decision,
  ri.final_decision_time,
  action.last_update,
  ri.item_type,
  CASE
    WHEN ri.item_type = 'ACCOUNT' AND usr.id IS NULL THEN shacc.unique_user_id
    ELSE usr.unique_user_id
  END AS unique_user_id,
  CASE
    WHEN ri.item_type = 'ACCOUNT' AND usr.display_name IS NULL THEN shacc.display_name
    ELSE usr.display_name
  END AS user_display_name,
  ri.unique_supervisor_id,
  sup.display_name AS supervisor_display_name,
  ri.business_role_id,
  brole.name AS brole_name,
  ri.unique_account_id,
  acct.name AS account_name,
  ri.unique_permission_id,
  perm.name AS permission_name,
  app.id AS application_id,
  app_ds.name AS application_name,
  ri.role_id,
  role.name AS role_name,
  rui.unique_user_id AS reviewer_id,
  revr.display_name AS reviewer_display_name,
  action.reviewer_comment,
  rui.delegated_unique_user_id,
  del.display_name AS delegated_display_name
FROM review_item ri
  JOIN review_instance inst ON inst.id = ri.review_instance_id
  JOIN ( SELECT review_instance.id AS inst_id,
          defn_1.id AS defn_id,
          CASE
            WHEN staged_defn.name IS NULL THEN defn_1.name
            ELSE staged_defn.name
          END AS name,
          CASE
            WHEN staged_defn.description IS NULL THEN defn_1.description
            ELSE staged_defn.description
          END AS description,
          CASE
            WHEN staged_defn.next_schedule_date IS NULL THEN defn_1.next_schedule_date
            ELSE staged_defn.next_schedule_date
          END AS next_schedule_date,
          defn_1.review_type,
          defn_1.deleted
            FROM review_definition defn_1
                JOIN review_instance ON defn_1.id = review_instance.review_definition
                LEFT JOIN review_definition staged_defn ON staged_defn.id = review_instance.staged_review_definition_id) defn ON inst.id = defn.inst_id
  JOIN reviewer_user_instance rui ON rui.reviewer_instance_id = ri.curr_reviewer_instance_id AND rui.delegated_unique_user_id IS NOT NULL
  LEFT JOIN spermission perm ON perm.unique_permission_id = ri.unique_permission_id AND perm.effective = true
  LEFT JOIN saccount acct ON acct.unique_account_id = ri.unique_account_id AND acct.effective = true
  LEFT JOIN application app ON app.id =
            CASE
              WHEN perm.id IS NULL THEN acct.application
              ELSE perm.application
            END
  LEFT JOIN ( SELECT sa.display_name,
                    sa.unique_account_id,
                    sau.unique_user_id
                FROM shared_accounts_v sa
                    JOIN suser sau ON sa.suser = sau.id) shacc ON shacc.unique_account_id = acct.unique_account_id AND ri.item_type = 'ACCOUNT' AND defn.review_type = 'ACCOUNT'
  LEFT JOIN data_source app_ds ON app_ds.id = app.id
  LEFT JOIN suser sup ON sup.unique_user_id = ri.unique_supervisor_id AND sup.effective = true
  LEFT JOIN business_role brole ON brole.id = ri.business_role_id
  LEFT JOIN role_policy role ON role.id = ri.role_id
  LEFT JOIN suser usr ON ri.unique_user_id = usr.unique_user_id AND usr.effective = true
  LEFT JOIN suser revr ON rui.unique_user_id = revr.unique_user_id AND revr.effective = true
  LEFT JOIN suser del ON rui.delegated_unique_user_id = del.unique_user_id AND del.effective = true
  LEFT JOIN review_item_action action ON action.review_item_id = ri.id AND action.action != 'REASSIGNED' 
  LEFT JOIN ( SELECT DISTINCT defn_1.name AS review_name,
                    ri_1.id AS rid,
                    string_agg(s.display_name, ' | ' ORDER BY (s.display_name)) AS review_owners
                FROM review_instance ri_1
                    JOIN review_definition defn_1 ON ri_1.review_definition = defn_1.id
                    LEFT JOIN reviewer_instance rev_owner ON ri_1.id = rev_owner.review_instance_id AND rev_owner.type = 'RO'
                    LEFT JOIN reviewer_user_instance rui_1 ON rui_1.reviewer_instance_id = rev_owner.id
                    LEFT JOIN suser s ON rui_1.unique_user_id = s.unique_user_id AND s.effective = true
                GROUP BY defn_1.name, ri_1.id) multi_owner ON multi_owner.rid = ri.review_instance_id;

ALTER TABLE reassignment_by_delegation_v
  OWNER TO igops;
GRANT ALL ON TABLE reassignment_by_delegation_v TO igops;
GRANT SELECT ON TABLE reassignment_by_delegation_v TO ig_report_role;

