DROP VIEW IF EXISTS access_review_details_v;

DROP VIEW IF EXISTS access_review_summary_v;

DROP VIEW IF EXISTS reassignments_v;

DROP VIEW IF EXISTS reviewer_item_v;

DROP VIEW IF EXISTS review_item_action_combined_v;

DROP VIEW IF EXISTS reviewer_user_instance_v;

CREATE VIEW access_review_details_v AS 
  SELECT DISTINCT item.id AS review_item_id,
    item.last_override_action_id AS last_override_action,
    instance.id AS review_instance_id,
    instance.status AS review_status,
    instance.start_date,
    instance.end_date,
    instance.valid_to_date,
    action.last_update,
    item.approved_time,
    instance.item_count,
    multi_owner.review_owners,
    multi_reviewer.reviewer_ids AS reviewer_id,
        CASE
            WHEN item.final_cert_by_unique_user_id IS NOT NULL THEN final_reviewer.display_name::text
            ELSE multi_reviewer.reviewers
        END AS reviewer_display_name,
        CASE
            WHEN item.final_cert_by_unique_user_id IS NOT NULL THEN "substring"(split_part(final_reviewer.dn::text, ','::text, 1), 4)
            ELSE multi_reviewer.user_cn
        END AS reviewer_cn,
    reassigned_by_history.reviewers AS reassigned_by_user_disp_name,
    reassigned_from_history.reviewers AS reassigned_from_user_disp_name,
    reassigned_by_history.reassigned_date,
    reassigned_by_history.reasons AS reassigned_reasons,
    reassigned_by_history.rev_comments AS reassigned_comments,
    defn.name AS review_name,
    substr(defn.description, 0, 4000) AS description,
    defn.review_type,
    defn.next_schedule_date,
        CASE
            WHEN item.item_type::text = 'ACCOUNT'::text AND usr.id IS NULL THEN shacc.suser
            ELSE usr.id
        END AS user_id,
        CASE
            WHEN item.item_type::text = 'ACCOUNT'::text AND usr.display_name IS NULL THEN shacc.display_name
            ELSE usr.display_name
        END AS user_display_name,
        CASE
            WHEN item.item_type::text = 'ACCOUNT'::text AND usr.display_name IS NULL THEN shacc.user_cn
            ELSE "substring"(split_part(usr.dn::text, ','::text, 1), 4)
        END AS user_cn,
    action.action AS reviewer_action,
    reason.name AS modify_reason_name,
    reason.id AS modify_reason_id,
        CASE
            WHEN fulfillment.review_item_id IS NOT NULL THEN true
            ELSE false
        END AS remove_completed,
    usr.unique_user_id,
    action.review_order,
    action.reviewer_comment,
    perm.id AS permission_id,
    perm.name AS permission_name,
    acct.id AS account_id,
    acct.account_id AS account_native_id,
    acct.name AS account_name,
    acct.relation_to_user_type AS account_relation,
    app.id AS application_id,
    app_ds.name AS application_name,
    item.final_decision,
    item.final_decision_time,
    role.id AS role_id,
    role.name AS role_name,
    item.item_type,
    brole.name AS brole_name,
    brole.id AS business_role_id,
    action.fulfillment_instructions,
    esc_reviewer.display_name AS esc_reviewer,
    sup.unique_user_id AS supervisor_id,
    sup.display_name AS supervisor_display_name
   FROM review_item item
     JOIN review_instance instance ON item.review_instance_id = instance.id AND (instance.status::text <> ALL (ARRAY['PREVIEW'::character varying, 'PREVIEW_CANCELED'::character varying, 'PREVIEW_STARTING'::character varying]::text[]))
     JOIN ( SELECT review_instance.id AS inst_id,
            defn_1.id AS defn_id,
                CASE
                    WHEN staged_defn.name IS NULL THEN defn_1.name
                    ELSE staged_defn.name
                END AS name,
                CASE
                    WHEN staged_defn.description IS NULL THEN defn_1.description
                    ELSE staged_defn.description
                END AS description,
                CASE
                    WHEN staged_defn.next_schedule_date IS NULL THEN defn_1.next_schedule_date
                    ELSE staged_defn.next_schedule_date
                END AS next_schedule_date,
            defn_1.review_type,
            defn_1.deleted
           FROM review_definition defn_1
             JOIN review_instance ON defn_1.id = review_instance.review_definition
             LEFT JOIN review_definition staged_defn ON staged_defn.id = review_instance.staged_review_definition_id) defn ON instance.id = defn.inst_id
     LEFT JOIN suser usr ON usr.unique_user_id::text = item.unique_user_id::text AND usr.effective = true
     LEFT JOIN suser sup ON sup.unique_user_id = item.unique_supervisor_id AND sup.effective = true
     LEFT JOIN spermission perm ON perm.unique_permission_id::text = item.unique_permission_id::text AND perm.effective = true
     LEFT JOIN saccount acct ON acct.unique_account_id::text = item.unique_account_id::text AND acct.effective = true
     LEFT JOIN application app ON app.id =
        CASE
            WHEN perm.id IS NULL THEN acct.application
            ELSE perm.application
        END
     LEFT JOIN business_role brole ON brole.id = item.business_role_id
     LEFT JOIN role_policy role ON role.id = item.role_id
     LEFT JOIN data_source app_ds ON app_ds.id = app.id
     LEFT JOIN review_item_action action ON action.review_item_id = item.id AND action.action::text <> 'REASSIGNED'::text
     LEFT JOIN review_item_action_reason reason ON reason.id = action.reason_id
     LEFT JOIN reviewer_instance inst2 ON instance.id = inst2.review_instance_id AND inst2.type::text = 'RO'::text
     LEFT JOIN reviewer_instance inst3 ON instance.id = inst3.review_instance_id
     LEFT JOIN ( SELECT DISTINCT item_1.id AS item_id,
            string_agg(multi_reviewers.id::text, ' | '::text ORDER BY multi_reviewers.display_name) AS reviewer_ids,
            string_agg(multi_reviewers.unique_user_id::text, ' | '::text ORDER BY multi_reviewers.display_name) AS reviewer_unique_ids,
            string_agg(multi_reviewers.display_name::text, ' | '::text ORDER BY (multi_reviewers.display_name::text)) AS reviewers,
            string_agg("substring"(split_part(multi_reviewers.dn::text, ','::text, 1), 4), ' | '::text ORDER BY multi_reviewers.display_name) AS user_cn
           FROM review_item item_1
             LEFT JOIN ( SELECT DISTINCT reviewer_user_instance.reviewer_instance_id,
                    reviewer_user_instance.unique_user_id
                   FROM reviewer_user_instance) rrui ON rrui.reviewer_instance_id = item_1.curr_reviewer_instance_id
             LEFT JOIN suser multi_reviewers ON rrui.unique_user_id::text = multi_reviewers.unique_user_id::text AND multi_reviewers.effective = true
          GROUP BY item_1.id) multi_reviewer ON multi_reviewer.item_id = item.id
     LEFT JOIN suser final_reviewer ON item.final_cert_by_unique_user_id::text = final_reviewer.unique_user_id::text AND final_reviewer.effective = true
     LEFT JOIN suser acting_reviewer ON action.acted_by_user_unique_id::text = acting_reviewer.unique_user_id::text AND acting_reviewer.effective = true
     LEFT JOIN ( SELECT ria.review_item_id,
            string_agg(usr_1.display_name::text, ' | '::text ORDER BY ria.assigned) AS reviewers
           FROM review_item_action ria
             LEFT JOIN reviewer_instance ri ON ri.id = ria.reviewer_instance_id
             LEFT JOIN suser usr_1 ON ri.single_reviewer_unique_id::text = usr_1.unique_user_id::text AND usr_1.effective = true
          WHERE ria.action::text = 'REASSIGNED'::text
          GROUP BY ria.review_item_id) reassigned_from_history ON reassigned_from_history.review_item_id = item.id
     LEFT JOIN ( SELECT ria.review_item_id,
            string_agg(usr_1.display_name::text, ' | '::text ORDER BY ria.assigned) AS reviewers,
            string_agg(to_char(ria.assigned, 'YYYY-MM-DD HH24:MI:SS'::text), ' | '::text ORDER BY ria.assigned) AS reassigned_date,
            string_agg(COALESCE(reason_1.name, 'NULL_REASON'::character varying)::text, ' | '::text ORDER BY ria.assigned) AS reasons,
            string_agg(COALESCE(NULLIF(ria.reviewer_comment::text, ''::text), 'NULL_COMMENT'::text), ' | '::text ORDER BY ria.assigned) AS rev_comments
           FROM review_item_action ria
             LEFT JOIN review_item_action_reason reason_1 ON ria.reason_id = reason_1.id
             LEFT JOIN suser usr_1 ON ria.reassigned_by_user_unique_id::text = usr_1.unique_user_id::text AND usr_1.effective = true
          WHERE ria.action::text = 'REASSIGNED'::text
          GROUP BY ria.review_item_id) reassigned_by_history ON reassigned_by_history.review_item_id = item.id
     LEFT JOIN shared_accounts_v shacc ON shacc.unique_account_id::text = acct.unique_account_id::text
     LEFT JOIN ( SELECT DISTINCT defn_1.name AS review_name,
            ri.id AS rid,
            string_agg(s.display_name::text, ' | '::text ORDER BY (s.display_name::text)) AS review_owners
           FROM review_instance ri
             JOIN review_definition defn_1 ON ri.review_definition = defn_1.id
             LEFT JOIN reviewer_instance rev_owner ON ri.id = rev_owner.review_instance_id AND rev_owner.type::text = 'RO'::text
             LEFT JOIN reviewer_user_instance rui ON rui.reviewer_instance_id = rev_owner.id
             LEFT JOIN suser s ON rui.unique_user_id::text = s.unique_user_id::text AND s.effective = true
          GROUP BY defn_1.name, ri.id) multi_owner ON multi_owner.rid = item.review_instance_id
     LEFT JOIN ( SELECT esc_reviewers.review_instance_id,
            string_agg(esc_reviewers.display_name::text, ' | '::text ORDER BY (esc_reviewers.display_name::text)) AS display_name
           FROM ( SELECT DISTINCT u.display_name,
                    rri.review_instance_id
                   FROM review_item ri
                     JOIN reviewer_instance rri ON ri.review_instance_id = rri.review_instance_id AND rri.type::text = 'EO'::text
                     JOIN reviewer_user_instance rriu ON rriu.reviewer_instance_id = rri.id
                     JOIN suser u ON u.unique_user_id::text = rriu.unique_user_id::text AND u.effective = true) esc_reviewers
          GROUP BY esc_reviewers.review_instance_id) esc_reviewer ON esc_reviewer.review_instance_id = item.review_instance_id
     LEFT JOIN ( SELECT cri.review_item_id,
            cia.action,
            cia.change_item_status
           FROM change_request_item cri
             LEFT JOIN change_item_action cia ON cri.latest_action = cia.id
          WHERE cri.change_request_type::text ~~ ('REMOVE'::text || '%'::text) AND cia.action::text = 'FULFILLED'::text AND (cia.change_item_status::text = 'VERIFIED'::text OR cia.change_item_status::text = 'COMPLETED'::text)) fulfillment ON item.id = fulfillment.review_item_id
  WHERE defn.deleted = false AND (acting_reviewer.unique_user_id::text ~~ (('%'::text || multi_reviewer.reviewer_unique_ids) || '%'::text) OR acting_reviewer.unique_user_id IS NULL AND (action.review_order = (( SELECT DISTINCT min(act.review_order) AS min
           FROM review_item_action act
          WHERE (act.action::text = 'PENDING'::text OR act.action::text = 'ESCALATE'::text) AND act.review_item_id = item.id)) OR action.acted_by_policy = true) OR acting_reviewer.unique_user_id IS NOT NULL AND action.shared_queue = true) AND (action.action::text <> 'PENDING'::text OR item.final_decision IS NULL);

CREATE VIEW access_review_summary_v AS 
   SELECT 
    instance.id AS instance_id,
    defn.name,
    defn.description,
    defn.review_type,
    defn.next_schedule_date,
    defn.unique_review_def_id,
    multi_owner.review_owners AS review_inst_owner_display_name,
    instance.status,
    instance.status_comment,
    instance.start_date,
    instance.expected_end_date,
    instance.end_date,
    instance.valid_to_date,
    instance.canceled_by,
    instance.task_count,
    instance.task_complete_count,
    CASE
      WHEN instance.task_complete_count = 0 THEN 0::numeric
      ELSE round(instance.task_complete_count::numeric / instance.task_count::numeric * 100::numeric, 1)
    END AS percent_complete,
    canceler.display_name AS canceled_by_name,
    instance.item_count,
    instance.item_complete_count,
    instances.reviewer_count,
    ( SELECT decisions.decision_count
           FROM ( SELECT inst.id AS instance,
                    it.final_decision AS decision,
                    count(it.final_decision) AS decision_count
                   FROM review_instance inst
                     LEFT JOIN review_item it ON it.review_instance_id = inst.id
                  GROUP BY inst.id, it.final_decision) decisions
          WHERE decisions.instance = instance.id AND decisions.decision = 'ASSIGN_USER') AS assign_count,
    ( SELECT decisions.decision_count
           FROM ( SELECT inst.id AS instance,
                    it.final_decision AS decision,
                    count(it.final_decision) AS decision_count
                   FROM review_instance inst
                     LEFT JOIN review_item it ON it.review_instance_id = inst.id
                  GROUP BY inst.id, it.final_decision) decisions
          WHERE decisions.instance = instance.id AND decisions.decision = 'KEEP') AS keep_count,
    ( SELECT decisions.decision_count
           FROM ( SELECT inst.id AS instance,
                    it.final_decision AS decision,
                    count(it.final_decision) AS decision_count
                   FROM review_instance inst
                     LEFT JOIN review_item it ON it.review_instance_id = inst.id
                  GROUP BY inst.id, it.final_decision) decisions
          WHERE decisions.instance = instance.id AND decisions.decision = 'REMOVE') AS remove_count,
    ( SELECT decisions.decision_count
           FROM ( SELECT inst.id AS instance,
                    it.final_decision AS decision,
                    count(it.final_decision) AS decision_count
                   FROM review_instance inst
                     LEFT JOIN review_item it ON it.review_instance_id = inst.id
                  GROUP BY inst.id, it.final_decision) decisions
          WHERE decisions.instance = instance.id AND decisions.decision IN ('MODIFY_ENTITY', 'MODIFY_ACCESS', 'MODIFY_SUPERVISOR')) AS modify_count
   FROM review_instance instance
     JOIN (SELECT 
          review_instance.id inst_id,
        defn.id defn_id,
        CASE WHEN staged_defn.name IS NULL THEN defn.name ELSE staged_defn.name END AS name,
        CASE WHEN staged_defn.description IS NULL THEN defn.description ELSE staged_defn.description END description,
          defn.review_type,
          defn.next_schedule_date,
          defn.deleted,
          defn.unique_review_def_id
        FROM review_definition defn
          JOIN review_instance ON defn.id = review_instance.review_definition
          LEFT JOIN review_definition staged_defn ON staged_defn.id = review_instance.staged_review_definition_id) defn ON defn.inst_id = instance.id
     JOIN ( SELECT inst.id AS instance,
            count(DISTINCT rui.unique_user_id) AS reviewer_count
           FROM review_instance inst
             LEFT JOIN reviewer_instance ri ON ri.review_instance_id = inst.id
             LEFT JOIN reviewer_user_instance rui ON rui.reviewer_instance_id = ri.id
             WHERE ri.task_count > 0
          GROUP BY inst.id) instances ON instances.instance = instance.id
     LEFT JOIN suser canceler ON instance.canceled_by = canceler.unique_user_id AND canceler.effective = true
     LEFT JOIN ( SELECT DISTINCT defn_1.name AS review_name,
            ri.id AS rid,
            string_agg(s.display_name, ' | ') AS review_owners
           FROM review_instance ri
             JOIN review_definition defn_1 ON ri.review_definition = defn_1.id
             LEFT JOIN reviewer_instance rev_owner_1 ON ri.id = rev_owner_1.review_instance_id AND rev_owner_1.type = 'RO'
             LEFT JOIN reviewer_user_instance rui ON rui.reviewer_instance_id = rev_owner_1.id
             LEFT JOIN suser s ON rui.unique_user_id = s.unique_user_id AND s.effective = true
          GROUP BY defn_1.name, ri.id) multi_owner ON multi_owner.rid = instance.id
    WHERE defn.deleted = false AND instance.status NOT IN ('PREVIEW', 'PREVIEW_CANCELED', 'PREVIEW_STARTING');

CREATE VIEW reassignments_v AS 
 SELECT history.review_instance_id,
    history.review_item_id,
    history.review_order,
    review.name AS review_name,
    review.status AS review_status,
    review.start_date,
    review.end_date,
        CASE
            WHEN item.item_type::text = 'PERMISSION'::text THEN perm.name
            WHEN item.item_type::text = 'ACCOUNT'::text THEN acct.name
            WHEN item.item_type::text = 'ROLE'::text THEN role.name
            WHEN item.item_type::text = 'BUSINESS_ROLE'::text THEN brole.name
            WHEN item.item_type::text = 'DIRECT_REPORT'::text OR item.item_type::text = 'USER'::text THEN suser.display_name
            ELSE NULL::character varying
        END AS review_item,
    item.item_type,
    history.reassigned_by_user_unique_id,
    history.reassigned_by,
    history.reassigned_from,
    history.reassigned_to,
    reason.id AS reason_id,
    reason.name AS reassigned_reasons,
    history.reviewer_comment,
    history.reassignment_date
   FROM ( SELECT ria_1.id,
            ria_1.review_item_id,
            ria_1.review_order,
            ria_1.reason_id,
            ria_1.reviewer_comment,
            ria_1.action,
            ria_1.reassigned_by_user_unique_id,
                CASE
                    WHEN reassign_from_multi.users IS NOT NULL AND reassign_from_multi.grps IS NOT NULL THEN concat(reassign_from_multi.users, concat(' | ', reassign_from_multi.grps))
                    ELSE COALESCE(reassign_from_multi.grps, reassign_from_multi.users, reassign_from.display_name::text)
                END AS reassigned_from,
            reassign_by.display_name AS reassigned_by,
            reassign_to.display_name AS reassigned_to,
            ria_1.review_instance_id,
                CASE
                    WHEN ria_1.assigned IS NULL AND ria_1.action::text = 'REASSIGNED'::text THEN null_date.assigned
                    ELSE ria_1.assigned
                END AS reassignment_date
           FROM review_item_action ria_1
             JOIN reviewer_instance ri ON ri.id = ria_1.reviewer_instance_id
             JOIN suser reassign_from ON ri.single_reviewer_unique_id::text = reassign_from.unique_user_id::text AND reassign_from.effective = true
             JOIN suser reassign_by ON reassign_by.unique_user_id::text = ria_1.reassigned_by_user_unique_id::text AND reassign_by.effective = true
             JOIN suser reassign_to ON reassign_to.unique_user_id::text = ria_1.reassigned_to_user_unique_id::text AND reassign_to.effective = true
             LEFT JOIN review_item_action null_date ON null_date.review_item_id = ria_1.review_item_id AND null_date.review_order = ria_1.review_order AND null_date.action::text = 'PENDING'::text
             LEFT JOIN ( SELECT ria_1_1.id,
                    string_agg(reassign_from_users_1.display_name::text, ' | '::text ORDER BY (reassign_from_users_1.display_name::text)) AS users,
                    string_agg(reassign_from_grps_1.name::text, ' | '::text ORDER BY (reassign_from_grps_1.name::text)) AS grps
                   FROM review_item_action ria_1_1
                     JOIN ( SELECT DISTINCT reviewer_set_instance.reviewer_instance_id,
                            reviewer_set_instance.unique_user_id,
                            reviewer_set_instance.unique_group_id
                           FROM reviewer_set_instance) rsi_1 ON ria_1_1.reviewer_instance_id = rsi_1.reviewer_instance_id
                     LEFT JOIN suser reassign_from_users_1 ON rsi_1.unique_user_id::text = reassign_from_users_1.unique_user_id::text AND reassign_from_users_1.effective = true
                     LEFT JOIN sgroup reassign_from_grps_1 ON reassign_from_grps_1.unique_group_id::text = rsi_1.unique_group_id::text AND reassign_from_grps_1.effective = true
                  WHERE ria_1_1.action::text = 'REASSIGNED'::text
                  GROUP BY ria_1_1.id) reassign_from_multi ON reassign_from_multi.id = ria_1.id
          WHERE ria_1.action::text = 'REASSIGNED'::text) history
     LEFT JOIN review_item_action_reason reason ON history.reason_id = reason.id
     JOIN review_item item ON item.id = history.review_item_id
     LEFT JOIN suser ON item.unique_user_id::text = suser.unique_user_id::text AND suser.effective = true
     LEFT JOIN spermission perm ON perm.unique_permission_id::text = item.unique_permission_id::text AND perm.effective = true
     LEFT JOIN saccount acct ON acct.unique_account_id::text = item.unique_account_id::text AND acct.effective = true
     LEFT JOIN business_role brole ON brole.id = item.business_role_id
     LEFT JOIN role_policy role ON role.id = item.role_id
     JOIN ( SELECT ri.id AS inst_id,
            defn.id AS defn_id,
            COALESCE(staged_defn.name, defn.name) AS name,
            ri.status,
            ri.start_date,
            ri.end_date
           FROM review_definition defn
             JOIN review_instance ri ON defn.id = ri.review_definition
             LEFT JOIN review_definition staged_defn ON staged_defn.id = ri.staged_review_definition_id
          WHERE ri.status::text <> 'PREVIEW_CANCELED'::text) review ON review.inst_id = history.review_instance_id;

ALTER TABLE public.reassignments_v
  OWNER TO igops;
GRANT ALL ON TABLE public.reassignments_v TO igops;
GRANT SELECT ON TABLE public.reassignments_v TO ig_report_role;

CREATE VIEW reviewer_item_v AS
  SELECT item.id AS item_id,
        CASE
            WHEN rui.unique_user_id IS NULL THEN item.final_cert_by_unique_user_id
            ELSE rui.unique_user_id
        END AS unique_user_id,
      defn.review_type
  FROM review_item item
     LEFT JOIN reviewer_user_instance rui ON rui.reviewer_instance_id = item.curr_reviewer_instance_id AND item.final_cert_by_unique_user_id IS NULL
     JOIN review_instance instance ON item.review_instance_id = instance.id
     JOIN review_definition defn ON instance.review_definition = defn.id;

CREATE VIEW review_item_action_combined_v AS 
SELECT DISTINCT ria.id,
    ria.id AS review_item_action,
    ri.id AS review_item,
    ria.review_instance_id,
    ria.review_item_status,
    ria.assigned,
    ria.last_update,
    ria.reviewer_comment,
    ria.action AS individual_action,
    rialast.id AS last_action,
    riaover.id AS last_override_action,
    riausers.display_name AS supervisor_display_name,
    usr.id AS user_id,
    usr.display_name AS user_display_name,
    usr.unique_user_id AS user_unique_id,
    usr.department AS user_department,
    usr.location AS user_location,
    usr.job_title AS user_job_title,
    usr.job_code AS user_job_code,
    usr.employee_type AS user_employee_type,
    usr.employee_status AS user_employee_status,
    usr.idm_tree_name AS user_idm_tree_name,
    usr.risk AS user_risk,
    usr.display_description AS user_display_description,
    NULL::text AS user_is_supervisor,
    usr.workforce_id AS user_workforce_id,
    perm.id AS permission_id,
    perm.name AS permission_name,
    acct.id AS account_id,
    acct.account_id AS account_external_id,
    acct.name AS account_name,
    app.id AS application_id,
    ds.name AS application_name,
    app.show_acct_name_on_review_item AS show_account_name,
    app.show_permission_hierachy,
    ri.item_type,
    ri.relation_to_user_type,
    ri.parent_review_item_id,
    rolepol.id AS role_id,
    rolepol.name AS role_name,
    brole.id AS business_role_id,
    brole.name AS business_role_name,
        CASE
            WHEN brole.id IS NOT NULL THEN brole.name
            WHEN rolepol.id IS NOT NULL THEN rolepol.name
            WHEN perm.id IS NOT NULL THEN perm.name
            WHEN acct.id IS NOT NULL THEN acct.name
            WHEN usr.id IS NOT NULL THEN usr.display_name
            ELSE NULL::character varying
        END AS item_display_name,
    ri.authorized_by_policy,
    ria.acted_by_user_unique_id,
    su.display_name,
    usr.deleted AS user_deleted,
    ri.auto_approved AS approved,
        CASE
            WHEN ri.auto_approved = true THEN 'AUTO_APPROVED'::text
            WHEN ri.auto_approved = false THEN 'USER_APPROVED'::text
            ELSE 'NOT_APPROVED'::text
        END as approved_so,
        CASE
            WHEN brole.id IS NOT NULL AND brole.approval_state::text = 'ARCHIVE'::text THEN true
            WHEN rolepol.id IS NOT NULL THEN rolepol.deleted
            WHEN perm.id IS NOT NULL THEN perm.deleted
            WHEN acct.id IS NOT NULL THEN acct.deleted
            WHEN usr.id IS NOT NULL THEN usr.deleted
            ELSE false
        END AS item_deleted,
    acct.cost AS acct_cost,
    acct.description AS acct_description,
    acct.disabled AS acct_disabled,
    acct.last_login AS acct_last_login,
    acct.privileged AS acct_privileged,
    acct.risk AS acct_risk,
    acct.state AS acct_state,
    acct.type AS acct_type,
    perm.cost AS perm_cost,
    perm.description AS perm_description,
    perm.logical_id AS perm_logical_id,
    perm.permission_id AS perm_external_id,
    perm.risk AS perm_risk,
    perm.type AS perm_type,
    perm.value AS perm_value,
    modifier.name,
    attr.attr_string_val,
    attrdef.attribute_key,
    attr_1.attr_string_val AS changed_from,
    attrdef_1.attribute_key AS changed_from_key
   FROM review_item_action ria
     JOIN review_item ri ON ri.id = ria.review_item_id
     JOIN review_instance r ON r.id = ri.review_instance_id
     JOIN reviewer_user_instance rrui ON rrui.reviewer_instance_id = ria.reviewer_instance_id
     LEFT JOIN review_item_attr_value attr ON attr.modify_action_id = ria.id AND ria.review_item_status::text = 'CERTIFIED'::text AND attr.change_type = 1
     LEFT JOIN attribute_definition attrdef ON attrdef.id = attr.attribute_definition_id
     LEFT JOIN review_item_attr_value attr_1 ON attr_1.modify_action_id = ria.id AND ria.review_item_status::text = 'CERTIFIED'::text AND attr_1.change_type = 2 AND attr_1.attribute_definition_id = attr.attribute_definition_id
     LEFT JOIN attribute_definition attrdef_1 ON attrdef_1.id = attr_1.attribute_definition_id
     LEFT JOIN suser usr ON usr.unique_user_id::text = ri.unique_user_id::text AND usr.effective = true
     LEFT JOIN suser su ON su.unique_user_id::text = ria.acted_by_user_unique_id::text AND su.deleted = false AND su.effective = true
     LEFT JOIN review_item_action rialast ON rialast.id = ri.last_action_id
     LEFT JOIN review_item_action riaover ON riaover.id = ri.last_override_action_id
     LEFT JOIN spermission perm ON perm.unique_permission_id::text = ri.unique_permission_id::text AND perm.effective = true
     LEFT JOIN saccount acct ON acct.unique_account_id::text = ri.unique_account_id::text AND acct.effective = true
     LEFT JOIN application app ON app.id =
        CASE
            WHEN perm.id IS NULL THEN acct.application
            ELSE perm.application
        END
     LEFT JOIN data_source ds ON ds.id = app.id
     LEFT JOIN role_policy rolepol ON rolepol.id = ri.role_id
     LEFT JOIN business_role brole ON brole.id = ri.business_role_id
     LEFT JOIN ( SELECT ria_1.review_item_id,
            string_agg(su_1.display_name::text, ' | '::text ORDER BY (su_1.display_name::text)) AS name
           FROM review_item_action ria_1
             JOIN reviewer_user_instance rrui_1 ON rrui_1.reviewer_instance_id = ria_1.reviewer_instance_id
             JOIN suser su_1 ON su_1.unique_user_id::text = rrui_1.unique_user_id::text AND su_1.deleted = false AND su_1.effective = true
          WHERE ria_1.review_item_status::text = 'REASSIGNED'::text
          GROUP BY ria_1.review_item_id) modifier ON modifier.review_item_id = ria.review_item_id
    LEFT JOIN ( SELECT suser.display_name,
            usi.review_item_action,
            usi.updated_supervisor_ids
           FROM ( SELECT ria.review_item_action, SPLIT_PART(assigned_user_unique_ids, ':', 3) AS updated_supervisor_ids
                    FROM review_item_action_users ria) usi
              JOIN suser on suser.unique_user_id = usi.updated_supervisor_ids AND suser.deleted = false AND suser.effective = true) riausers ON riausers.review_item_action::text = ria.id::text;

CREATE VIEW reviewer_user_instance_v AS 
SELECT revi.id AS reviewer_instance_id,
    ria.review_item_id,
    ria.id AS review_item_action_id,
    ria.review_instance_id,
    ria.action,
    ria.last_update,
    ria.review_item_status,
    ria.reviewer_type,
    ria.reviewer_comment,
    su2.display_name AS single_reviewer,
    rsi_user1.multiple_selected_reviewers,
    rsi.multiple_reviewers,
    rsi_group1.multiple_groups,
    br.name AS brole_name
   FROM review_item_action ria
     LEFT JOIN review_instance ri ON ri.id = ria.review_instance_id
     LEFT JOIN reviewer_instance revi ON revi.id = ria.reviewer_instance_id
     LEFT JOIN reviewer_set_definition rsd_brole ON rsd_brole.reviewer_definition_id = revi.business_role_id AND rsd_brole.unique_business_role_id IS NOT NULL
     LEFT JOIN ( SELECT DISTINCT rsi_user.reviewer_instance_id,
            string_agg(su1.display_name::text, ' | '::text ORDER BY (su1.display_name::text)) AS multiple_selected_reviewers
           FROM reviewer_set_instance rsi_user
             JOIN suser su1 ON su1.unique_user_id::text = rsi_user.unique_user_id::text AND su1.effective = true
          WHERE rsi_user.unique_user_id IS NOT NULL
          GROUP BY rsi_user.reviewer_instance_id) rsi_user1 ON rsi_user1.reviewer_instance_id = revi.id
     LEFT JOIN ( SELECT DISTINCT rsi_group.reviewer_instance_id,
            string_agg(DISTINCT sgrp1.name::text, ' | '::text ORDER BY (sgrp1.name::text)) AS multiple_groups
           FROM reviewer_set_instance rsi_group
             JOIN sgroup sgrp1 ON sgrp1.unique_group_id::text = rsi_group.unique_group_id::text AND sgrp1.effective = true
          WHERE rsi_group.unique_group_id IS NOT NULL
          GROUP BY rsi_group.reviewer_instance_id) rsi_group1 ON rsi_group1.reviewer_instance_id = revi.id
     LEFT JOIN ( SELECT DISTINCT name_list.reviewer_instance_id,
            string_agg(name_list.display_name::text, ' | '::text ORDER BY (name_list.display_name::text)) AS multiple_reviewers
           FROM ( SELECT DISTINCT rsi_1.reviewer_instance_id,
                    su2_1.unique_user_id,
                    su2_1.display_name
                   FROM reviewer_set_instance rsi_1
                     LEFT JOIN sgroup sgrp ON sgrp.unique_group_id::text = rsi_1.unique_group_id::text AND sgrp.effective = true
                     LEFT JOIN sgroup_user sgrp_user ON sgrp_user.sgroup = sgrp.id
                     LEFT JOIN suser su1 ON su1.id = sgrp_user.suser AND su1.effective = true
                     LEFT JOIN suser su2_1 ON su2_1.unique_user_id::text = su1.unique_user_id::text OR su2_1.unique_user_id::text = rsi_1.unique_user_id::text AND su2_1.effective = true) name_list
          GROUP BY name_list.reviewer_instance_id) rsi ON rsi.reviewer_instance_id = revi.id
     LEFT JOIN business_role br ON br.id = revi.business_role_id
     LEFT JOIN suser su2 ON su2.unique_user_id::text = revi.single_reviewer_unique_id::text AND su2.effective = true AND revi.shared_queue = false;

ALTER TABLE access_review_details_v
  OWNER TO igops;
GRANT ALL ON TABLE access_review_details_v TO igops;
GRANT SELECT ON TABLE access_review_details_v TO ig_report_role;

ALTER TABLE access_review_summary_v
  OWNER TO igops;
GRANT ALL ON TABLE access_review_summary_v TO igops;
GRANT SELECT ON TABLE access_review_summary_v TO ig_report_role;

ALTER TABLE reassignments_v
  OWNER TO igops;
GRANT ALL ON TABLE reassignments_v TO igops;
GRANT SELECT ON TABLE reassignments_v TO ig_report_role;

ALTER TABLE reviewer_item_v
  OWNER TO igops;
GRANT ALL ON TABLE reviewer_item_v TO igops;
GRANT SELECT ON TABLE reviewer_item_v TO ig_report_role;

ALTER TABLE review_item_action_combined_v
  OWNER TO igops;
GRANT ALL ON TABLE review_item_action_combined_v TO igops;
GRANT SELECT ON TABLE review_item_action_combined_v TO ig_report_role;

ALTER TABLE reviewer_user_instance_v 
  OWNER TO igops;
GRANT ALL ON TABLE reviewer_user_instance_v TO igops;
GRANT SELECT ON TABLE reviewer_user_instance_v TO ig_report_role;