DROP VIEW IF EXISTS request_lifecycle_v;

DROP VIEW IF EXISTS perm_assignment_changes_v;

DROP VIEW IF EXISTS user_perm_snapshot_v;

CREATE VIEW request_lifecycle_v AS
 SELECT perm.name AS permissionname,
    perm.unique_permission_id,
    stepuser.display_name AS stepapprovername,
    stepuser.unique_user_id AS stepapproveruserid,
    rras.finish_time AS stepapprovaltime,
    stepuser2.display_name AS statussettername,
    stepuser2.unique_user_id AS statussetteruserid,
    rris.status_time AS statussettime,
    rr.id AS cartid,
    rri.id,
    rras.step AS stepapprovalorder,
    rr.request_status,
    rris.item_approval_status AS stepapproveordenystatus,
    role.name AS rolename,
    ds.name AS appname,
    requser.display_name AS requestusername,
    requser.unique_user_id AS requestuserid,
    rr.creation_time AS requesttime,
    apprvuser.display_name AS approveordenyusername,
    apprvuser.unique_user_id AS approveordenyuserid,
        CASE
            WHEN rras.id IS NULL THEN NULL::character varying
            ELSE rri.item_approval_status
        END AS approveordenystatus,
    rri.item_approval_status,
    rri.last_status_update_time AS approveordenytime,
    recipuser.display_name AS recipientusername,
    recipuser.unique_user_id AS recipientuserid,
    rri.add_item AS add_request,
    rri.reason AS item_reason,
    rr.reason AS request_reason,
    rras.finish_comment AS approval_comment,
    cia.change_item_status AS fulfillment_status,
    cia.item_comment AS fulfillment_comment,
    cia.last_updated AS fulfillment_date,
    fufusr.display_name AS fulfillerusername,
    fufusr.unique_user_id AS fulfilleruserid,
        CASE
            WHEN cri.fulfiller_type::text = 'GROUP'::text THEN sgrp.name
            WHEN cri.fulfiller_type::text = 'USER'::text THEN susr.display_name
            ELSE NULL::character varying
        END AS assigned_fulfiller
   FROM resource_request_item rri
     JOIN resource_request rr ON rr.id = rri.resource_request_id
     LEFT JOIN resource_request_item_step rris ON rris.resource_request_item_id = rri.id
     LEFT JOIN suser requser ON requser.unique_user_id::text = rr.unique_requester_id::text AND requser.effective = true
     LEFT JOIN suser recipuser ON recipuser.unique_user_id::text = rri.unique_recipient_id::text AND recipuser.effective = true
     LEFT JOIN spermission perm ON perm.unique_permission_id::text = rri.unique_item_id::text AND perm.effective = true AND rri.item_type::text = 'PERMISSION'::text
     LEFT JOIN role_policy role ON role.unique_policy_id::text = rri.unique_item_id::text AND role.deleted = false AND rri.item_type::text = 'TECHNICAL_ROLE'::text
     LEFT JOIN application app ON app.id::character varying(255)::text = rri.unique_item_id::text AND rri.item_type::text = 'APPLICATION'::text
     LEFT JOIN data_source ds ON ds.id = app.id AND ds.deleted = false
     LEFT JOIN fulfillment_item_combined fic ON fic.id = rri.change_request_item_id
     LEFT JOIN change_request_item cri ON cri.id = fic.id
     LEFT JOIN change_item_action cia ON cia.id = cri.latest_action
     LEFT JOIN suser fufusr ON cia.fulfiller_id::text = fufusr.unique_user_id::text AND fufusr.effective = true
     LEFT JOIN resource_request_approval_step rras ON rras.id = rris.approval_step_id
     LEFT JOIN suser stepuser ON stepuser.unique_user_id::text = rras.finished_by::text AND stepuser.effective = true
     LEFT JOIN suser stepuser2 ON stepuser2.unique_user_id::text = rris.status_set_by::text AND stepuser2.effective = true
     LEFT JOIN suser apprvuser ON apprvuser.unique_user_id::text = rri.status_changed_by::text AND apprvuser.effective = true AND rras.id IS NOT NULL
     LEFT JOIN sgroup sgrp ON sgrp.unique_group_id::text = cri.fulfiller::text AND sgrp.effective = true
     LEFT JOIN suser susr ON susr.unique_user_id::text = cri.fulfiller::text AND susr.effective = true;

CREATE VIEW user_perm_snapshot_v AS 
 SELECT ds.unique_id AS appl_qid,
    ds.name AS appl_name,
    perm.unique_permission_id AS perm_qid,
    perm.name AS perm_name,
    perm.description,
    perm.type,
    root_perm.root_permission,
    assoc_perm.associated_permission,
    suser.display_name,
    suser.unique_user_id,
    suser.last_name AS user_last_name,
    suser.first_name AS user_first_name,
    suser.unique_user_id AS user_qid,
    effective_user.id AS effective_user_id,
    suser.dn AS user_dn,
    suser.collected_entity,
    suser.idm_tree_name AS user_idm_tree_name,
    snap.id AS snap_id,
    snap.timestamp AS snap_timestamp
   FROM spermission perm
     LEFT JOIN data_source ds ON perm.application = ds.id
     LEFT JOIN saccount_permission ap ON perm.collected_entity = ap.spermission
     LEFT JOIN saccount_user sau ON sau.saccount = ap.saccount
     JOIN suser ON sau.suser = suser.collected_entity
     JOIN suser effective_user ON suser.unique_user_id = effective_user.unique_user_id AND effective_user.effective = true
     LEFT JOIN snapshot_sversions au_snap_sv ON sau.sversion = au_snap_sv.sversions
     LEFT JOIN snapshot snap ON au_snap_sv.snapshot = snap.id AND snap.type = 'PUBLICATION'
     LEFT JOIN ( SELECT spermission.unique_permission_id,
            spermission.collected_entity,
            string_agg(perm2.name, ' • ' ORDER BY (perm2.name)) AS root_permission
           FROM spermission
             JOIN (SELECT DISTINCT se.entailed, se.entailing FROM spermission_entailment se) entail ON spermission.collected_entity = entail.entailed
             JOIN spermission perm2 ON perm2.collected_entity = entail.entailing AND perm2.deleted = false
          GROUP BY spermission.unique_permission_id, spermission.collected_entity) root_perm ON root_perm.unique_permission_id = perm.unique_permission_id AND root_perm.collected_entity = perm.collected_entity
     LEFT JOIN ( SELECT spermission.unique_permission_id,
            spermission.collected_entity,
            string_agg(perm3.name, ' • ' ORDER BY (perm3.name)) AS associated_permission
           FROM spermission
             JOIN (SELECT DISTINCT se.entailed, se.entailing FROM spermission_entailment se) entail ON spermission.collected_entity = entail.entailing
             JOIN spermission perm3 ON perm3.collected_entity = entail.entailed AND perm3.deleted = false
          GROUP BY spermission.unique_permission_id, spermission.collected_entity) assoc_perm ON assoc_perm.unique_permission_id = perm.unique_permission_id AND assoc_perm.collected_entity = perm.collected_entity
  WHERE perm.type NOT IN ('BoundEntitlement', 'ResourceParm', 'BoundResourceParm') AND perm.merged = false AND snap.timestamp IS NOT NULL;

CREATE VIEW perm_assignment_changes_v AS
WITH snaps AS(
  SELECT DISTINCT * FROM user_perm_snapshot_v
), dates AS(
  SELECT DISTINCT
    ups.snap_timestamp,
    ups2.snap_timestamp AS prev_time,
    ups3.snap_timestamp AS next_time
  FROM (SELECT DISTINCT snaps.snap_timestamp FROM snaps) ups
    LEFT JOIN (SELECT DISTINCT snaps.snap_timestamp FROM snaps) ups2 ON ups2.snap_timestamp = (SELECT MAX(ups4.snap_timestamp) FROM snaps ups4 WHERE ups4.snap_timestamp < ups.snap_timestamp)
    LEFT JOIN (SELECT DISTINCT snaps.snap_timestamp FROM snaps) ups3 ON ups3.snap_timestamp = (SELECT MIN(ups4.snap_timestamp) FROM snaps ups4 WHERE ups4.snap_timestamp > ups.snap_timestamp)
)
SELECT
  'ADDED' AS added_removed,
  ups.appl_qid,
  ups.appl_name,
  ups.perm_qid,
  ups.perm_name,
  ups.description,
  ups.type,
  ups.root_permission,
  ups.associated_permission,
  ups.display_name,
  ups.unique_user_id,
  ups.user_last_name,
  ups.user_first_name,
  ups.user_qid,
  ups.effective_user_id,
  ups.user_dn,
  ups.collected_entity,
  ups.user_idm_tree_name,
  ups.snap_id,
  ups.snap_timestamp AS add_remove_date
FROM snaps ups
  JOIN dates ON ups.snap_timestamp = dates.snap_timestamp
WHERE NOT EXISTS
  (SELECT
    1
  FROM snaps ups2
  WHERE ups2.snap_timestamp = dates.prev_time AND ups2.perm_qid =  ups.perm_qid AND ups2.unique_user_id = ups.unique_user_id)

UNION ALL

SELECT
  'REMOVED' AS added_removed,
  ups.appl_qid,
  ups.appl_name,
  ups.perm_qid,
  ups.perm_name,
  ups.description,
  ups.type,
  ups.root_permission,
  ups.associated_permission,
  ups.display_name,
  ups.unique_user_id,
  ups.user_last_name,
  ups.user_first_name,
  ups.user_qid,
  ups.effective_user_id,
  ups.user_dn,
  ups.collected_entity,
  ups.user_idm_tree_name,
  ups.snap_id,
  dates.next_time AS add_remove_date
FROM snaps ups
  JOIN dates ON ups.snap_timestamp = dates.snap_timestamp
WHERE ups.snap_timestamp != (SELECT MAX(ups4.snap_timestamp) FROM snaps ups4) AND
  NOT EXISTS
    (SELECT
      1
    FROM snaps ups2
    WHERE ups2.snap_timestamp = dates.next_time AND ups2.perm_qid =  ups.perm_qid AND ups2.unique_user_id = ups.unique_user_id);

ALTER TABLE request_lifecycle_v
  OWNER TO igops;
GRANT ALL ON TABLE request_lifecycle_v TO igops;
GRANT SELECT ON TABLE request_lifecycle_v TO ig_report_role;

ALTER TABLE user_perm_snapshot_v
  OWNER TO igops;
GRANT ALL ON TABLE user_perm_snapshot_v TO igops;
GRANT SELECT ON TABLE user_perm_snapshot_v TO ig_report_role;

ALTER TABLE perm_assignment_changes_v
  OWNER TO igops;
GRANT ALL ON TABLE perm_assignment_changes_v TO igops;
GRANT SELECT ON TABLE perm_assignment_changes_v TO ig_report_role;