DROP VIEW IF EXISTS access_review_details_v;
DROP VIEW IF EXISTS business_role_hist_v;
DROP VIEW IF EXISTS reviewer_item_v;
DROP VIEW IF EXISTS role_parameter_v;

CREATE VIEW access_review_details_v AS 
  SELECT DISTINCT item.id AS review_item_id,
    item.last_override_action_id AS last_override_action,
    instance.id AS review_instance_id,
    instance.status AS review_status,
    instance.start_date,
    instance.end_date,
    instance.valid_to_date,
    action.last_update,
    item.approved_time,
    instance.item_count,
    multi_owner.review_owners,
    multi_reviewer.reviewer_ids AS reviewer_id,
        CASE
            WHEN item.final_cert_by_unique_user_id IS NOT NULL THEN final_reviewer.display_name::text
            ELSE multi_reviewer.reviewers
        END AS reviewer_display_name,
        CASE
            WHEN item.final_cert_by_unique_user_id IS NOT NULL THEN "substring"(split_part(final_reviewer.dn::text, ','::text, 1), 4)
            ELSE multi_reviewer.user_cn
        END AS reviewer_cn,
    reassigned_by_history.reviewers AS reassigned_by_user_disp_name,
    reassigned_from_history.reviewers AS reassigned_from_user_disp_name,
    reassigned_by_history.reassigned_date,
    reassigned_by_history.reasons AS reassigned_reasons,
    reassigned_by_history.rev_comments AS reassigned_comments,
    defn.name AS review_name,
    substr(defn.description, 0, 4000) AS description,
    defn.review_type,
    defn.next_schedule_date,
        CASE
            WHEN item.item_type::text = 'ACCOUNT'::text AND usr.id IS NULL THEN shacc.suser
            ELSE usr.id
        END AS user_id,
        CASE
            WHEN item.item_type::text = 'ACCOUNT'::text AND usr.display_name IS NULL THEN shacc.display_name
            ELSE usr.display_name
        END AS user_display_name,
        CASE
            WHEN item.item_type::text = 'ACCOUNT'::text AND usr.display_name IS NULL THEN shacc.user_cn
            ELSE "substring"(split_part(usr.dn::text, ','::text, 1), 4)
        END AS user_cn,
    action.action AS reviewer_action,
    reason.name AS modify_reason_name,
    reason.id AS modify_reason_id,
        CASE
            WHEN fulfillment.review_item_id IS NOT NULL THEN true
            ELSE false
        END AS remove_completed,
    usr.unique_user_id,
    action.review_order,
    action.reviewer_comment,
    perm.id AS permission_id,
    perm.name AS permission_name,
    acct.id AS account_id,
    acct.account_id AS account_native_id,
    acct.name AS account_name,
    acct.relation_to_user_type AS account_relation,
    app.id AS application_id,
    app_ds.name AS application_name,
    item.final_decision,
    item.final_decision_time,
    role.id AS role_id,
    role.name AS role_name,
    item.item_type,
    brole.name AS brole_name,
    brole.id AS business_role_id,
    action.fulfillment_instructions,
    esc_reviewer.display_name AS esc_reviewer,
    sup.unique_user_id AS supervisor_id,
    sup.display_name AS supervisor_display_name
   FROM review_item item
     JOIN review_instance instance ON item.review_instance_id = instance.id AND (instance.status::text <> ALL (ARRAY['PREVIEW'::character varying, 'PREVIEW_CANCELED'::character varying, 'PREVIEW_STARTING'::character varying]::text[]))
     JOIN ( SELECT review_instance.id AS inst_id,
            defn_1.id AS defn_id,
                CASE
                    WHEN staged_defn.name IS NULL THEN defn_1.name
                    ELSE staged_defn.name
                END AS name,
                CASE
                    WHEN staged_defn.description IS NULL THEN defn_1.description
                    ELSE staged_defn.description
                END AS description,
                CASE
                    WHEN staged_defn.next_schedule_date IS NULL THEN defn_1.next_schedule_date
                    ELSE staged_defn.next_schedule_date
                END AS next_schedule_date,
            defn_1.review_type,
            defn_1.deleted
           FROM review_definition defn_1
             JOIN review_instance ON defn_1.id = review_instance.review_definition
             LEFT JOIN review_definition staged_defn ON staged_defn.id = review_instance.staged_review_definition_id) defn ON instance.id = defn.inst_id
     LEFT JOIN suser usr ON usr.unique_user_id::text = item.unique_user_id::text AND usr.effective = true
     LEFT JOIN suser sup ON sup.unique_user_id = item.unique_supervisor_id AND sup.effective = true
     LEFT JOIN spermission perm ON perm.unique_permission_id::text = item.unique_permission_id::text AND perm.effective = true
     LEFT JOIN saccount acct ON acct.unique_account_id::text = item.unique_account_id::text AND acct.effective = true
     LEFT JOIN application app ON app.id =
        CASE
            WHEN perm.id IS NULL THEN acct.application
            ELSE perm.application
        END
     LEFT JOIN business_role brole ON brole.id = item.business_role_id
     LEFT JOIN role_policy role ON role.id = item.role_id
     LEFT JOIN data_source app_ds ON app_ds.id = app.id
     LEFT JOIN review_item_action action ON action.review_item_id = item.id AND action.action::text <> 'REASSIGNED'::text
     LEFT JOIN review_item_action_reason reason ON reason.id = action.reason_id
     LEFT JOIN reviewer_instance inst2 ON instance.id = inst2.review_instance_id AND inst2.type::text = 'RO'::text
     LEFT JOIN reviewer_instance inst3 ON instance.id = inst3.review_instance_id
     LEFT JOIN ( SELECT DISTINCT item_1.id AS item_id,
            string_agg(multi_reviewers.id::text, ' | '::text ORDER BY multi_reviewers.display_name) AS reviewer_ids,
            string_agg(multi_reviewers.unique_user_id::text, ' | '::text ORDER BY multi_reviewers.display_name) AS reviewer_unique_ids,
            string_agg(multi_reviewers.display_name::text, ' | '::text ORDER BY (multi_reviewers.display_name::text)) AS reviewers,
            string_agg("substring"(split_part(multi_reviewers.dn::text, ','::text, 1), 4), ' | '::text ORDER BY multi_reviewers.display_name) AS user_cn
           FROM review_item item_1
             LEFT JOIN ( SELECT DISTINCT reviewer_user_instance.reviewer_instance_id,
                    reviewer_user_instance.unique_user_id
                   FROM reviewer_user_instance) rrui ON rrui.reviewer_instance_id = item_1.curr_reviewer_instance_id
             LEFT JOIN suser multi_reviewers ON rrui.unique_user_id::text = multi_reviewers.unique_user_id::text AND multi_reviewers.effective = true
          GROUP BY item_1.id) multi_reviewer ON multi_reviewer.item_id = item.id
     LEFT JOIN suser final_reviewer ON item.final_cert_by_unique_user_id::text = final_reviewer.unique_user_id::text AND final_reviewer.effective = true
     LEFT JOIN suser acting_reviewer ON action.acted_by_user_unique_id::text = acting_reviewer.unique_user_id::text AND acting_reviewer.effective = true
     LEFT JOIN ( SELECT ria.review_item_id,
            string_agg(usr_1.display_name::text, ' | '::text ORDER BY ria.assigned) AS reviewers
           FROM review_item_action ria
             LEFT JOIN reviewer_instance ri ON ri.id = ria.reviewer_instance_id
             LEFT JOIN suser usr_1 ON ri.single_reviewer_unique_id::text = usr_1.unique_user_id::text AND usr_1.effective = true
          WHERE ria.action::text = 'REASSIGNED'::text
          GROUP BY ria.review_item_id) reassigned_from_history ON reassigned_from_history.review_item_id = item.id
     LEFT JOIN ( SELECT ria.review_item_id,
            string_agg(usr_1.display_name::text, ' | '::text ORDER BY ria.assigned) AS reviewers,
            string_agg(to_char(ria.assigned, 'YYYY-MM-DD HH24:MI:SS'::text), ' | '::text ORDER BY ria.assigned) AS reassigned_date,
            string_agg(COALESCE(reason_1.name, 'NULL_REASON'::character varying)::text, ' | '::text ORDER BY ria.assigned) AS reasons,
            string_agg(COALESCE(NULLIF(ria.reviewer_comment::text, ''::text), 'NULL_COMMENT'::text), ' | '::text ORDER BY ria.assigned) AS rev_comments
           FROM review_item_action ria
             LEFT JOIN review_item_action_reason reason_1 ON ria.reason_id = reason_1.id
             LEFT JOIN suser usr_1 ON ria.reassigned_by_user_unique_id::text = usr_1.unique_user_id::text AND usr_1.effective = true
          WHERE ria.action::text = 'REASSIGNED'::text
          GROUP BY ria.review_item_id) reassigned_by_history ON reassigned_by_history.review_item_id = item.id
     LEFT JOIN shared_accounts_v shacc ON shacc.unique_account_id::text = acct.unique_account_id::text
     LEFT JOIN ( SELECT DISTINCT defn_1.name AS review_name,
            ri.id AS rid,
            string_agg(s.display_name::text, ' | '::text ORDER BY (s.display_name::text)) AS review_owners
           FROM review_instance ri
             JOIN review_definition defn_1 ON ri.review_definition = defn_1.id
             LEFT JOIN reviewer_instance rev_owner ON ri.id = rev_owner.review_instance_id AND rev_owner.type::text = 'RO'::text
             LEFT JOIN reviewer_user_instance rui ON rui.reviewer_instance_id = rev_owner.id
             LEFT JOIN suser s ON rui.unique_user_id::text = s.unique_user_id::text AND s.effective = true
          GROUP BY defn_1.name, ri.id) multi_owner ON multi_owner.rid = item.review_instance_id
     LEFT JOIN ( SELECT esc_reviewers.review_instance_id,
            string_agg(esc_reviewers.display_name::text, ' | '::text ORDER BY (esc_reviewers.display_name::text)) AS display_name
           FROM ( SELECT DISTINCT u.display_name,
                    rri.review_instance_id
                   FROM review_item ri
                     JOIN reviewer_instance rri ON ri.review_instance_id = rri.review_instance_id AND rri.type::text = 'EO'::text
                     JOIN reviewer_user_instance rriu ON rriu.reviewer_instance_id = rri.id
                     JOIN suser u ON u.unique_user_id::text = rriu.unique_user_id::text AND u.effective = true) esc_reviewers
          GROUP BY esc_reviewers.review_instance_id) esc_reviewer ON esc_reviewer.review_instance_id = item.review_instance_id
     LEFT JOIN ( SELECT cri.review_item_id,
            cia.action,
            cia.change_item_status
           FROM change_request_item cri
             LEFT JOIN change_item_action cia ON cri.latest_action = cia.id
          WHERE cri.change_request_type::text ~~ ('REMOVE'::text || '%'::text) AND cia.action::text = 'FULFILLED'::text AND (cia.change_item_status::text = 'VERIFIED'::text OR cia.change_item_status::text = 'COMPLETED'::text)) fulfillment ON item.id = fulfillment.review_item_id
  WHERE defn.deleted = false AND (acting_reviewer.unique_user_id::text ~~ (('%'::text || multi_reviewer.reviewer_unique_ids) || '%'::text) OR acting_reviewer.unique_user_id IS NULL AND (action.review_order = (( SELECT DISTINCT min(act.review_order) AS min
           FROM review_item_action act
          WHERE (act.action::text = 'PENDING'::text OR act.action::text = 'ESCALATE'::text) AND act.review_item_id = item.id)) OR action.acted_by_policy = true) OR acting_reviewer.unique_user_id IS NOT NULL AND action.shared_queue = true) AND (action.action::text <> 'PENDING'::text OR item.final_decision IS NULL);
  
CREATE VIEW business_role_hist_v AS
 SELECT b_role.id AS business_role_id,
    b_role.unique_id AS business_role_unique_id,
    b_role.name,
    b_role.description,
    b_role.grace_period,
    b_role.risk,
    b_role.creation_time,
    b_role.update_time,
    b_role.approval_time,
    b_role.publish_time,
    COALESCE(user_owners.user_owners, ''::text) AS user_owners,
    COALESCE(group_owners.group_owners, ''::text) AS group_owners,
    b_role.approval_state,
    su1.display_name AS created_by,
    su2.display_name AS published_by,
    su3.display_name AS changed_by,
    su4.display_name AS approved_by,
    COALESCE(user_manage.user_managers, ''::text) AS user_managers,
    COALESCE(group_manage.group_managers, ''::text) AS group_managers,
    su5.display_name AS user_fulfiller,
    sgroup.name AS group_fulfiller,
    b_role.approval_note,
    ap.name AS policy_name,
    ap.implicit AS approval_not_required,
    ap.description AS policy_description,
    ap.creation_time AS policy_creation_time,
    ap.update_time AS last_policy_update_time,
    COALESCE(pol_users.policy_users, ''::text) AS policy_users,
    COALESCE(pol_groups.policy_groups, ''::text) AS policy_groups,
    br_crit.criteria AS membership_criteria,
    COALESCE(incl_users.included_users, ''::text) AS included_users,
    COALESCE(incl_groups.included_groups, ''::text) AS included_groups,
    COALESCE(excl_users.excluded_users, ''::text) AS excluded_users,
    COALESCE(excl_groups.excluded_groups, ''::text) AS excluded_groups,
    auth_perm.auth_permission,
    COALESCE(auth_tech_role.auth_tech_role, ''::text) AS auth_tech_role,
    COALESCE(auth_app.auth_application, ''::text) AS auth_application,
    incl_brole.bnames AS included_broles
   FROM business_role b_role
     LEFT JOIN suser su1 ON b_role.created_by::text = su1.unique_user_id::text AND su1.display_name IS NOT NULL AND su1.deleted = false AND su1.effective = true
     LEFT JOIN suser su2 ON b_role.published_by::text = su2.unique_user_id::text AND su2.display_name IS NOT NULL AND su2.deleted = false AND su2.effective = true
     LEFT JOIN suser su3 ON b_role.changed_by::text = su3.unique_user_id::text AND su3.display_name IS NOT NULL AND su3.deleted = false AND su3.effective = true
     LEFT JOIN suser su4 ON b_role.approved_by::text = su4.unique_user_id::text AND su4.display_name IS NOT NULL AND su4.deleted = false AND su4.effective = true
     JOIN auth_scope ON auth_scope.unique_business_role_id::text = b_role.unique_id::text
     LEFT JOIN auth_role_mapping ar_map ON ar_map.auth_scope = auth_scope.id AND ar_map.auth_role = 29
     LEFT JOIN ( SELECT auth_scope_1.unique_business_role_id,
            string_agg(DISTINCT suser.display_name::text, ' | '::text ORDER BY (suser.display_name::text)) AS user_managers
           FROM auth_role_mapping ar_map_1
             JOIN suser ON ar_map_1.unique_user_id::text = suser.unique_user_id::text AND suser.display_name IS NOT NULL AND suser.deleted = false AND suser.effective = true
             JOIN auth_scope auth_scope_1 ON ar_map_1.auth_scope = auth_scope_1.id
          WHERE ar_map_1.unique_user_id IS NOT NULL AND ar_map_1.auth_role = 27
          GROUP BY auth_scope_1.unique_business_role_id) user_manage ON user_manage.unique_business_role_id::text = b_role.unique_id::text
     LEFT JOIN ( SELECT auth_scope_1.unique_business_role_id,
            string_agg(DISTINCT sgroup_1.name::text, ' | '::text ORDER BY (sgroup_1.name::text)) AS group_managers
           FROM auth_role_mapping ar_map_1
             JOIN sgroup sgroup_1 ON ar_map_1.unique_group_id::text = sgroup_1.unique_group_id::text AND sgroup_1.name IS NOT NULL
             JOIN auth_scope auth_scope_1 ON ar_map_1.auth_scope = auth_scope_1.id
          WHERE ar_map_1.unique_group_id IS NOT NULL AND ar_map_1.auth_role = 27
          GROUP BY auth_scope_1.unique_business_role_id) group_manage ON group_manage.unique_business_role_id::text = b_role.unique_id::text
     LEFT JOIN suser su5 ON ar_map.unique_user_id::text = su5.unique_user_id::text AND su5.display_name IS NOT NULL AND su5.deleted = false AND su5.effective = true
     LEFT JOIN sgroup ON ar_map.unique_group_id::text = sgroup.unique_group_id::text AND sgroup.name IS NOT NULL AND sgroup.deleted = false AND sgroup.effective = true
     LEFT JOIN approval_policy ap ON
        CASE
            WHEN b_role.approval_policy IS NULL THEN ap.global = true AND ap.deleted = false
            ELSE b_role.approval_policy = ap.id
        END
     LEFT JOIN business_role_memb_crit br_crit ON b_role.id = br_crit.business_role_id
     LEFT JOIN ( SELECT incl_brole2.business_role_id,
            string_agg(incl_brole2.name::text, ' | '::text ORDER BY incl_brole2.business_role_id) AS bnames
           FROM business_role_incl_brole incl_brole2
          GROUP BY incl_brole2.business_role_id) incl_brole ON incl_brole.business_role_id = b_role.id
     LEFT JOIN ( SELECT br_owner_1.business_role_id,
            string_agg(DISTINCT su.display_name::text, ' | '::text ORDER BY (su.display_name::text)) AS user_owners
           FROM business_role_owner br_owner_1
             JOIN suser su ON br_owner_1.unique_owner_id::text = su.unique_user_id::text AND su.display_name IS NOT NULL AND su.deleted = false AND su.effective = true
          GROUP BY br_owner_1.business_role_id) user_owners ON user_owners.business_role_id = b_role.id
     LEFT JOIN ( SELECT br_owner_1.business_role_id,
            string_agg(DISTINCT sgroup_1.name::text, ' | '::text ORDER BY (sgroup_1.name::text)) AS group_owners
           FROM business_role_owner br_owner_1
             JOIN sgroup sgroup_1 ON br_owner_1.unique_owner_id::text = sgroup_1.unique_group_id::text AND sgroup_1.name IS NOT NULL AND sgroup_1.deleted = false AND sgroup_1.effective = true
          GROUP BY br_owner_1.business_role_id) group_owners ON group_owners.business_role_id = b_role.id
     LEFT JOIN ( SELECT apmemb.approval_policy_id,
            string_agg(DISTINCT su.display_name::text, ' | '::text ORDER BY (su.display_name::text)) AS policy_users
           FROM approval_policy_memb apmemb
             JOIN suser su ON apmemb.unique_member_id::text = su.unique_user_id::text AND su.display_name IS NOT NULL AND su.deleted = false AND su.effective = true
          WHERE apmemb.is_user = true
          GROUP BY apmemb.approval_policy_id) pol_users ON pol_users.approval_policy_id = b_role.approval_policy
     LEFT JOIN ( SELECT apmemb.approval_policy_id,
            string_agg(DISTINCT sgroup_1.name::text, ' | '::text ORDER BY (sgroup_1.name::text)) AS policy_groups
           FROM approval_policy_memb apmemb
             JOIN sgroup sgroup_1 ON apmemb.unique_member_id::text = sgroup_1.unique_group_id::text AND sgroup_1.name IS NOT NULL AND sgroup_1.deleted = false AND sgroup_1.effective = true
          WHERE apmemb.is_user = false
          GROUP BY apmemb.approval_policy_id) pol_groups ON pol_groups.approval_policy_id = b_role.approval_policy
     LEFT JOIN ( SELECT br_incl_mem.business_role_id,
            string_agg(DISTINCT su.display_name::text, ' | '::text ORDER BY (su.display_name::text)) AS included_users
           FROM business_role_incl_memb br_incl_mem
             JOIN suser su ON br_incl_mem.unique_member_id::text = su.unique_user_id::text AND su.display_name IS NOT NULL AND su.deleted = false AND su.effective = true
          WHERE br_incl_mem.is_user = true
          GROUP BY br_incl_mem.business_role_id) incl_users ON incl_users.business_role_id = b_role.id
     LEFT JOIN ( SELECT br_incl_mem.business_role_id,
            string_agg(DISTINCT sgroup_1.name::text, ' | '::text ORDER BY (sgroup_1.name::text)) AS included_groups
           FROM business_role_incl_memb br_incl_mem
             JOIN sgroup sgroup_1 ON br_incl_mem.unique_member_id::text = sgroup_1.unique_group_id::text AND sgroup_1.name IS NOT NULL AND sgroup_1.deleted = false AND sgroup_1.effective = true
          WHERE br_incl_mem.is_user = false
          GROUP BY br_incl_mem.business_role_id) incl_groups ON incl_groups.business_role_id = b_role.id
     LEFT JOIN ( SELECT br_excl_mem.business_role_id,
            string_agg(DISTINCT suser.display_name::text, ' | '::text ORDER BY (suser.display_name::text)) AS excluded_users
           FROM business_role_excl_memb br_excl_mem
             JOIN suser ON br_excl_mem.unique_member_id::text = suser.unique_user_id::text AND suser.display_name IS NOT NULL AND suser.deleted = false AND suser.effective = true
          WHERE br_excl_mem.is_user = true
          GROUP BY br_excl_mem.business_role_id) excl_users ON excl_users.business_role_id = b_role.id
     LEFT JOIN ( SELECT br_excl_mem.business_role_id,
            string_agg(DISTINCT sgroup_1.name::text, ' | '::text ORDER BY (sgroup_1.name::text)) AS excluded_groups
           FROM business_role_excl_memb br_excl_mem
             JOIN sgroup sgroup_1 ON br_excl_mem.unique_member_id::text = sgroup_1.unique_group_id::text AND sgroup_1.name IS NOT NULL AND sgroup_1.deleted = false AND sgroup_1.effective = true
          WHERE br_excl_mem.is_user = false
          GROUP BY br_excl_mem.business_role_id) excl_groups ON excl_groups.business_role_id = b_role.id
     LEFT JOIN ( SELECT br_incl_item.business_role_id,
            string_agg(DISTINCT spermission.name::text, ' | '::text ORDER BY (spermission.name::text)) AS auth_permission
           FROM business_role_incl_item br_incl_item
             JOIN spermission ON br_incl_item.unique_item_id::text = spermission.unique_permission_id::text AND spermission.name IS NOT NULL AND spermission.deleted = false AND spermission.effective = true
          WHERE br_incl_item.item_type::text = 'PERMISSION'::text
          GROUP BY br_incl_item.business_role_id) auth_perm ON auth_perm.business_role_id = b_role.id
     LEFT JOIN ( SELECT br_incl_item.business_role_id,
            string_agg(DISTINCT role_policy.name::text, ' | '::text ORDER BY (role_policy.name::text)) AS auth_tech_role
           FROM business_role_incl_item br_incl_item
             JOIN role_policy ON br_incl_item.unique_item_id::text = role_policy.unique_policy_id::text AND role_policy.deleted = false
          WHERE br_incl_item.item_type::text = 'TECHNICAL_ROLE'::text
          GROUP BY br_incl_item.business_role_id) auth_tech_role ON auth_tech_role.business_role_id = b_role.id
     LEFT JOIN ( SELECT br_incl_item.business_role_id,
            string_agg(DISTINCT data_source.name::text, ' | '::text ORDER BY (data_source.name::text)) AS auth_application
           FROM business_role_incl_item br_incl_item
             JOIN data_source ON br_incl_item.unique_item_id::text = data_source.id::text AND data_source.deleted = false
          WHERE br_incl_item.item_type::text = 'APPLICATION'::text
          GROUP BY br_incl_item.business_role_id) auth_app ON auth_app.business_role_id = b_role.id
  WHERE b_role.approval_state::text = 'ARCHIVE'::text OR b_role.approval_state::text = 'PUBLISHED'::text;

CREATE VIEW reviewer_item_v AS
  SELECT item.id AS item_id,
        CASE
            WHEN rui.unique_user_id IS NULL THEN item.final_cert_by_unique_user_id
            ELSE rui.unique_user_id
        END AS unique_user_id,
    	defn.review_type
  FROM review_item item
     LEFT JOIN reviewer_user_instance rui ON rui.reviewer_instance_id = item.curr_reviewer_instance_id AND item.final_cert_by_unique_user_id IS NULL
     JOIN review_instance instance ON item.review_instance_id = instance.id
     JOIN review_definition defn ON instance.review_definition = defn.id;

CREATE VIEW role_parameter_v AS 
  SELECT DISTINCT 
    role.id AS role_id,
    role.name AS role_name,
    role.unique_policy_id
  FROM role_policy role
  WHERE EXISTS ( 
    SELECT 1
    FROM review_item item
    WHERE item.role_id = role.id);

ALTER TABLE access_review_details_v
  OWNER TO igops;
GRANT ALL ON TABLE access_review_details_v TO igops;
GRANT SELECT ON TABLE access_review_details_v TO ig_report_role;

ALTER TABLE business_role_hist_v
  OWNER TO igops;
GRANT ALL ON TABLE business_role_hist_v TO igops;
GRANT SELECT ON TABLE business_role_hist_v TO ig_report_role;

ALTER TABLE reviewer_item_v
  OWNER TO igops;
GRANT ALL ON TABLE reviewer_item_v TO igops;
GRANT SELECT ON TABLE reviewer_item_v TO ig_report_role;

ALTER TABLE role_parameter_v
  OWNER TO igops;
GRANT ALL ON TABLE role_parameter_v TO igops;
GRANT SELECT ON TABLE role_parameter_v TO ig_report_role;