DROP VIEW IF EXISTS reassignments_v;

CREATE VIEW reassignments_v AS 
 SELECT history.review_instance_id,
    history.review_item_id,
    history.review_order,
    review.name AS review_name,
    review.status AS review_status,
    review.start_date,
    review.end_date,
        CASE
            WHEN item.item_type::text = 'PERMISSION'::text THEN perm.name
            WHEN item.item_type::text = 'ACCOUNT'::text THEN acct.name
            WHEN item.item_type::text = 'ROLE'::text THEN role.name
            WHEN item.item_type::text = 'BUSINESS_ROLE'::text THEN brole.name
            WHEN item.item_type::text = 'DIRECT_REPORT'::text OR item.item_type::text = 'USER'::text THEN suser.display_name
            ELSE NULL::character varying
        END AS review_item,
    item.item_type,
    history.reassigned_by_user_unique_id,
    history.reassigned_by,
    history.reassigned_from,
    history.reassigned_to,
    reason.id AS reason_id,
    reason.name AS reassigned_reasons,
    history.reviewer_comment,
    history.reassignment_date
   FROM ( SELECT ria_1.id,
            ria_1.review_item_id,
            ria_1.review_order,
            ria_1.reason_id,
            ria_1.reviewer_comment,
            ria_1.action,
            ria_1.reassigned_by_user_unique_id,
                CASE
                    WHEN reassign_from_multi.users IS NOT NULL AND reassign_from_multi.grps IS NOT NULL THEN concat(reassign_from_multi.users, concat(' | ', reassign_from_multi.grps))
                    ELSE COALESCE(reassign_from_multi.grps, reassign_from_multi.users, reassign_from.display_name::text)
                END AS reassigned_from,
            reassign_by.display_name AS reassigned_by,
            reassign_to.display_name AS reassigned_to,
            ria_1.review_instance_id,
                CASE
                    WHEN ria_1.assigned IS NULL AND ria_1.action::text = 'REASSIGNED'::text THEN null_date.assigned
                    ELSE ria_1.assigned
                END AS reassignment_date
           FROM review_item_action ria_1
             JOIN reviewer_instance ri ON ri.id = ria_1.reviewer_instance_id
             JOIN suser reassign_from ON ri.single_reviewer_unique_id::text = reassign_from.unique_user_id::text AND reassign_from.effective = true
             JOIN suser reassign_by ON reassign_by.unique_user_id::text = ria_1.reassigned_by_user_unique_id::text AND reassign_by.effective = true
             JOIN suser reassign_to ON reassign_to.unique_user_id::text = ria_1.reassigned_to_user_unique_id::text AND reassign_to.effective = true
             LEFT JOIN review_item_action null_date ON null_date.review_item_id = ria_1.review_item_id AND null_date.review_order = ria_1.review_order AND null_date.action::text = 'PENDING'::text
             LEFT JOIN ( SELECT ria_1_1.id,
                    string_agg(reassign_from_users_1.display_name::text, ' | '::text ORDER BY (reassign_from_users_1.display_name::text)) AS users,
                    string_agg(reassign_from_grps_1.name::text, ' | '::text ORDER BY (reassign_from_grps_1.name::text)) AS grps
                   FROM review_item_action ria_1_1
                     JOIN ( SELECT DISTINCT reviewer_set_instance.reviewer_instance_id,
                            reviewer_set_instance.unique_user_id,
                            reviewer_set_instance.unique_group_id
                           FROM reviewer_set_instance) rsi_1 ON ria_1_1.reviewer_instance_id = rsi_1.reviewer_instance_id
                     LEFT JOIN suser reassign_from_users_1 ON rsi_1.unique_user_id::text = reassign_from_users_1.unique_user_id::text AND reassign_from_users_1.effective = true
                     LEFT JOIN sgroup reassign_from_grps_1 ON reassign_from_grps_1.unique_group_id::text = rsi_1.unique_group_id::text AND reassign_from_grps_1.effective = true
                  WHERE ria_1_1.action::text = 'REASSIGNED'::text
                  GROUP BY ria_1_1.id) reassign_from_multi ON reassign_from_multi.id = ria_1.id
          WHERE ria_1.action::text = 'REASSIGNED'::text) history
     LEFT JOIN review_item_action_reason reason ON history.reason_id = reason.id
     JOIN review_item item ON item.id = history.review_item_id
     LEFT JOIN suser ON item.unique_user_id::text = suser.unique_user_id::text AND suser.effective = true
     LEFT JOIN spermission perm ON perm.unique_permission_id::text = item.unique_permission_id::text AND perm.effective = true
     LEFT JOIN saccount acct ON acct.unique_account_id::text = item.unique_account_id::text AND acct.effective = true
     LEFT JOIN business_role brole ON brole.id = item.business_role_id
     LEFT JOIN role_policy role ON role.id = item.role_id
     JOIN ( SELECT ri.id AS inst_id,
            defn.id AS defn_id,
            COALESCE(staged_defn.name, defn.name) AS name,
            ri.status,
            ri.start_date,
            ri.end_date
           FROM review_definition defn
             JOIN review_instance ri ON defn.id = ri.review_definition
             LEFT JOIN review_definition staged_defn ON staged_defn.id = ri.staged_review_definition_id
          WHERE ri.status::text <> 'PREVIEW_CANCELED'::text) review ON review.inst_id = history.review_instance_id;

ALTER TABLE reassignments_v
  OWNER TO igops;
GRANT ALL ON TABLE reassignments_v TO igops;
GRANT SELECT ON TABLE reassignments_v TO ig_report_role;