DROP VIEW IF EXISTS permission_assignments_v;
DROP VIEW IF EXISTS assignment_attribute_def_v;
DROP VIEW IF EXISTS auth_user_resource_v;

CREATE VIEW permission_assignments_v AS
WITH recent_collections AS (
    SELECT 
        MAX(id) OVER  (PARTITION BY data_collector ORDER BY data_collector)
    FROM data_collection
        )

SELECT
    usr.display_name AS user_display_name,
    usr.unique_user_id,
    perm.name AS permission,
    perm.unique_permission_id,
    perm.description AS permission_description,
    perm.risk AS permission_risk,
    perm.type AS permission_type,
    ds.name AS application,
    ds.unique_id AS unique_application_id,
    roles.role_names AS role_name,
    CASE
        WHEN ur.is_authorized = true THEN true
        ELSE false
    END AS is_authorized,
    req.display_name AS requester_display_name,
    req.unique_user_id AS requester_unique_id,
    app.display_name AS approver_display_name,
    app.unique_user_id AS approver_unique_id,
    rri.reason AS request_item_reason,
    ri.reason AS request_reason,
    rrap.finish_comment AS approver_comment,
    cph.start_time,
    cph.end_time,
    cph.revocable,
    cph.value,
    cph.generic1,
    cph.generic2,
    cph.generic3,
    cph.generic4,
    cph.generic5,
    cph.generic6
FROM ( SELECT *
            FROM spermission
            WHERE (spermission.exclude_from_catalog = false OR spermission.exclude_from_catalog IS NULL) AND spermission.effective = true) perm
    LEFT JOIN data_source ds ON ds.id = perm.application
    JOIN ( SELECT DISTINCT suser_account_permission.suser,
            suser_account_permission.spermission
            FROM suser_account_permission) sap ON perm.id = sap.spermission
    JOIN suser usr ON usr.id = sap.suser AND usr.effective = true
    LEFT JOIN user_resource ur ON ur.unique_user_id::text = usr.unique_user_id::text AND ur.unique_resource_id::text = perm.unique_permission_id::text AND ur.deleted = false
    LEFT JOIN resource_request_item rri ON rri.unique_item_id::text = perm.unique_permission_id::text AND rri.unique_recipient_id::text = usr.unique_user_id::text AND rri.item_approval_status::text = 'APPROVED'::text AND rri.add_item = true AND rri.last_status_update_time = (( SELECT max(rri2.last_status_update_time) AS max
            FROM resource_request_item rri2
            WHERE rri2.unique_item_id::text = rri.unique_item_id::text AND rri2.unique_recipient_id::text = rri.unique_recipient_id::text))
    LEFT JOIN resource_request ri ON ri.id = rri.resource_request_id
    LEFT JOIN resource_request_approval_step rrap ON rrap.resource_request_id = ri.id
    LEFT JOIN suser req ON ri.unique_requester_id::text = req.unique_user_id::text AND req.effective = true AND req.deleted = false
    LEFT JOIN suser app ON rri.status_changed_by::text = app.unique_user_id::text AND app.effective = true AND app.deleted = false
    LEFT JOIN (
            SELECT
                STRING_AGG(rp.name,' | ') AS role_names,
                su.unique_user_id,
                rperm.unique_permission_id
            FROM role_user ru
                JOIN role_permission rperm ON rperm.policy = ru.policy
                JOIN role_policy rp ON ru.policy = rp.id AND rp.state = 'ACTIVE' AND rp.deleted = false
                JOIN suser su ON su.id = ru.suser
            WHERE ru.deleted = false
            GROUP BY su.unique_user_id, rperm.unique_permission_id
             ) roles ON roles.unique_user_id = usr.unique_user_id AND roles.unique_permission_id = perm.unique_permission_id
    LEFT JOIN cpermission_holder cph ON cph.holder_link = usr.dn AND cph.unique_permission_id = perm.unique_permission_id AND (cph.data_collection IN (SELECT * FROM recent_collections));

CREATE VIEW assignment_attribute_def_v AS 
 SELECT DISTINCT attribute_definition.id,
    attribute_definition.allow_static_mapping,
    attribute_definition.attribute_data_type,
    attribute_definition.attribute_key,
    attribute_definition.attribute_state,
    attribute_definition.attribute_type,
    attribute_definition.curatable,
    attribute_definition.entity_type,
    attribute_definition.length,
    attribute_definition.listable,
    attribute_definition.multi_valued,
    attribute_definition.quick_info,
    attribute_definition.quick_searchable,
    attribute_definition.required,
    attribute_definition.searchable,
    attribute_definition.sortable,
    attribute_definition.truncatable,
    attribute_definition.type_ahead_searchable,
    attribute_definition.localized_display_name,
    llm.label_map AS attribute_name,
    attribute_definition.joinable,
    attribute_definition.target_entity,
    attribute_definition.enum_type,
    attribute_definition.composition,
    attribute_definition.excluded_from_curation,
    attribute_definition.excluded_from_search,
    attribute_definition.advanced_searchable,
    attribute_definition.review_criteria,
    attribute_definition.business_role_criteria,
    attribute_definition.excluded_from_quick_search,
    attribute_definition.excluded_from_sorting
   FROM attribute_definition
     LEFT JOIN localized_label_map llm ON llm.localized_label = attribute_definition.localized_display_name
  WHERE attribute_definition.attribute_key::text ~~ 'generic%'::text AND attribute_definition.entity_type::text = 'PERMISSION_TO_HOLDERS'::text AND llm.label_map_key::text = 'en'::text
  ORDER BY attribute_definition.id;

CREATE VIEW auth_user_resource_v AS 
SELECT
    br.name AS brole,
    br.unique_id AS unique_business_role_id,
    su.display_name AS user_name,
    su.unique_user_id,
    sp.name AS permission_name,
    sp.unique_permission_id,
    rp.name AS role_name,
    rp.unique_policy_id,
    ds.name AS application,
    ds.unique_id AS unique_application_id,
    aur.start_date,
    aur.end_date,
    aur.in_validity_period,
    aur.in_grace_period,
    aur.indirect_authorization,
    aur.auto_grant,
    aur.auto_revoke,
    aur.create_date,
    aur.deleted,
    aur.delete_date
FROM auth_user_resource aur
    JOIN user_resource ur ON aur.user_resource_id = ur.id
    JOIN business_role br ON aur.unique_business_role_id = br.unique_id AND br.approval_state = 'PUBLISHED'
    JOIN suser su ON su.unique_user_id = ur.unique_user_id AND su.effective = true
    LEFT JOIN spermission sp ON sp.unique_permission_id = ur.unique_resource_id AND sp.effective = true
    LEFT JOIN role_policy rp ON rp.unique_policy_id = ur.unique_resource_id AND rp.deleted = false
    LEFT JOIN data_source ds ON (ds.id::text = ur.unique_resource_id OR ds.id = sp.application) AND ds.deleted = false;

ALTER TABLE permission_assignments_v
  OWNER TO igops;
GRANT ALL ON TABLE permission_assignments_v TO igops;
GRANT SELECT ON TABLE permission_assignments_v TO ig_report_role;


ALTER TABLE assignment_attribute_def_v
  OWNER TO igops;
GRANT ALL ON TABLE assignment_attribute_def_v TO igops;
GRANT SELECT ON TABLE assignment_attribute_def_v TO ig_report_role;

ALTER TABLE auth_user_resource_v
  OWNER TO igops;
GRANT ALL ON TABLE auth_user_resource_v TO igops;
GRANT SELECT ON TABLE auth_user_resource_v TO ig_report_role;